/* 
 *gpe-phonepanel
 *
 *Status panel for GPE Phone Edition.
 *
 *(c) kernel concepts 2006
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>

#include <libintl.h>

#include <glib.h>
#include <dbus/dbus-glib-lowlevel.h>
#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-bindings.h>

#define _(x) gettext(x)

typedef struct
{
  gchar *interface;
  gchar *method;
  void (*callback) ();
}
t_listener;

static GList *listeners = NULL;
static DBusConnection *bus = NULL;

static DBusHandlerResult
msg_filter (DBusConnection *connection, DBusMessage *message, void *user_data)
{
  GMainLoop *mainloop = (GMainLoop *) user_data;
  GList *iter;

  for (iter = listeners; iter; iter = iter->next)
  {
    t_listener *li = (t_listener *) iter->data;

    if (dbus_message_is_signal (message, li->interface, li->method))
    {
      li->callback (message);
      return DBUS_HANDLER_RESULT_HANDLED;
    }
    else if (dbus_message_is_method_call (message, li->interface, li->method))
    {
      li->callback (message);
      return DBUS_HANDLER_RESULT_HANDLED;
    }
  }

  /* A signal from the bus saying we are about to be disconnected */
  if (dbus_message_is_signal (message, DBUS_INTERFACE_LOCAL, "Disconnected"))
  {
    g_main_loop_quit (mainloop);
    return DBUS_HANDLER_RESULT_HANDLED;
  }

  return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
}

void
dbus_add_msg_listener (const gchar *interface, const gchar *method,
                       void (*callback) ())
{
  //fix me, using hash table may be better.

  t_listener *new_listener = g_malloc0 (sizeof (t_listener));
  gchar *filterstr =
    g_strdup_printf ("type='signal',interface='%s'", interface);

  new_listener->interface = g_strdup (interface);
  new_listener->method = g_strdup (method);
  new_listener->callback = callback;

  listeners = g_list_append (listeners, (gpointer) new_listener);

  dbus_bus_add_match (bus, filterstr, NULL);

}

void
dbus_add_msg_listener_c (const gchar *interface, const gchar *method,
                         void (*callback) ())
{
  //fix me, using hash table may be better.

  t_listener *new_listener = g_malloc0 (sizeof (t_listener));
  gchar *filterstr =
    g_strdup_printf ("type='method_call',interface='%s'", interface);

  new_listener->interface = g_strdup (interface);
  new_listener->method = g_strdup (method);
  new_listener->callback = callback;

  listeners = g_list_append (listeners, (gpointer) new_listener);

  dbus_bus_add_match (bus, filterstr, NULL);

}

DBusConnection *
dbus_init_im (void)
{
  DBusError error;
  GMainLoop *mainloop;

  mainloop = g_main_loop_new (NULL, TRUE);
  dbus_error_init (&error);

  bus = dbus_bus_get (DBUS_BUS_SESSION, &error);
  if (!bus)
  {
    g_warning ("Failed to connect to the D-BUS daemon: %s", error.message);
    dbus_error_free (&error);
    return NULL;
  }

  dbus_connection_setup_with_g_main (bus, NULL);

  /* we do not request a namespace to support multiple listeners to these signals */

  dbus_connection_add_filter (bus, msg_filter, mainloop, NULL);
  g_main_loop_unref (mainloop);
  return bus;
}

