/*  libim - Library of Instant Messaging and Presence enabler
 * im.c - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 * This software and associated documentation files (the "Software") 
 * are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 * All Rights Reserved. 
 *
 * A copyright license is hereby granted for redistribution and use of 
 * the Software in source and binary forms, with or without modification, 
 * provided that the following conditions are met: 
 * - Redistributions of source code must retain the above copyright notice, 
 * this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 * notice, this copyright license and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution. 
 * - Neither the name of LiPS  nor the names of its Members may be used 
 * to endorse or promote products derived from the Software without 
 * specific prior written permission. 
 *
 * A patent license for any Necessary Claims owned by Members of LiPS Forum 
 * to make, have made, use, import, offer to sell, lease and sell or otherwise 
 * distribute any implementation compliant with the any specification adopted 
 * by the LiPS Forumcan be obtained from the respective Members on reasonable 
 * and non-discriminatory terms and conditions and under reciprocity, as 
 * regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 * "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 * AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 * ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 */


/**
 *@file API implememt of the Instance Message Interface - IM
 *
 *@version 0.1
 */

#include "im.h"
#include "imevent.h"
#include "imevent_imp.h"
#include "im_dbus.h"
#include "im_client_dbus.h"

void
im_version (int *major, int *minor)
{
  *major = IM_VERSION_MAJOR;
  *minor = IM_VERSION_MINOR;
};

im_err_t
im_open_session (im_sid_t *pSid,
                 im_protocol_t protocol,
                 char *version, char *server, char *port, char *user)
{
  DBusError error;
  DBusMessage *message = NULL;
  DBusMessage *reply = NULL;
  char *msg = IM_DBUS_MSG_OPEN_SESSION;
  const char *uname = dbus_bus_get_unique_name (bus_conn);
  im_err_t rt = IM_ERROR_NONE;

  message = dbus_message_new_method_call (IM_DBUS_SERVICE,
                                          IM_DBUS_OBJECT,
                                          IM_DBUS_INTERFACE, msg);

  dbus_message_append_args (message,
                            DBUS_TYPE_STRING, &uname,
                            DBUS_TYPE_INT32, &protocol,
                            DBUS_TYPE_STRING, &server,
                            DBUS_TYPE_STRING, &port,
                            DBUS_TYPE_STRING, &version,
                            DBUS_TYPE_STRING, &user, DBUS_TYPE_INVALID);

  dbus_error_init (&error);

  reply = dbus_connection_send_with_reply_and_block (bus_conn,
                                                     message,
                                                     IM_DBUS_CALL_DELAY,
                                                     &error);

  dbus_message_unref (message);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to send message '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_message_get_args (reply, &error,
                         DBUS_TYPE_UINT32, pSid,
                         DBUS_TYPE_INT32, &rt, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to get reply '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    *pSid = 0;
    return IM_ERROR_DBUS;
  }

  dbus_error_free (&error);
  dbus_message_unref (reply);
  if (rt != IM_ERROR_NONE)
    *pSid = 0;
  return rt;
};

im_err_t
im_close_session (im_sid_t sid)
{
  DBusError error;
  DBusMessage *message = NULL;
  DBusMessage *reply = NULL;
  char *msg = IM_DBUS_MSG_CLOSE_SESSION;

  message = dbus_message_new_method_call (IM_DBUS_SERVICE,
                                          IM_DBUS_OBJECT,
                                          IM_DBUS_INTERFACE, msg);

  dbus_message_append_args (message, DBUS_TYPE_UINT32, &sid, DBUS_TYPE_INVALID);

  dbus_error_init (&error);

  reply = dbus_connection_send_with_reply_and_block (bus_conn,
                                                     message,
                                                     IM_DBUS_CALL_DELAY,
                                                     &error);

  dbus_message_unref (message);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to send message '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  im_err_t im_err = IM_ERROR_NONE;

  dbus_message_get_args (reply, &error,
                         DBUS_TYPE_INT32, &im_err, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to get reply '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  g_warning ("im_err = %d \n", im_err);

  dbus_error_free (&error);
  dbus_message_unref (reply);
  return IM_ERROR_NONE;
};

im_err_t
im_init ()
{
  if (im_evt_src == NULL)
  {
    if (evt_init ((evt_get_t) im_evt_get, NULL, &im_evt_src) != EVT_ERROR_NONE)
      return IM_ERROR_GENERAL;
    return im_client_dbus_init ();
  }
  return IM_ERROR_NONE;
}

// im_err_t im_start()
// {
//   return IM_ERROR_NONE;
// }
//
// im_err_t im_stop()
// {
//   return IM_ERROR_NONE;
// }

im_err_t
im_finalize ()
{
  if (im_evt_src != NULL)
  {
    evt_finalize (im_evt_src);
    im_evt_src = NULL;
    return im_client_dbus_free ();
  }
  return IM_ERROR_NONE;
}

