/*  libim - Library of Instant Messaging and Presence enabler
 * im_client_dbus.c - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 * This software and associated documentation files (the "Software") 
 * are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 * All Rights Reserved. 
 *
 * A copyright license is hereby granted for redistribution and use of 
 * the Software in source and binary forms, with or without modification, 
 * provided that the following conditions are met: 
 * - Redistributions of source code must retain the above copyright notice, 
 * this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 * notice, this copyright license and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution. 
 * - Neither the name of LiPS  nor the names of its Members may be used 
 * to endorse or promote products derived from the Software without 
 * specific prior written permission. 
 *
 * A patent license for any Necessary Claims owned by Members of LiPS Forum 
 * to make, have made, use, import, offer to sell, lease and sell or otherwise 
 * distribute any implementation compliant with the any specification adopted 
 * by the LiPS Forumcan be obtained from the respective Members on reasonable 
 * and non-discriminatory terms and conditions and under reciprocity, as 
 * regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 * "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 * AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 * ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 */


/**
 *@file API implememt of the Instance Message Interface - IM DBus definition
 *
 *@version 0.1
 */

#include "im_client_dbus.h"
#include "im_dbus.h"
#include "im.h"
#include "imevent_imp.h"
#include "immsg_imp.h"
#include "imcontact_imp.h"
#include "impresence_imp.h"
#include <stdio.h>
#include <string.h>

DBusConnection *bus_conn;

/**
 *@brief pass dbus message for building an event
 *
 *@note for inner usage only.
 */

evt_src_t *im_evt_src = NULL;

im_err_t
im_roster (DBusMessageIter *iter, evt_t **evt)
{
  dbus_uint32_t tdata;
  char *cdata;
  im_roster_t *im_roster = NULL;

  im_roster_new_with_sid (&im_roster, (*evt)->sid);
  im_roster_item_t *im_roster_item = NULL;

  dbus_message_iter_next (iter);
  while (dbus_message_iter_get_arg_type (iter) != DBUS_TYPE_INVALID)
  {
    im_roster_item = g_new0 (im_roster_item_t, 1);

    dbus_message_iter_get_basic (iter, &cdata);
    im_roster_item->contactor = g_strdup (cdata);
    g_print ("=============|-evt_data: contactor = %s\n", cdata);

    dbus_message_iter_next (iter);
    dbus_message_iter_get_basic (iter, &cdata);
    im_roster_item->nickname = g_strdup (cdata);
    g_print ("=============|-evt_data: nickname = %s\n", cdata);

    dbus_message_iter_next (iter);
    dbus_message_iter_get_basic (iter, &tdata);
    im_roster_item->subs_state = tdata;
    g_print ("=============|-evt_data: subs_state = %d\n", tdata);

    dbus_message_iter_next (iter);
    dbus_message_iter_get_basic (iter, &cdata);
    im_roster_item->group = g_strdup (cdata);
    g_print ("=============|-evt_data: group = %s\n", cdata);

    im_roster_prepend (im_roster, im_roster_item);
    dbus_message_iter_next (iter);
  }

  (*evt)->evt_data = im_roster;
  return IM_ERROR_NONE;
}

im_err_t
im_presence (DBusMessageIter *iter, evt_t **evt)
{
  dbus_uint32_t tdata;
  char *cdata;
  im_presence_t *presence;

  char *cuser;
  char *desc;
  im_presence_state_t state;
  im_protocol_t prtl;
  int current_type;

  dbus_message_iter_next (iter);
  if ((current_type =
       dbus_message_iter_get_arg_type (iter)) == DBUS_TYPE_INVALID)
    return IM_ERROR_DBUS;
  dbus_message_iter_get_basic (iter, &cdata);
  cuser = g_strdup (cdata);
  g_print ("=============|-evt_data: from = %s\n", cuser);

  dbus_message_iter_next (iter);
  if ((current_type =
       dbus_message_iter_get_arg_type (iter)) == DBUS_TYPE_INVALID)
  {
    g_free (cuser);
    return IM_ERROR_DBUS;
  }
  dbus_message_iter_get_basic (iter, &tdata);
  state = tdata;
  g_print ("=============|-evt_data: state = %d\n", tdata);

  dbus_message_iter_next (iter);
  if ((current_type =
       dbus_message_iter_get_arg_type (iter)) == DBUS_TYPE_INVALID)
  {
    g_free (cuser);
    return IM_ERROR_DBUS;
  }
  dbus_message_iter_get_basic (iter, &cdata);
  desc = g_strdup (cdata);
  g_print ("=============|-evt_data: desc = %s\n", cdata);

  dbus_message_iter_next (iter);
  if ((current_type =
       dbus_message_iter_get_arg_type (iter)) == DBUS_TYPE_INVALID)
  {
    g_free (cuser);
    g_free (desc);
    return IM_ERROR_DBUS;
  }
  dbus_message_iter_get_basic (iter, &tdata);
  prtl = tdata;
  g_print ("=============|-evt_data: protocol = %d\n", tdata);

  im_presence_new (prtl, NULL, &presence);
  im_presence_set_from (presence, cuser);
  im_presence_set_state (presence, state);
  im_presence_set_desc (presence, desc);
  (*evt)->evt_data = presence;
  g_free (cuser);
  g_free (desc);
  return IM_ERROR_NONE;
}


im_err_t
im_evt_message (DBusMessageIter *iter, evt_t **evt)
{
  dbus_uint32_t tdata;
  char *cdata;
  void *body = NULL;
  int body_size = 0;
  char *encode = NULL;
  im_msg_t *msg;
  char *user;
  im_msg_id_t msg_id;
  time_t msg_time;
  im_msg_type_t msg_type;
  im_msg_ctype_t msg_ctype;
  im_msg_state_t msg_state;
  int current_type;

  dbus_message_iter_next (iter);
  if ((current_type =
       dbus_message_iter_get_arg_type (iter)) == DBUS_TYPE_INVALID)
    return IM_ERROR_DBUS;
  dbus_message_iter_get_basic (iter, &cdata);
  user = g_strdup (cdata);
  g_print ("=============|-evt_data: user = %s\n", cdata);

  dbus_message_iter_next (iter);
  if ((current_type = dbus_message_iter_get_arg_type (iter)) != DBUS_TYPE_ARRAY
      || dbus_message_iter_get_element_type (iter) != DBUS_TYPE_BYTE)
  {
    g_free (user);
    return IM_ERROR_DBUS;
  }
  DBusMessageIter sub;

  dbus_message_iter_recurse (iter, &sub);
  body_size = dbus_message_iter_get_array_len (&sub);
  dbus_message_iter_get_fixed_array (&sub, &cdata, &body_size);
  body = g_memdup (cdata, body_size);
  g_print ("=============|-evt_data: body[%d] = %s\n", body_size, cdata);

  dbus_message_iter_next (iter);
  if ((current_type =
       dbus_message_iter_get_arg_type (iter)) == DBUS_TYPE_INVALID)
  {
    g_free (user);
    g_free (body);
    return IM_ERROR_DBUS;
  }
  dbus_message_iter_get_basic (iter, &cdata);
  encode = g_strdup (cdata);
  g_print ("=============|-evt_data: encode = %s\n", cdata);

  dbus_message_iter_next (iter);
  if ((current_type =
       dbus_message_iter_get_arg_type (iter)) == DBUS_TYPE_INVALID)
  {
    g_free (user);
    g_free (body);
    g_free (encode);
    return IM_ERROR_DBUS;
  }
  dbus_message_iter_get_basic (iter, &tdata);
  msg_id = tdata;
  g_print ("=============|-evt_data: msg_id = %d\n", tdata);

  dbus_message_iter_next (iter);
  if ((current_type =
       dbus_message_iter_get_arg_type (iter)) == DBUS_TYPE_INVALID)
  {
    g_free (user);
    g_free (body);
    g_free (encode);
    return IM_ERROR_DBUS;
  }
  dbus_message_iter_get_basic (iter, &tdata);
  msg_time = tdata;
  g_print ("=============|-evt_data: msg_time = %d\n", tdata);

  dbus_message_iter_next (iter);
  if ((current_type =
       dbus_message_iter_get_arg_type (iter)) == DBUS_TYPE_INVALID)
  {
    g_free (user);
    g_free (body);
    g_free (encode);
    return IM_ERROR_DBUS;
  }
  dbus_message_iter_get_basic (iter, &tdata);
  msg_type = tdata;
  g_print ("=============|-evt_data: msg_type = %d\n", tdata);

  dbus_message_iter_next (iter);
  if ((current_type =
       dbus_message_iter_get_arg_type (iter)) == DBUS_TYPE_INVALID)
  {
    g_free (user);
    g_free (body);
    g_free (encode);
    return IM_ERROR_DBUS;
  }
  dbus_message_iter_get_basic (iter, &tdata);
  msg_ctype = tdata;
  g_print ("=============|-evt_data: msg_ctype = %d\n", tdata);

  dbus_message_iter_next (iter);
  if ((current_type =
       dbus_message_iter_get_arg_type (iter)) == DBUS_TYPE_INVALID)
  {
    g_free (user);
    g_free (body);
    g_free (encode);
    return IM_ERROR_DBUS;
  }
  dbus_message_iter_get_basic (iter, &tdata);
  msg_state = tdata;
  g_print ("=============|-evt_data: msg_state = %d\n", tdata);

  im_msg_new ((*evt)->sid, msg_type, msg_ctype, &msg);
  im_msg_set_id (msg, msg_id);
  im_msg_set_time (msg, msg_time);
  im_msg_set_state (msg, msg_state);
  im_msg_set_from (msg, user);
  im_msg_set_body (msg, body, body_size, encode);
  (*evt)->evt_data = msg;
  g_free (user);
  g_free (body);
  g_free (encode);
  return IM_ERROR_NONE;
}

im_err_t
im_evt_subscribe (DBusMessageIter *iter, evt_t **evt)
{
  dbus_bool_t bdata;
  char *cdata;
  int current_type;
  im_subs_msg_t *subs_msg = g_new0 (im_subs_msg_t, 1);

  dbus_message_iter_next (iter);
  if ((current_type =
       dbus_message_iter_get_arg_type (iter)) == DBUS_TYPE_INVALID)
    return IM_ERROR_DBUS;
  dbus_message_iter_get_basic (iter, &cdata);
  subs_msg->contactor = g_strdup (cdata);
  g_print ("=============|-evt_data: contactor = %s\n", cdata);

  dbus_message_iter_next (iter);
  if ((current_type =
       dbus_message_iter_get_arg_type (iter)) == DBUS_TYPE_INVALID)
  {
    im_subs_msg_free (subs_msg);
    return IM_ERROR_DBUS;
  }
  dbus_message_iter_get_basic (iter, &cdata);
  subs_msg->msg = g_strdup (cdata);
  g_print ("=============|-evt_data: msg = %s\n", cdata);

  dbus_message_iter_next (iter);
  if ((current_type =
       dbus_message_iter_get_arg_type (iter)) == DBUS_TYPE_INVALID)
  {
    im_subs_msg_free (subs_msg);
    return IM_ERROR_DBUS;
  }
  dbus_message_iter_get_basic (iter, &bdata);
  subs_msg->subscribe = bdata;
  g_print ("=============|-evt_data: subscribe = %d\n", bdata);

  (*evt)->evt_data = subs_msg;
  return IM_ERROR_NONE;
}

im_err_t
im_evt_acceptance (DBusMessageIter *iter, evt_t **evt)
{
  dbus_uint32_t tdata;
  char *cdata;
  int current_type;
  im_accept_msg_t *accept_msg = g_new0 (im_accept_msg_t, 1);

  dbus_message_iter_next (iter);
  if ((current_type =
       dbus_message_iter_get_arg_type (iter)) == DBUS_TYPE_INVALID)
    return IM_ERROR_DBUS;
  dbus_message_iter_get_basic (iter, &cdata);
  accept_msg->contactor = g_strdup (cdata);
  g_print ("=============|-evt_data: contactor = %s\n", cdata);

  dbus_message_iter_next (iter);
  if ((current_type =
       dbus_message_iter_get_arg_type (iter)) == DBUS_TYPE_INVALID)
  {
    (*evt)->evt_data = accept_msg;
    g_free (accept_msg->contactor);
    g_free (accept_msg);
    return IM_ERROR_DBUS;
  }
  dbus_message_iter_get_basic (iter, &cdata);
  accept_msg->msg = g_strdup (cdata);
  g_print ("=============|-evt_data: msg = %s\n", cdata);

  dbus_message_iter_next (iter);
  if ((current_type =
       dbus_message_iter_get_arg_type (iter)) == DBUS_TYPE_INVALID)
  {
    (*evt)->evt_data = accept_msg;
    g_free (accept_msg->contactor);
    g_free (accept_msg->msg);
    g_free (accept_msg);
    return IM_ERROR_DBUS;
  }
  dbus_message_iter_get_basic (iter, &tdata);
  accept_msg->accept = tdata;
  g_print ("=============|-evt_data: accept = %d\n", tdata);

  (*evt)->evt_data = accept_msg;
  return IM_ERROR_NONE;
}

im_err_t
im_evt_buddy_op (DBusMessageIter *iter, evt_t **evt)
{
  dbus_uint32_t tdata;
  char *cdata;
  im_buddy_op_t *op = NULL;
  im_roster_item_t *buddy = (im_roster_item_t *) g_new0 (im_roster_item_t, 1);

  dbus_message_iter_next (iter);
  if (dbus_message_iter_get_arg_type (iter) != DBUS_TYPE_STRING)
    goto FAIL;
  dbus_message_iter_get_basic (iter, &cdata);
  buddy->contactor = g_strdup (cdata);
  g_print ("=============|-evt_data: contactor = %s\n", cdata);

  dbus_message_iter_next (iter);
  if (dbus_message_iter_get_arg_type (iter) != DBUS_TYPE_STRING)
    goto FAIL;
  dbus_message_iter_get_basic (iter, &cdata);
  buddy->nickname = g_strdup (cdata);
  g_print ("=============|-evt_data: nickname = %s\n", cdata);

  dbus_message_iter_next (iter);
  if (dbus_message_iter_get_arg_type (iter) != DBUS_TYPE_INT32)
    goto FAIL;
  dbus_message_iter_get_basic (iter, &tdata);
  buddy->subs_state = tdata;
  g_print ("=============|-evt_data: subs_state = %d\n", tdata);

  dbus_message_iter_next (iter);
  if (dbus_message_iter_get_arg_type (iter) != DBUS_TYPE_STRING)
    goto FAIL;
  dbus_message_iter_get_basic (iter, &cdata);
  buddy->group = g_strdup (cdata);
  g_print ("=============|-evt_data: group = %s\n", cdata);

  dbus_message_iter_next (iter);
  if (dbus_message_iter_get_arg_type (iter) != DBUS_TYPE_INT32)
    goto FAIL;
  dbus_message_iter_get_basic (iter, &tdata);
  g_print ("=============|-evt_data: operation = %s\n",
           im_buddy_operation_2_str (tdata));

  im_buddy_op_new (buddy, tdata, &op);
  (*evt)->evt_data = op;
  return IM_ERROR_NONE;
FAIL:
  im_roster_item_free (buddy);
  return IM_ERROR_DBUS;
}

im_err_t
im_evt_error (DBusMessageIter *iter, evt_t **evt)
{
  dbus_uint32_t tdata;
  char *cdata;
  int current_type;
  im_network_error_t *error = g_new0 (im_network_error_t, 1);

  dbus_message_iter_next (iter);
  if ((current_type =
       dbus_message_iter_get_arg_type (iter)) == DBUS_TYPE_INVALID)
    return IM_ERROR_DBUS;
  dbus_message_iter_get_basic (iter, &tdata);
  error->err_type = tdata;
  g_print ("=============|-evt_data: err_type = %d\n", tdata);

  dbus_message_iter_next (iter);
  if ((current_type =
       dbus_message_iter_get_arg_type (iter)) == DBUS_TYPE_INVALID)
  {
    (*evt)->evt_data = error;
    g_free (error);
    return IM_ERROR_DBUS;
  }
  dbus_message_iter_get_basic (iter, &cdata);
  error->msg = g_strdup (cdata);
  g_print ("=============|-evt_data: msg = %s\n", cdata);

  (*evt)->evt_data = error;
  return IM_ERROR_NONE;
}

evt_err_t
im_client_incoming_evt_handle (DBusMessageIter *iter, evt_t **evt)
{
  evt_err_t rt =
    evt_get_handler (im_evt_src, (*evt)->sid, (*evt)->evt_type,
                     &(*evt)->evt_handler);
  if (rt != EVT_ERROR_NONE)
    return rt;
  switch ((*evt)->evt_type)
  {
    case IM_EVT_ROSTER:
      rt = im_roster (iter, evt);
      break;
    case IM_EVT_PRESENCE:
      rt = im_presence (iter, evt);
      break;
    case IM_EVT_MESSAGE:
      rt = im_evt_message (iter, evt);
      break;
    case IM_EVT_SUBSCRIBE:
      rt = im_evt_subscribe (iter, evt);
      break;
    case IM_EVT_ACCEPTANCE:
      rt = im_evt_acceptance (iter, evt);
      break;
    case IM_EVT_BUDDY:
      rt = im_evt_buddy_op (iter, evt);
      break;
    case IM_EVT_ERROR:
      rt = im_evt_error (iter, evt);
      break;
    default:
      break;
  }
  return rt;
}

evt_err_t
im_client_incoming_asyn_handle (DBusMessageIter *iter, evt_t **evt)
{
  evt_err_t rt = EVT_ERROR_NONE;
  dbus_uint32_t tdata;

  dbus_message_iter_next (iter);
  (*evt)->async_data = g_new0 (evt_async_t, 1);
  if (dbus_message_iter_get_arg_type (iter) != DBUS_TYPE_UINT32)
    return IM_ERROR_DBUS;
  dbus_message_iter_get_basic (iter, &tdata);
  (*evt)->async_data->rid = tdata;
  g_print ("=============|-async_data->rid = %d\n", tdata);
  async_evt_get_handler (im_evt_src, (*evt)->sid, (*evt)->async_data->rid,
                         &(*evt)->evt_handler);

  dbus_message_iter_next (iter);
  if (dbus_message_iter_get_arg_type (iter) != DBUS_TYPE_INT32)
    return IM_ERROR_DBUS;
  dbus_message_iter_get_basic (iter, &tdata);
  (*evt)->async_data->err_code = tdata;
  g_print ("=============|-async_data->err_code = %d\n", tdata);

  dbus_message_iter_next (iter);
  if (dbus_message_iter_get_arg_type (iter) != DBUS_TYPE_INT32)
    return IM_ERROR_DBUS;
  dbus_message_iter_get_basic (iter, &tdata);
  (*evt)->async_data->state = tdata;
  g_print ("=============|-async_data->state = %d\n", tdata);

  switch ((*evt)->evt_type)
  {
    case IM_REQ_PRESENCE:
    case IM_REQ_PRESENCE_PROBE:
      rt = im_presence (iter, evt);
      break;
    default:
      break;
  }
  g_print ("in %s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
  return rt;
}

evt_err_t
im_evt_get (evt_src_t *evt_src, sid_t sid, evt_t **evt, int timeout)
{
  return EVT_ERROR_NONE;
}

evt_err_t
_im_evt_get (evt_src_t *evt_src,
             sid_t sid, evt_t **evt, int timeout, DBusMessageIter *pIter)
{
  dbus_int32_t tdata;
  evt_type_t evt_type = -1;
  evt_data_free_t evt_data_free = NULL;
  int asynch = 0;
  evt_err_t rt = EVT_ERROR_NONE;

  dbus_message_iter_next (pIter);
  if (dbus_message_iter_get_arg_type (pIter) != DBUS_TYPE_INT32)
    return IM_ERROR_DBUS;
  dbus_message_iter_get_basic (pIter, &tdata);  //get event type
  evt_type = tdata;
  switch (evt_type)
  {
    case IM_EVT_ROSTER:
      evt_data_free = (evt_data_free_t) im_roster_free;
      asynch = 0;
      break;
    case IM_EVT_PRESENCE:
      evt_data_free = (evt_data_free_t) im_presence_free;
      asynch = 0;
      break;
    case IM_EVT_MESSAGE:
      evt_data_free = (evt_data_free_t) im_msg_free;
      asynch = 0;
      break;
    case IM_EVT_SUBSCRIBE:
      evt_data_free = (evt_data_free_t) im_subs_msg_free;
      asynch = 0;
      break;
    case IM_EVT_ACCEPTANCE:
      evt_data_free = (evt_data_free_t) im_accept_msg_free;
      asynch = 0;
      break;
    case IM_EVT_ERROR:
      evt_data_free = (evt_data_free_t) im_network_error_free;
      asynch = 0;
      break;
    case IM_EVT_LOGIN:
      asynch = 0;
      break;
    case IM_EVT_LOGOUT:
      asynch = 0;
      break;
    case IM_EVT_BUDDY:
      evt_data_free = (evt_data_free_t) im_buddy_op_free_all;
      asynch = 0;
      break;
    case IM_REQ_LOGIN:
      asynch = 1;
      break;
    case IM_REQ_LOGOUT:
      asynch = 1;
      break;
    case IM_REQ_ROSTER:
      asynch = 1;
      break;
    case IM_REQ_SUBSCRIBE:
      asynch = 1;
      break;
    case IM_REQ_ACCEPTANCE:
      asynch = 1;
      break;
    case IM_REQ_PRESENCE:
      evt_data_free = (evt_data_free_t) im_presence_free;
      asynch = 1;
      break;
    case IM_REQ_PRESENCE_PROBE:
      evt_data_free = (evt_data_free_t) im_presence_free;
      asynch = 1;
      break;
    case IM_REQ_MESSAGE:
      asynch = 1;
      break;
    case IM_REQ_GROUP:
      asynch = 1;
      break;
    case IM_REQ_BUDDY:
      asynch = 1;
      break;
    case IM_REQ_SYNCH:
      asynch = 1;
      break;
    default:
      return IM_ERROR_DBUS;
  }
  g_print ("=============in %s:%s %d\n", __FILE__, __FUNCTION__, __LINE__);
  g_print ("=============|-sid = %d\n", sid);
  g_print ("=============|-evt_type = '%s<%d>'\n", im_evt_type_2_str (evt_type),
           evt_type);
  if (_evt_get (evt_src, sid, evt_type, evt_data_free, evt) != EVT_ERROR_NONE)
    return IM_ERROR_NOMEM;
  if (asynch)
  {
    rt = im_client_incoming_asyn_handle (pIter, evt);
  }
  else
  {
    rt = im_client_incoming_evt_handle (pIter, evt);
  }
  return rt;
};


void
im_client_incoming_evt (DBusMessage *message)
{
  DBusMessageIter iter;

  dbus_message_iter_init (message, &iter);
  if (dbus_message_iter_get_arg_type (&iter) == DBUS_TYPE_UINT32)
  {
    dbus_uint32_t sid;

    dbus_message_iter_get_basic (&iter, &sid);  //get sid
    evt_t *evt = NULL;

    if (_im_evt_get (im_evt_src, sid, &evt, -1, &iter) != EVT_ERROR_NONE)
    {
      g_message ("Error when receiving event: %s:%s %d\n", __FILE__,
                 __FUNCTION__, __LINE__);
      return;
    }
    evt_call (im_evt_src, evt);
    evt_release (im_evt_src, evt);
  }
}

im_err_t
im_client_dbus_start_service ()
{
  dbus_uint32_t result;
  DBusError error;
  im_err_t rt = IM_ERROR_NONE;

  dbus_error_init (&error);
  if (dbus_bus_start_service_by_name (bus_conn,
                                      IM_DBUS_SERVICE, 0, &result,
                                      &error) == FALSE)
  {
    if (dbus_error_is_set (&error))
      g_warning ("Failed to start up service '%s': %s",
                 IM_DBUS_SERVICE, error.message);
    rt = IM_ERROR_DBUS;
  }
  else
  {
    if (result == DBUS_START_REPLY_SUCCESS)
      g_warning ("service '%s' successfully started up", IM_DBUS_SERVICE);
    else if (result == DBUS_START_REPLY_ALREADY_RUNNING)
      g_warning ("service '%s' already running", IM_DBUS_SERVICE);
  }
  dbus_error_free (&error);
  return rt;
};

im_err_t
im_client_dbus_close_conn ()
{
  dbus_connection_close (bus_conn);
  bus_conn = NULL;
  return IM_ERROR_NONE;
};

im_err_t
im_client_dbus_init (void)
{
  im_err_t rt = IM_ERROR_NONE;

  bus_conn = dbus_init_im ();
  if (!bus_conn)
  {
    return IM_ERROR_DBUS;
  }

  if (!dbus_bus_name_has_owner (bus_conn, IM_DBUS_SERVICE, NULL))
  {
    g_warning
      ("No IM_Proxy service is running.\nNow starting the service...\n");
    if ((rt = im_client_dbus_start_service ()) != IM_ERROR_NONE)
    {
      im_client_dbus_close_conn ();
      return rt;
    }
  }

  dbus_add_msg_listener_c (IM_CLIENT_INTERFACE,
                           IM_CLIENT_MSG_EVT_IN, im_client_incoming_evt);

  return rt;
};

im_err_t
im_client_dbus_send (const char *msg)
{
  DBusError error;
  DBusMessage *message = NULL;
  DBusMessage *reply = NULL;

  dbus_error_init (&error);

  message = dbus_message_new_method_call (IM_DBUS_SERVICE,
                                          IM_DBUS_OBJECT,
                                          IM_DBUS_INTERFACE, msg);

  reply = dbus_connection_send_with_reply_and_block (bus_conn,
                                                     message,
                                                     IM_DBUS_CALL_DELAY,
                                                     &error);

  dbus_message_unref (message);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to send message '%s': %s", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }
  dbus_message_unref (reply);
  dbus_error_free (&error);
  return IM_ERROR_NONE;
};

im_err_t
im_client_dbus_free (void)
{
  return im_client_dbus_close_conn ();
};

