/*  libim - Library of Instant Messaging and Presence enabler
 * imconnect.h - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 */


/**
 *@file API definition of the Instance Message Interface - IM Connection
 *
 *@version 0.1
 */

#ifndef __LIPS_IM_CONNECT_H
#define __LIPS_IM_CONNECT_H

#include "imtype.h"
#include "imerror.h"
#include "imuri.h"
#include "imcontact.h"
#include "impresence.h"
#include "immsg.h"
#include "imevent.h"

#define IM_BUDDY_DEFAULT_GROUP "default"

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */

  /**
   *im connection status definition
    */
  typedef enum
  {
    IM_CONNECTION_DISCONNECTED,
    IM_CONNECTION_LOGINING,
    IM_CONNECTION_LOGINED,
    IM_CONNECTION_LOGOUTING
  } im_connection_status_t;

  //typedef struct _im_trans im_trans;

  //typedef struct _im_trans_mgr im_trans_mgr;

  /**
   *im connection definition
    */
  typedef struct _im_connection im_connection;

  typedef struct _im_trans_mgr_t im_trans_mgr_t;

  typedef struct
  {
    char *uri;
    void *msg;
    size_t msg_size;
    im_msg_id_t msg_id;
    time_t msg_time;
    char *encode;
    im_msg_type_t msg_type;
    im_msg_ctype_t msg_ctype;
    im_msg_state_t msg_state;
  }
  im_msg_param_t;

  im_err_t im_conn_login (im_connection *conn, im_sid_t sid, char *passwd,
                          im_rid_t *pRid);

  im_err_t im_conn_login_cancel (im_connection *conn);

  im_err_t im_conn_logout (im_connection *conn, im_sid_t sid, im_rid_t *pRid);

  im_err_t im_conn_change_presence (const im_connection *conn,
                                    const im_sid_t sid, const char *uri,
                                    const im_presence_state_t pstate,
                                    const char *desc, im_rid_t *pRid);

  im_err_t im_conn_presence_probe (const im_connection *conn,
                                   const im_sid_t sid, const char *uri,
                                   im_rid_t *pRid);

  im_err_t im_conn_send_msg (const im_connection *conn, const im_sid_t sid,
                             const char *to, const void *msg,
                             const size_t bsize, const char *encode,
                             const im_msg_type_t mtype,
                             const im_msg_ctype_t mctype, im_rid_t *pRid);

  im_err_t im_conn_subscribe (const im_connection *conn, const im_sid_t sid,
                              const char *to, const char *msg,
                              const bool subscribe, const char *nick,
                              const char *group, im_rid_t *pRid);

  im_err_t im_conn_accept (const im_connection *conn, const im_sid_t sid,
                           const char *to, const char *msg, const bool accept,
                           im_rid_t *pRid);

  im_err_t im_conn_get_rosters (const im_connection *conn, const im_sid_t sid,
                                im_rid_t *pRid);

  im_err_t im_conn_buddy_op (const im_connection *conn, const im_sid_t sid,
                             const char *contactor, const char *nickname,
                             const char *group,
                             const im_buddy_operation_t op_type,
                             im_rid_t *pRid);

  im_err_t im_conn_buddies_synch (const im_connection *conn,
                                  const im_sid_t sid, im_rid_t *pRid);

  im_err_t im_conn_on_login (im_connection *conn, im_err_code_t err_code);

  im_err_t im_conn_on_logout (im_connection *conn);

  im_err_t im_conn_on_presence (const im_connection *conn, const char *uri,
                                const im_presence_state_t pstate,
                                const im_protocol_t prtl, const char *desc);

  im_err_t im_conn_on_message (const im_connection *conn,
                               im_msg_param_t *msg);

  im_err_t im_conn_on_subscribe (const im_connection *conn, const char *from,
                                 const char *msg, const bool subscribe);

  im_err_t im_conn_on_accept (const im_connection *conn, const char *from,
                              const char *msg, const bool accept);

  im_err_t im_conn_on_error (im_connection *conn, const im_err_code_t err_code,
                             const char *msg);

  im_err_t im_conn_on_roster (const im_connection *conn);

  im_err_t im_conn_on_buddy_op (const im_connection *conn, im_buddy_op_t *op);

  im_connection *im_conn_ref (im_connection *conn);

  im_connection *im_conn_unref (im_connection *conn);

  im_trans_mgr_t *im_conn_get_trans_mgr (im_connection *conn);

  im_trans_mgr_t *im_conn_get_pendings (im_connection *conn);

#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif                          /* __LIPS_IM_CONNECT_H */

