/*  libim - Library of Instant Messaging and Presence enabler
 * imconnect_base.h - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 */


/**
 *@file API definition of the Instance Message Interface - IM Connection base
 *
 *@version 0.1
 */

#ifndef __LIPS_IM_CONNECT_BASE_H
#define __LIPS_IM_CONNECT_BASE_H

#include "im.h"
#include "imconnect.h"
#include "imtrans.h"
#include "im_mutex.h"
#include "imcontact_imp.h"

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */

  struct _im_connection
  {
    char *server_addr;          ///< Server address.
    char *server_port;          ///< Server port.
    im_uri_t *login_uri;        ///< Login user uri.
    im_roster_t *roster;        ///< Buddy list.
    im_connection_status_t status;      ///< Connection status.
    im_protocol_t protocol;     ///< Protocol type.
    char *version;              ///< Version.
    uint32_t ref;               ///< Reference counter.
    im_trans_mgr_t *tmgr;       ///< Trans_manager queue.
    im_trans_mgr_t *pendings;   ///< Pending transactions queue.
    im_rec_mutex *mutex;        ///< Mutex.
    bool cancel_flag;           ///< if cancel the login process.
    char *passwd;               ///< Password.

    im_trans_t *login_t;        ///< login transaction.
    im_trans_t *logout_t;       ///< logout transaction.

    void *data;                 ///< protocol special data.

      im_err_t (*free) (im_connection *conn);

      im_err_t (*login) (im_connection *conn, char *passwd);

      im_err_t (*login_cancel) (im_connection *conn);

      im_err_t (*logout) (im_connection *conn);

      im_err_t (*on_login) (im_connection *conn, im_err_code_t err_code);

      im_err_t (*on_logout) (im_connection *conn);

      im_err_t (*on_presence) (const im_connection *conn, const char *uri,
                               const im_presence_state_t pstate,
                               const char *desc);

      im_err_t (*on_message) (const im_connection *conn, im_msg_param_t *msg);

      im_err_t (*on_subscribe) (const im_connection *conn, const char *from,
                                const char *msg, const bool subscribe);

      im_err_t (*on_accept) (const im_connection *conn, const char *from,
                             const char *msg, const bool accept);

      im_err_t (*on_error) (im_connection *conn, const im_err_code_t err_code,
                            const char *msg);

      im_err_t (*on_roster) (const im_connection *conn);

      im_err_t (*on_buddy_op) (const im_connection *conn, im_buddy_op_t *op);
  };

  im_err_t im_conn_init_base (im_connection *conn, char *ver);

  im_err_t im_conn_free_base (im_connection *conn);

#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif                          /* __LIPS_IM_CONNECT_BASE_H */

