/*  libim - Library of Instant Messaging and Presence enabler
 * imcontact.c - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 * This software and associated documentation files (the "Software") 
 * are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 * All Rights Reserved. 
 *
 * A copyright license is hereby granted for redistribution and use of 
 * the Software in source and binary forms, with or without modification, 
 * provided that the following conditions are met: 
 * - Redistributions of source code must retain the above copyright notice, 
 * this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 * notice, this copyright license and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution. 
 * - Neither the name of LiPS  nor the names of its Members may be used 
 * to endorse or promote products derived from the Software without 
 * specific prior written permission. 
 *
 * A patent license for any Necessary Claims owned by Members of LiPS Forum 
 * to make, have made, use, import, offer to sell, lease and sell or otherwise 
 * distribute any implementation compliant with the any specification adopted 
 * by the LiPS Forumcan be obtained from the respective Members on reasonable 
 * and non-discriminatory terms and conditions and under reciprocity, as 
 * regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 * "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 * AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 * ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 */


/**
 *@file API implememt of the Instance Message Interface - IM Contact Implementation
 *
 *@version 0.1
 */

#include "imcontact.h"
#include "imcontact_imp.h"
#include <string.h>
#include <glib.h>

char *
im_subs_state_2_str (im_subs_state_t state)
{
  switch (state)
  {
    case IM_SUB_INVALID:
      return "Invalid";
    case IM_SUB_NONE:
      return "NONE";
    case IM_SUB_TO:
      return "TO";
    case IM_SUB_FROM:
      return "FROM";
    case IM_SUB_BOTH:
      return "BOTH";
    default:
      return "Invalid";
  }
};

im_err_t
im_roster_item_free (im_roster_item_t *item)
{
  g_assert (item);
  g_free (item->contactor);
  g_free (item->nickname);
  g_free (item->group);
  g_free (item);
  return IM_ERROR_NONE;
};

static void
im_roster_free_item (im_roster_item_t *item, gpointer user_data)
{
  im_roster_item_free (item);
};

struct _im_roster_t
{
  im_sid_t sid;
  GList *head;
};

im_err_t
im_roster_new (im_roster_t **pRoster)
{
  *pRoster = g_new0 (im_roster_t, 1);
  (*pRoster)->head = NULL;
  return *pRoster ? IM_ERROR_NONE : IM_ERROR_NOMEM;
};

im_err_t
im_roster_new_with_sid (im_roster_t **pRoster, im_sid_t sid)
{
  *pRoster = g_new0 (im_roster_t, 1);
  (*pRoster)->sid = sid;
  (*pRoster)->head = NULL;
  return *pRoster ? IM_ERROR_NONE : IM_ERROR_NOMEM;
};

im_err_t
im_roster_get_sid (im_roster_t *roster, im_sid_t *pSid)
{
  g_assert (roster);
  *pSid = roster->sid;
  return IM_ERROR_NONE;
};

im_err_t
im_roster_set_sid (im_roster_t *roster, im_sid_t sid)
{
  g_assert (roster);
  roster->sid = sid;
  return IM_ERROR_NONE;
};

im_err_t
im_roster_append (im_roster_t *roster, im_roster_item_t *item)
{
  g_assert (roster);
  roster->head = g_list_append (roster->head, item);
  return IM_ERROR_NONE;
};

im_err_t
im_roster_prepend (im_roster_t *roster, im_roster_item_t *item)
{
  g_assert (roster);
  roster->head = g_list_prepend (roster->head, item);
  return IM_ERROR_NONE;
};

im_err_t
im_roster_get_length (im_roster_t *roster, unsigned int *pLength)
{
  g_assert (roster);
  *pLength = g_list_length (roster->head);
  return IM_ERROR_NONE;
};

im_err_t
im_roster_get_item (im_roster_t *roster, unsigned int position,
                    im_roster_item_t **pItem)
{
  g_assert (roster);
  *pItem = (im_roster_item_t *) g_list_nth_data (roster->head, position);
  return *pItem ? IM_ERROR_NONE : IM_ERROR_NULL;
};

static int
_im_uri_cmp (im_roster_item_t *item, char *uri)
{
  if (!item)
    return -1;                  //if item is NULL, return FALSE.
  return strcmp (item->contactor, uri);
}

im_err_t
im_roster_get_by_uri (im_roster_t *roster, char *uri,
                      im_roster_item_t **pItem)
{
  g_assert (roster);
  GList *list =
    g_list_find_custom (roster->head, uri, (GCompareFunc) _im_uri_cmp);
  if (list)
  {
    *pItem = list->data;
    return IM_ERROR_NONE;
  }
  *pItem = NULL;
  return IM_ERROR_NULL;
};

im_err_t
im_roster_insert (im_roster_t *roster, im_roster_item_t *item, int position)
{
  g_assert (roster);
  roster->head = g_list_insert (roster->head, item, position);
  return IM_ERROR_NONE;
};

im_err_t
im_roster_remove (im_roster_t *roster, im_roster_item_t *item)
{
  g_assert (roster);
  roster->head = g_list_remove (roster->head, item);
  return IM_ERROR_NONE;
};

im_err_t
im_roster_position (im_roster_t *roster, im_roster_item_t *item,
                    int *pPosition)
{
  g_assert (roster);
  *pPosition = g_list_index (roster->head, item);
  return (*pPosition >= 0) ? IM_ERROR_NONE : IM_ERROR_NULL;
};

im_err_t
im_roster_free (im_roster_t *roster)
{
  g_assert (roster);
  //  GList *list = (*pRoster)->head;
  g_list_foreach (roster->head, (GFunc) im_roster_free_item, NULL);
  g_list_free (roster->head);
  return IM_ERROR_NONE;
};

struct _im_buddy_op_t
{
  im_roster_item_t *buddy;
  im_buddy_operation_t op_type;
};

im_err_t
im_buddy_op_new (im_roster_item_t *buddy, im_buddy_operation_t operation,
                 im_buddy_op_t **pBuddy_op)
{
  if (!buddy)
    return IM_ERROR_NULL;
  *pBuddy_op = g_new0 (im_buddy_op_t, 1);
  if (!*pBuddy_op)
    return IM_ERROR_NOMEM;
  (*pBuddy_op)->buddy = buddy;
  (*pBuddy_op)->op_type = operation;
  return IM_ERROR_NONE;
};

im_err_t
im_buddy_op_free (im_buddy_op_t *buddy_op)
{
  g_free (buddy_op);
  return IM_ERROR_NONE;
};

im_err_t
im_buddy_op_free_all (im_buddy_op_t *buddy_op)
{
  g_assert (buddy_op);
  im_roster_item_free (buddy_op->buddy);
  return im_buddy_op_free (buddy_op);
};

im_err_t
im_buddy_op_get_buddy (im_buddy_op_t *buddy_op, im_roster_item_t **pBuddy)
{
  g_assert (buddy_op);
  *pBuddy = buddy_op->buddy;
  return IM_ERROR_NONE;
}

im_err_t
im_buddy_op_get_operation (im_buddy_op_t *buddy_op,
                           im_buddy_operation_t *pOperation)
{
  g_assert (buddy_op);
  *pOperation = buddy_op->op_type;
  return IM_ERROR_NONE;
};

im_err_t
im_buddy_op_set_operation (im_buddy_op_t *buddy_op,
                           im_buddy_operation_t operation)
{
  g_assert (buddy_op);
  buddy_op->op_type = operation;
  return IM_ERROR_NONE;
};

char *
im_buddy_operation_2_str (im_buddy_operation_t op_type)
{
  switch (op_type)
  {
    case IM_BUDDY_ADD:
      return "Add";
    case IM_BUDDY_REMOVE:
      return "Remove";
    case IM_BUDDY_UPDATE:
      return "Update";
    case IM_BUDDY_INFO:
      return "Info";
    default:
      return "invalid_op";
  }
};

