/*  libim - Library of Instant Messaging and Presence enabler
 * imcontact_imp.h - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 * This software and associated documentation files (the "Software") 
 * are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 * All Rights Reserved. 
 *
 * A copyright license is hereby granted for redistribution and use of 
 * the Software in source and binary forms, with or without modification, 
 * provided that the following conditions are met: 
 * - Redistributions of source code must retain the above copyright notice, 
 * this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 * notice, this copyright license and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution. 
 * - Neither the name of LiPS  nor the names of its Members may be used 
 * to endorse or promote products derived from the Software without 
 * specific prior written permission. 
 *
 * A patent license for any Necessary Claims owned by Members of LiPS Forum 
 * to make, have made, use, import, offer to sell, lease and sell or otherwise 
 * distribute any implementation compliant with the any specification adopted 
 * by the LiPS Forumcan be obtained from the respective Members on reasonable 
 * and non-discriminatory terms and conditions and under reciprocity, as 
 * regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 * "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 * AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 * ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 */

/**
 *@file imcontact_imp.h
 *API definition of the Instance Message Interface - imcontact_imp
 *
 *@version $VERSION$
 *@author ZHAO Liangjing <liangjing.zhao@orange-ft.com>, (C) 2006
 */

#ifndef __LIPS_IM_IMCONTACT_IMP_H
#define __LIPS_IM_IMCONTACT_IMP_H

#include "imcontact.h"

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */

  /**
   *@addtogroup im_roster
   *@{
   */

  /**
   *@brief create a new #im_roster_t structure
   *
   *@param pRoster the pointer to the pointer of the new #im_roster_t structure.
   *@return an #im_err_t value.
   *        - #IM_ERROR_NONE no error.
   *        - #IM_ERROR_NOMEM not enough memory.
   */
  im_err_t im_roster_new (im_roster_t **pRoster);

  /**
   *@brief create a new #im_roster_t structure and initialize it's session id & protocol
   *
   *@param pRoster the pointer to the pointer of the new #im_roster_t structure.
   *@param sid the intial session id.
   *@return an #im_err_t value.
   *        - #IM_ERROR_NONE no error.
   *        - #IM_ERROR_NOMEM not enough memory.
   */
  im_err_t im_roster_new_with_sid (im_roster_t **pRoster, im_sid_t sid);

  /**
   *@brief set the sid of an #im_roster_t.
   *
   *@param roster the #im_roster_t.
   *@param sid the session id.
   *@return an #im_err_t value.
   *        - #IM_ERROR_NONE no error.
   */
  im_err_t im_roster_set_sid (im_roster_t *roster, im_sid_t sid);

  /**
   *@brief append an #im_roster_item_t to the end of #im_roster_t
   *
   *@param roster the #im_roster_t into where the #im_roster_item_t will append.
   *@param item the #im_roster_item_t will append to.
   *@return an #im_err_t value.
   *        - #IM_ERROR_NONE no error.
   *        - #IM_ERROR_NOMEM not enough memory.


   */
  im_err_t im_roster_append (im_roster_t *roster, im_roster_item_t *item);

  /**
   *@brief prepend an #im_roster_item_t to the start of #im_roster_t
   *
   *@param roster the #im_roster_t into where the #im_roster_item_t will prepend.
   *@param item the #im_roster_item_t will prepend to.
   *@return an #im_err_t value.
   *        - #IM_ERROR_NONE no error.
   *        - #IM_ERROR_NOMEM not enough memory.
   */
  im_err_t im_roster_prepend (im_roster_t *roster, im_roster_item_t *item);

  /**
   *@brief Inserts a new #im_roster_item_t into the #im_roster_t at the given position.
   *
   *@param roster the #im_roster_t into where the #im_roster_item_t will insert.
   *@param item the #im_roster_item_t will insert to.
   *@param position the position to insert the item.
   *                If this is negative, or is larger than the number of elements
   *                in the list, the new element is added on to the end of the list.
   *@return an #im_err_t value.
   *        - #IM_ERROR_NONE no error.
   *        - #IM_ERROR_NOMEM not enough memory.
   */
  im_err_t im_roster_insert (im_roster_t *roster, im_roster_item_t *item,
                             int position);

  /**
   *@brief remove an #im_roster_item_t from the #im_roster_t
   *
   *The roster will unchanged if there is no such roster item.
   *@param roster the #im_roster_t from which the #im_roster_item_t will be removed.
   *@param item the #im_roster_item_t will be removed.
   *@return an #im_err_t value.
   *        - #IM_ERROR_NONE no error.
   */
  im_err_t im_roster_remove (im_roster_t *roster, im_roster_item_t *item);

  /**@} */

#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif                          /* __LIPS_IM_IMCONTACT_IMP_H */

