/*  libim - Library of Instant Messaging and Presence enabler
 * imevent.c - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 * This software and associated documentation files (the "Software") 
 * are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 * All Rights Reserved. 
 *
 * A copyright license is hereby granted for redistribution and use of 
 * the Software in source and binary forms, with or without modification, 
 * provided that the following conditions are met: 
 * - Redistributions of source code must retain the above copyright notice, 
 * this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 * notice, this copyright license and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution. 
 * - Neither the name of LiPS  nor the names of its Members may be used 
 * to endorse or promote products derived from the Software without 
 * specific prior written permission. 
 *
 * A patent license for any Necessary Claims owned by Members of LiPS Forum 
 * to make, have made, use, import, offer to sell, lease and sell or otherwise 
 * distribute any implementation compliant with the any specification adopted 
 * by the LiPS Forumcan be obtained from the respective Members on reasonable 
 * and non-discriminatory terms and conditions and under reciprocity, as 
 * regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 * "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 * AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 * ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 */


/**
 *@file API implememt of the Instance Message Interface - IM Event
 *
 *@version 0.1
 */

#include "imtype.h"
#include "imevent.h"
#include "imevent_imp.h"
#include "im_dbus.h"
#include "im_client_dbus.h"

char *
im_evt_type_2_str (im_evt_type_t etype)
{
  switch (etype)
  {
    case IM_EVT_INVALID:
      return "IM_EVT_INVALID";
    case IM_REQ_LOGIN:
      return "IM_REQ_LOGIN";
    case IM_REQ_LOGOUT:
      return "IM_REQ_LOGOUT";
    case IM_REQ_ROSTER:
      return "IM_REQ_ROSTER";
    case IM_REQ_SUBSCRIBE:
      return "IM_REQ_SUBSCRIBE";
    case IM_REQ_ACCEPTANCE:
      return "IM_REQ_ACCEPTANCE";
    case IM_REQ_PRESENCE:
      return "IM_REQ_PRESENCE";
    case IM_REQ_PRESENCE_PROBE:
      return "IM_REQ_PRESENCE_PROBE";
    case IM_REQ_MESSAGE:
      return "IM_REQ_MESSAGE";
    case IM_REQ_GROUP:
      return "IM_REQ_GROUP";
    case IM_REQ_BUDDY:
      return "IM_REQ_BUDDY";
    case IM_REQ_SYNCH:
      return "IM_REQ_SYNCH";
    case IM_EVT_ROSTER:
      return "IM_EVT_ROSTER";
    case IM_EVT_PRESENCE:
      return "IM_EVT_PRESENCE";
    case IM_EVT_MESSAGE:
      return "IM_EVT_MESSAGE";
    case IM_EVT_SUBSCRIBE:
      return "IM_EVT_SUBSCRIBE";
    case IM_EVT_ACCEPTANCE:
      return "IM_EVT_ACCEPTANCE";
    case IM_EVT_BUDDY:
      return "IM_EVT_BUDDY";
    case IM_EVT_ERROR:
      return "IM_EVT_ERROR";
    case IM_EVT_LOGIN:
      return "IM_EVT_LOGIN";
    case IM_EVT_LOGOUT:
      return "IM_EVT_LOGOUT";
    default:
      return "";
  }
};

im_err_t
im_subs_msg_free (im_subs_msg_t *msg)
{
  g_assert (msg);
  if (msg->contactor)
    g_free (msg->contactor);
  if (msg->msg)
    g_free (msg->msg);
  if (msg->group)
    g_free (msg->group);
  if (msg->nick)
    g_free (msg->nick);
  g_free (msg);
  return IM_ERROR_NONE;
};

im_err_t
im_accept_msg_free (im_accept_msg_t *msg)
{
  g_assert (msg);
  if (msg->contactor)
    g_free (msg->contactor);
  if (msg->msg)
    g_free (msg->msg);
  g_free (msg);
  return IM_ERROR_NONE;
};

im_err_t
im_param_login_free (im_param_login_t *param)
{
  g_assert (param);
  g_free (param->passwd);
  g_free (param);
  return IM_ERROR_NONE;
};

im_err_t
im_network_error_free (im_network_error_t *err)
{
  g_assert (err);
  g_free (err->msg);
  g_free (err);
  return IM_ERROR_NONE;
};

im_err_t
im_evt_register (sid_t sid, evt_type_t type, evt_handler_t *handler)
{
  DBusError error;
  DBusMessage *message = NULL;
  DBusMessage *reply = NULL;
  char *msg = IM_DBUS_MSG_REG;
  evt_err_t result = EVT_ERROR_MEM;
  im_evt_type_t etype = type;

  message = dbus_message_new_method_call (IM_DBUS_SERVICE,
                                          IM_DBUS_OBJECT,
                                          IM_DBUS_INTERFACE, msg);

  dbus_message_append_args (message,
                            DBUS_TYPE_UINT32, &sid,
                            DBUS_TYPE_INT32, &etype, DBUS_TYPE_INVALID);

  dbus_error_init (&error);

  reply = dbus_connection_send_with_reply_and_block (bus_conn,
                                                     message,
                                                     IM_DBUS_CALL_DELAY,
                                                     &error);

  dbus_message_unref (message);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to send message '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_message_get_args (reply, &error,
                         DBUS_TYPE_INT32, &result, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to get reply '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_error_free (&error);
  dbus_message_unref (reply);
  if (result == IM_ERROR_NONE)
    evt_register (im_evt_src, sid, type, handler);
  return result;
};

im_err_t
im_evt_unregister (sid_t sid, evt_type_t type)
{
  DBusError error;
  DBusMessage *message = NULL;
  DBusMessage *reply = NULL;
  char *msg = IM_DBUS_MSG_UNREG;
  evt_err_t result = EVT_ERROR_MEM;
  im_evt_type_t etype = type;

  message = dbus_message_new_method_call (IM_DBUS_SERVICE,
                                          IM_DBUS_OBJECT,
                                          IM_DBUS_INTERFACE, msg);

  dbus_message_append_args (message,
                            DBUS_TYPE_UINT32, &sid,
                            DBUS_TYPE_INT32, &etype, DBUS_TYPE_INVALID);

  dbus_error_init (&error);

  reply = dbus_connection_send_with_reply_and_block (bus_conn,
                                                     message,
                                                     IM_DBUS_CALL_DELAY,
                                                     &error);

  dbus_message_unref (message);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to send message '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_message_get_args (reply, &error,
                         DBUS_TYPE_INT32, &result, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to get reply '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_error_free (&error);
  dbus_message_unref (reply);

  if (result == EVT_ERROR_NONE)
    evt_unregister (im_evt_src, sid, type);
  return result;
};


// evt_err_t im_evt_release(evt_src_t *evt_src,
//                          evt_t*evt)
// {
//   if(evt->evt_data)
//   {
//     switch(evt->evt_type)
//     {
//     case IM_EVT_ROSTER:
//       im_roster_free((im_roster_t *)(evt->evt_data));
//       break;
//     case IM_REQ_ROSTER:
//       break;
//     case IM_EVT_PRESENCE:
//     case IM_REQ_PRESENCE:
//     case IM_REQ_PRESENCE_PROBE:
//       im_presence_free((im_presence_t *)(evt->evt_data));
//       break;
//     case IM_EVT_MESSAGE:
//       im_msg_free((im_msg_t *)(evt->evt_data));
//       break;
//     case IM_EVT_SUBSCRIBE:
//       im_subs_msg_free((im_subs_msg_t *)(evt->evt_data));
//       break;
//     case IM_EVT_ACCEPTANCE:
//       im_accept_msg_free((im_accept_msg_t *)(evt->evt_data));
//       break;
//     case IM_EVT_ERROR:
//       im_network_error_free((im_network_error_t *)(evt->evt_data));
//       break;
//     case IM_EVT_BUDDY:
//       im_buddy_op_free_all((im_buddy_op_t *)(evt->evt_data));
//       break;
//     case IM_EVT_LOGIN:
//       break;
//     case IM_EVT_LOGOUT:
//       break;
//     default:
//       break;
//     }
//   }
//   return _evt_release(evt_src, evt);
// };

/**
 *Asynchronous requests
 */
im_err_t
im_req_cancel (im_sid_t sid, im_rid_t rid)
{
  return IM_ERROR_NOSUPPORT;
};

im_err_t
im_login (im_sid_t sid,
          evt_handler_t *handler, im_rid_t *pRid, im_param_login_t *login)
{
  DBusError error;
  DBusMessage *message = NULL;
  DBusMessage *reply = NULL;
  im_err_t err = IM_ERROR_NONE;
  char *msg = IM_DBUS_MSG_LOGIN;
  im_rid_t rid = 0;
  char *uri;

  im_uri_get_uri_full (login->user, &uri);

  message = dbus_message_new_method_call (IM_DBUS_SERVICE,
                                          IM_DBUS_OBJECT,
                                          IM_DBUS_INTERFACE, msg);

  dbus_message_append_args (message,
                            DBUS_TYPE_UINT32, &sid,
                            DBUS_TYPE_STRING, &(login->passwd),
                            DBUS_TYPE_INVALID);

  dbus_error_init (&error);

  reply = dbus_connection_send_with_reply_and_block (bus_conn,
                                                     message,
                                                     IM_DBUS_CALL_DELAY,
                                                     &error);

  dbus_message_unref (message);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to send message '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_message_get_args (reply, &error,
                         DBUS_TYPE_UINT32, &rid,
                         DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to get reply '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_error_free (&error);
  dbus_message_unref (reply);
  *pRid = rid;
  if (err != IM_ERROR_NONE)
  {
    return err;
  }
  return async_evt_register (im_evt_src, sid, rid, handler);
};

/**
 *im_logout can not be canceled!
 */
im_err_t
im_logout (im_sid_t sid, evt_handler_t *handler, im_rid_t *pRid)
{
  DBusError error;
  DBusMessage *message = NULL;
  DBusMessage *reply = NULL;
  im_err_t err = IM_ERROR_NONE;
  char *msg = IM_DBUS_MSG_LOGOUT;
  im_rid_t rid = 0;

  message = dbus_message_new_method_call (IM_DBUS_SERVICE,
                                          IM_DBUS_OBJECT,
                                          IM_DBUS_INTERFACE, msg);

  dbus_message_append_args (message, DBUS_TYPE_UINT32, &sid, DBUS_TYPE_INVALID);

  dbus_error_init (&error);

  reply = dbus_connection_send_with_reply_and_block (bus_conn,
                                                     message,
                                                     IM_DBUS_CALL_DELAY,
                                                     &error);

  dbus_message_unref (message);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to send message '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_message_get_args (reply, &error,
                         DBUS_TYPE_UINT32, &rid,
                         DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to get reply '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_error_free (&error);
  dbus_message_unref (reply);
  *pRid = rid;
  if (err != IM_ERROR_NONE)
  {
    return err;
  }
  return async_evt_register (im_evt_src, sid, rid, handler);
};

im_err_t
im_send_message (im_sid_t sid,
                 evt_handler_t *handler, im_rid_t *pRid, im_msg_t *smsg)
{
  DBusError error;
  DBusMessage *message = NULL;
  DBusMessage *reply = NULL;
  im_err_t err = IM_ERROR_NONE;
  char *msg = IM_DBUS_MSG_SENDMSG;
  im_rid_t rid = 0;
  char *to;
  char *body;
  char *encode;
  char *blank = "";
  size_t bsize;
  im_msg_type_t mtype;
  im_msg_ctype_t mctype;

  im_msg_get_recipient (smsg, &to);
  im_msg_get_body (smsg, (void **) &body);
  im_msg_get_body_length (smsg, &bsize);
  im_msg_get_encode (smsg, &encode);
  im_msg_get_type (smsg, &mtype);
  im_msg_get_ctype (smsg, &mctype);
  dbus_uint32_t body_len = bsize;

  message = dbus_message_new_method_call (IM_DBUS_SERVICE,
                                          IM_DBUS_OBJECT,
                                          IM_DBUS_INTERFACE, msg);

  dbus_message_append_args (message,
                            DBUS_TYPE_UINT32, &sid,
                            DBUS_TYPE_STRING, (to) ? &to : &blank,
                            DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, &body, body_len,
                            DBUS_TYPE_STRING, (encode) ? &encode : &blank,
                            DBUS_TYPE_INT32, &mtype,
                            DBUS_TYPE_INT32, &mctype, DBUS_TYPE_INVALID);

  dbus_error_init (&error);

  reply = dbus_connection_send_with_reply_and_block (bus_conn,
                                                     message,
                                                     IM_DBUS_CALL_DELAY,
                                                     &error);

  dbus_message_unref (message);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to send message '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    g_free (to);
    g_free (body);
    g_free (encode);
    return IM_ERROR_DBUS;
  }

  dbus_message_get_args (reply, &error,
                         DBUS_TYPE_UINT32, &rid,
                         DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to get reply '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    dbus_message_unref (reply);
    g_free (to);
    g_free (body);
    g_free (encode);
    return IM_ERROR_DBUS;
  }

  dbus_error_free (&error);
  dbus_message_unref (reply);
  g_free (to);
  g_free (body);
  g_free (encode);
  *pRid = rid;
  if (err != IM_ERROR_NONE)
  {
    return err;
  }
  return async_evt_register (im_evt_src, sid, rid, handler);
};

im_err_t
im_change_presence (im_sid_t sid,
                    evt_handler_t *handler,
                    im_rid_t *pRid, im_presence_t *presence)
{
  DBusError error;
  DBusMessage *message = NULL;
  DBusMessage *reply = NULL;
  im_err_t err = IM_ERROR_NONE;
  char *msg = IM_DBUS_MSG_CHANGE_PRESENCE;
  im_rid_t rid = 0;
  char *desc;
  char *to;
  im_presence_state_t status;

  im_presence_get_state (presence, &status);
  im_presence_get_desc (presence, &desc);
  im_presence_get_to (presence, &to);
  if (!to)
    to = g_strdup ("");
  message = dbus_message_new_method_call (IM_DBUS_SERVICE,
                                          IM_DBUS_OBJECT,
                                          IM_DBUS_INTERFACE, msg);

  dbus_message_append_args (message,
                            DBUS_TYPE_UINT32, &sid,
                            DBUS_TYPE_STRING, &to,
                            DBUS_TYPE_UINT32, &status,
                            DBUS_TYPE_STRING, &desc, DBUS_TYPE_INVALID);

  dbus_error_init (&error);

  reply = dbus_connection_send_with_reply_and_block (bus_conn,
                                                     message,
                                                     IM_DBUS_CALL_DELAY,
                                                     &error);

  dbus_message_unref (message);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to send message '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    g_free (desc);
    g_free (to);
    return IM_ERROR_DBUS;
  }

  dbus_message_get_args (reply, &error,
                         DBUS_TYPE_UINT32, &rid,
                         DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to get reply '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    g_free (desc);
    g_free (to);
    return IM_ERROR_DBUS;
  }

  dbus_error_free (&error);
  dbus_message_unref (reply);
  g_free (desc);
  g_free (to);
  *pRid = rid;
  if (err != IM_ERROR_NONE)
  {
    return err;
  }
  return async_evt_register (im_evt_src, sid, rid, handler);
};

im_err_t
im_presence_probe (im_sid_t sid,
                   evt_handler_t *handler, im_rid_t *pRid, char *uri)
{
  if (!uri)
    return IM_ERROR_WRONG_URI;
  DBusError error;
  DBusMessage *message = NULL;
  DBusMessage *reply = NULL;
  im_err_t err = IM_ERROR_NONE;
  char *msg = IM_DBUS_MSG_PRESENCE_PROBE;
  im_rid_t rid = 0;

  message = dbus_message_new_method_call (IM_DBUS_SERVICE,
                                          IM_DBUS_OBJECT,
                                          IM_DBUS_INTERFACE, msg);

  dbus_message_append_args (message,
                            DBUS_TYPE_UINT32, &sid,
                            DBUS_TYPE_STRING, &uri, DBUS_TYPE_INVALID);

  dbus_error_init (&error);

  reply = dbus_connection_send_with_reply_and_block (bus_conn,
                                                     message,
                                                     IM_DBUS_CALL_DELAY,
                                                     &error);

  dbus_message_unref (message);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to send message '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_message_get_args (reply, &error,
                         DBUS_TYPE_UINT32, &rid,
                         DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to get reply '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_error_free (&error);
  dbus_message_unref (reply);
  *pRid = rid;
  if (err != IM_ERROR_NONE)
  {
    return err;
  }
  return async_evt_register (im_evt_src, sid, rid, handler);
};

im_err_t
im_subscribe (im_sid_t sid,
              evt_handler_t *handler,
              im_rid_t *pRid, im_subs_msg_t *subs_msg)
{
  DBusError error;
  DBusMessage *message = NULL;
  DBusMessage *reply = NULL;
  im_err_t err = IM_ERROR_NONE;
  char *msg = IM_DBUS_MSG_SUBSCRIBE;
  im_rid_t rid = 0;

  message = dbus_message_new_method_call (IM_DBUS_SERVICE,
                                          IM_DBUS_OBJECT,
                                          IM_DBUS_INTERFACE, msg);

  dbus_message_append_args (message,
                            DBUS_TYPE_UINT32, &sid,
                            DBUS_TYPE_STRING, &(subs_msg->contactor),
                            DBUS_TYPE_BOOLEAN, &(subs_msg->subscribe),
                            DBUS_TYPE_STRING, &(subs_msg->msg),
                            DBUS_TYPE_STRING, &(subs_msg->nick),
                            DBUS_TYPE_STRING, &(subs_msg->group),
                            DBUS_TYPE_INVALID);

  dbus_error_init (&error);

  reply = dbus_connection_send_with_reply_and_block (bus_conn,
                                                     message,
                                                     IM_DBUS_CALL_DELAY,
                                                     &error);

  dbus_message_unref (message);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to send message '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_message_get_args (reply, &error,
                         DBUS_TYPE_UINT32, &rid,
                         DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to get reply '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_error_free (&error);
  dbus_message_unref (reply);
  *pRid = rid;
  if (err != IM_ERROR_NONE)
  {
    return err;
  }
  return async_evt_register (im_evt_src, sid, rid, handler);
};

im_err_t
im_acceptance (im_sid_t sid,
               evt_handler_t *handler,
               im_rid_t *pRid, im_accept_msg_t *accept_msg)
{
  DBusError error;
  DBusMessage *message = NULL;
  DBusMessage *reply = NULL;
  im_err_t err = IM_ERROR_NONE;
  char *msg = IM_DBUS_MSG_ACCEPT;
  im_rid_t rid = 0;

  message = dbus_message_new_method_call (IM_DBUS_SERVICE,
                                          IM_DBUS_OBJECT,
                                          IM_DBUS_INTERFACE, msg);

  dbus_message_append_args (message,
                            DBUS_TYPE_UINT32, &sid,
                            DBUS_TYPE_STRING, &(accept_msg->contactor),
                            DBUS_TYPE_BOOLEAN, &(accept_msg->accept),
                            DBUS_TYPE_STRING, &(accept_msg->msg),
                            DBUS_TYPE_INVALID);

  dbus_error_init (&error);

  reply = dbus_connection_send_with_reply_and_block (bus_conn,
                                                     message,
                                                     IM_DBUS_CALL_DELAY,
                                                     &error);

  dbus_message_unref (message);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to send message '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_message_get_args (reply, &error,
                         DBUS_TYPE_UINT32, &rid,
                         DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to get reply '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_error_free (&error);
  dbus_message_unref (reply);
  *pRid = rid;
  if (err != IM_ERROR_NONE)
  {
    return err;
  }
  return async_evt_register (im_evt_src, sid, rid, handler);
};

im_err_t
im_get_roster (im_sid_t sid, evt_handler_t *handler, im_rid_t *pRid)
{
  DBusError error;
  DBusMessage *message = NULL;
  DBusMessage *reply = NULL;
  im_err_t err = IM_ERROR_NONE;
  char *msg = IM_DBUS_MSG_GET_ROSTER;
  im_rid_t rid = 0;

  message = dbus_message_new_method_call (IM_DBUS_SERVICE,
                                          IM_DBUS_OBJECT,
                                          IM_DBUS_INTERFACE, msg);

  dbus_message_append_args (message, DBUS_TYPE_UINT32, &sid, DBUS_TYPE_INVALID);

  dbus_error_init (&error);

  reply = dbus_connection_send_with_reply_and_block (bus_conn,
                                                     message,
                                                     IM_DBUS_CALL_DELAY,
                                                     &error);

  dbus_message_unref (message);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to send message '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_message_get_args (reply, &error,
                         DBUS_TYPE_UINT32, &rid,
                         DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to get reply '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_error_free (&error);
  dbus_message_unref (reply);
  *pRid = rid;
  if (err != IM_ERROR_NONE)
  {
    return err;
  }
  return async_evt_register (im_evt_src, sid, rid, handler);
};

im_err_t
im_buddy_op (im_sid_t sid,
             evt_handler_t *handler, im_rid_t *pRid, im_buddy_op_t *op)
{
  DBusError error;
  DBusMessage *message = NULL;
  DBusMessage *reply = NULL;
  im_err_t err = IM_ERROR_NONE;
  char *msg = IM_DBUS_MSG_BUDDY;
  im_rid_t rid = 0;
  im_roster_item_t *item = NULL;
  im_buddy_operation_t op_type;
  char *blank = "";

  im_buddy_op_get_buddy (op, &item);
  im_buddy_op_get_operation (op, &op_type);
  if (!item->contactor)
    return IM_ERROR_PARAM;
  message = dbus_message_new_method_call (IM_DBUS_SERVICE,
                                          IM_DBUS_OBJECT,
                                          IM_DBUS_INTERFACE, msg);

  dbus_message_append_args (message,
                            DBUS_TYPE_UINT32, &sid,
                            DBUS_TYPE_STRING, &(item->contactor),
                            DBUS_TYPE_STRING,
                            item->nickname ? &(item->nickname) : &blank,
                            DBUS_TYPE_STRING,
                            item->group ? &(item->group) : &blank,
                            DBUS_TYPE_INT32, &op_type, DBUS_TYPE_INVALID);

  dbus_error_init (&error);

  reply = dbus_connection_send_with_reply_and_block (bus_conn,
                                                     message,
                                                     IM_DBUS_CALL_DELAY,
                                                     &error);

  dbus_message_unref (message);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to send message '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_message_get_args (reply, &error,
                         DBUS_TYPE_UINT32, &rid,
                         DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to get reply '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_error_free (&error);
  dbus_message_unref (reply);
  *pRid = rid;
  if (err != IM_ERROR_NONE)
  {
    return err;
  }
  return async_evt_register (im_evt_src, sid, rid, handler);
};

im_err_t
im_buddies_synch (im_sid_t sid, evt_handler_t *handler, im_rid_t *pRid)
{
  DBusError error;
  DBusMessage *message = NULL;
  DBusMessage *reply = NULL;
  im_err_t err = IM_ERROR_NONE;
  char *msg = IM_DBUS_MSG_SYNCH;
  im_rid_t rid = 0;

  message = dbus_message_new_method_call (IM_DBUS_SERVICE,
                                          IM_DBUS_OBJECT,
                                          IM_DBUS_INTERFACE, msg);

  dbus_message_append_args (message, DBUS_TYPE_UINT32, &sid, DBUS_TYPE_INVALID);

  dbus_error_init (&error);

  reply = dbus_connection_send_with_reply_and_block (bus_conn,
                                                     message,
                                                     IM_DBUS_CALL_DELAY,
                                                     &error);

  dbus_message_unref (message);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to send message '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_message_get_args (reply, &error,
                         DBUS_TYPE_UINT32, &rid,
                         DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to get reply '%s': %s\n", msg, error.message);
    dbus_error_free (&error);
    return IM_ERROR_DBUS;
  }

  dbus_error_free (&error);
  dbus_message_unref (reply);
  *pRid = rid;
  if (err != IM_ERROR_NONE)
  {
    return err;
  }
  return async_evt_register (im_evt_src, sid, rid, handler);
};

