/*  libim - Library of Instant Messaging and Presence enabler
 * immsg.c - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 * This software and associated documentation files (the "Software") 
 * are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 * All Rights Reserved. 
 *
 * A copyright license is hereby granted for redistribution and use of 
 * the Software in source and binary forms, with or without modification, 
 * provided that the following conditions are met: 
 * - Redistributions of source code must retain the above copyright notice, 
 * this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 * notice, this copyright license and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution. 
 * - Neither the name of LiPS  nor the names of its Members may be used 
 * to endorse or promote products derived from the Software without 
 * specific prior written permission. 
 *
 * A patent license for any Necessary Claims owned by Members of LiPS Forum 
 * to make, have made, use, import, offer to sell, lease and sell or otherwise 
 * distribute any implementation compliant with the any specification adopted 
 * by the LiPS Forumcan be obtained from the respective Members on reasonable 
 * and non-discriminatory terms and conditions and under reciprocity, as 
 * regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 * "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 * AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 * ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 */


/**
 *@file API implememt of the Instance Message Interface - IM Message
 *
 *@version 0.1
 */

#include "immsg.h"
#include "immsg_imp.h"
#include <glib.h>
#include <string.h>

///IM Message structure definition.
/**
 *This structure encapsulates all necessary data for an IM message
*/
struct _im_msg_t
{
  im_sid_t sid;                 ///< Session id.
  im_msg_id_t msg_id;           ///< Message id.
  time_t msg_time;              ///< Arrive / Send time.
  im_msg_type_t msg_type;       ///< Message Type.
  im_msg_ctype_t msg_ctype;     ///< Messsage Content Type.
  im_msg_state_t msg_state;     ///< Message state.

  char *recipient;              ///< Destinations, should be one or more uri string(s), separated by semicolon.
  char *from;                   ///< From uri string.
  char *encode;                 ///< Content Encode: e.g. en-US, fr-FR, zh-CH, UTF-8

  size_t body_length;           ///< Length of message body.
  void *body;                   ///< Body data.
};

char *
im_msg_type_2_str (im_msg_type_t msg_type)
{
  switch (msg_type)
  {
    case IM_MSG_TYPE_NORMAL:
      return "NORMAL";
    case IM_MSG_TYPE_CHAT:
      return "CHAT";
    case IM_MSG_TYPE_GROUPCHAT:
      return "GROUPCHAT";
    case IM_MSG_TYPE_HEADLINE:
      return "HEADLINE";
    case IM_MSG_TYPE_ERROR:
      return "ERROR";
    default:
      return "Invalid";
  }
};

im_err_t
im_msg_new (im_sid_t sid, im_msg_type_t type, im_msg_ctype_t ctype,
            im_msg_t **msg)
{
  *msg = g_new0 (im_msg_t, 1);
  if (*msg == NULL)
    return IM_ERROR_NULL;
  (*msg)->sid = sid;
  (*msg)->msg_type = type;
  (*msg)->msg_ctype = ctype;
  return IM_ERROR_NONE;
};

im_err_t
im_msg_set_id (im_msg_t *msg, im_msg_id_t msg_id)
{
  g_assert (msg);
  msg->msg_id = msg_id;
  return IM_ERROR_NONE;
};

im_err_t
im_msg_get_sid (im_msg_t *msg, im_sid_t *sid)
{
  g_assert (msg != NULL);
  *sid = msg->sid;
  return IM_ERROR_NONE;
};

im_err_t
im_msg_get_id (im_msg_t *msg, im_msg_id_t *msg_id)
{
  g_assert (msg != NULL);
  *msg_id = msg->msg_id;
  return IM_ERROR_NONE;
};

im_err_t
im_msg_get_time (im_msg_t *msg, time_t *msg_time)
{
  g_assert (msg != NULL);
  *msg_time = msg->msg_time;
  return IM_ERROR_NONE;
};

im_err_t
im_msg_set_time (im_msg_t *msg, time_t msg_time)
{
  g_assert (msg != NULL);
  msg->msg_time = msg_time;
  return IM_ERROR_NONE;
};

im_err_t
im_msg_get_ctype (im_msg_t *msg, im_msg_ctype_t *ctype)
{
  g_assert (msg != NULL);
  *ctype = msg->msg_ctype;
  return IM_ERROR_NONE;
};

im_err_t
im_msg_get_type (im_msg_t *msg, im_msg_type_t *type)
{
  g_assert (msg != NULL);
  *type = msg->msg_type;
  return IM_ERROR_NONE;
};

im_err_t
im_msg_get_state (im_msg_t *msg, im_msg_state_t *msg_state)
{
  g_assert (msg != NULL);
  *msg_state = msg->msg_state;
  return IM_ERROR_NONE;
};

im_err_t
im_msg_set_state (im_msg_t *msg, im_msg_state_t msg_state)
{
  g_assert (msg != NULL);
  msg->msg_state = msg_state;
  return IM_ERROR_NONE;
};

im_err_t
im_msg_head_length (im_msg_t *msg, size_t *hsize)
{
  g_assert (msg != NULL);
  *hsize =
    strlen (msg->recipient) + strlen (msg->from) + strlen (msg->encode) + 3;
  return IM_ERROR_NONE;
};

im_err_t
im_msg_get_recipient (im_msg_t *msg, char **recipient)
{
  g_assert (msg != NULL);
  *recipient = g_strdup (msg->recipient);
  if (*recipient == NULL)
    return IM_ERROR_NOMEM;
  return IM_ERROR_NONE;
};

im_err_t
im_msg_set_recipient (im_msg_t *msg, char *recipient)
{
  g_assert (msg != NULL);
  if (msg->recipient)
    g_free (msg->recipient);
  msg->recipient = g_strdup (recipient);
  if (msg->recipient == NULL)
    return IM_ERROR_NOMEM;
  return IM_ERROR_NONE;
};

im_err_t
im_msg_get_from (im_msg_t *msg, char **from)
{
  g_assert (msg != NULL);
  *from = g_strdup (msg->from);
  if (*from == NULL)
    return IM_ERROR_NOMEM;
  return IM_ERROR_NONE;
};

im_err_t
im_msg_set_from (im_msg_t *msg, char *from)
{
  g_assert (msg != NULL);
  if (msg->from)
    g_free (from);
  msg->from = g_strdup (from);
  if (msg->from == NULL)
    return IM_ERROR_NOMEM;
  return IM_ERROR_NONE;
};

im_err_t
im_msg_get_body_length (im_msg_t *msg, size_t *bsize)
{
  g_assert (msg != NULL);
  if (bsize == NULL)
    return IM_ERROR_NOMEM;
  *bsize = msg->body_length;
  return IM_ERROR_NONE;
};

im_err_t
im_msg_get_body (im_msg_t *msg, void **body)
{
  g_assert (msg != NULL);
  *body = g_memdup (msg->body, msg->body_length);
  if (*body == NULL)
    return IM_ERROR_NOMEM;
  return IM_ERROR_NONE;
};

im_err_t
im_msg_get_encode (im_msg_t *msg, char **encode)
{
  g_assert (msg != NULL);
  *encode = g_strdup (msg->encode);
  if (*encode == NULL)
    return IM_ERROR_NOMEM;
  return IM_ERROR_NONE;
};

im_err_t
im_msg_set_body (im_msg_t *msg, void *body, size_t len, char *encode)
{
  g_assert (msg != NULL);
  msg->body_length = len;
  if (msg->body)
    g_free (msg->body);
  msg->body = g_memdup (body, len);
  if (msg->body == NULL)
    return IM_ERROR_NOMEM;
  if (msg->encode)
    g_free (msg->encode);
  msg->encode = g_strdup (encode);
  if (msg->encode == NULL)
    return IM_ERROR_NOMEM;
  return IM_ERROR_NONE;
};

im_err_t
im_msg_free (im_msg_t *msg)
{
  g_assert (msg);
  g_free (msg->body);
  g_free (msg->from);
  g_free (msg->recipient);
  g_free (msg);
  return IM_ERROR_NONE;
};

