/*  libim - Library of Instant Messaging and Presence enabler
 * impresence.c - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 * This software and associated documentation files (the "Software") 
 * are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 * All Rights Reserved. 
 *
 * A copyright license is hereby granted for redistribution and use of 
 * the Software in source and binary forms, with or without modification, 
 * provided that the following conditions are met: 
 * - Redistributions of source code must retain the above copyright notice, 
 * this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 * notice, this copyright license and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution. 
 * - Neither the name of LiPS  nor the names of its Members may be used 
 * to endorse or promote products derived from the Software without 
 * specific prior written permission. 
 *
 * A patent license for any Necessary Claims owned by Members of LiPS Forum 
 * to make, have made, use, import, offer to sell, lease and sell or otherwise 
 * distribute any implementation compliant with the any specification adopted 
 * by the LiPS Forumcan be obtained from the respective Members on reasonable 
 * and non-discriminatory terms and conditions and under reciprocity, as 
 * regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 * "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 * AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 * ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 */


/**
 *@file API implememt of the Instance Message Interface - IM Presence
 *
 *@version 0.1
 */

#include "impresence.h"
#include "impresence_imp.h"
#include "imtype_imp.h"
#include <glib.h>


///Presence structure
/**
 * This structure encapsulates all user availability data.
*/
struct _im_presence_t
{
  im_presence_state_t status;   ///< User status.
  char *description;            ///< Extension description for the presense.
  GHashTable *ex_info;          ///< A hash table contains all protocol specific presence properties.
  char *to;                     ///< to uri.
  char *from;                   ///< from uri.
  im_protocol_t prtl;           ///< protocol.
};

char *
im_presence_2_str (im_presence_state_t state)
{
  switch (state)
  {
    case IM_PRESENCE_AVAILABLE:
      return "Available";
    case IM_PRESENCE_AWAY:
      return "Away";
    case IM_PRESENCE_FARAWAY:
      return "Faraway";
    case IM_PRESENCE_DND:
      return "Do not disturb";
    case IM_PRESENCE_INVISIBLE:
      return "Invisible";
    case IM_PRESENCE_CUSTOM:
      return "Custom";
    case IM_PRESENCE_OFFLINE:
      return "Offline";
    default:
      return "";
  }
};

im_err_t
im_presence_new (im_protocol_t prtl, char *to, im_presence_t **p)
{
  *p = g_new0 (im_presence_t, 1);
  if (!*p)
    return IM_ERROR_NULL;
  (*p)->prtl = prtl;
  (*p)->to = g_strdup (to);
  (*p)->ex_info =
    g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
  if ((*p)->ex_info == NULL)
    return IM_ERROR_GENERAL;    //lack memory
  return IM_ERROR_NONE;
};

im_err_t
im_presence_free (im_presence_t *presence)
{
  g_assert (presence);          //presence != NULL;
  if (presence->description)
    g_free (presence->description);
  if (presence->to)
    g_free (presence->to);
  if (presence->from)
    g_free (presence->from);
  if (presence->ex_info)
    g_hash_table_destroy (presence->ex_info);
  g_free (presence);
  return IM_ERROR_NONE;
};

im_err_t
im_presence_get_state (im_presence_t *presence, im_presence_state_t *state)
{
  g_assert (presence);          //presence != NULL;
  *state = presence->status;
  return IM_ERROR_NONE;
};

im_err_t
im_presence_set_state (im_presence_t *presence, im_presence_state_t state)
{
  g_assert (presence);          //presence != NULL;
  presence->status = state;
  return IM_ERROR_NONE;
};

im_err_t
im_presence_get_desc (im_presence_t *presence, char **desc)
{
  g_assert (presence);          //presence != NULL;
  *desc = g_strdup (presence->description);
  if (!*desc)
    return IM_ERROR_NOMEM;
  return IM_ERROR_NONE;
};

im_err_t
im_presence_set_desc (im_presence_t *presence, char *desc)
{
  g_assert (presence);          //presence != NULL;
  if (presence->description)
    g_free (presence->description);
  presence->description = g_strdup (desc);
  if (!presence->description)
    return IM_ERROR_NOMEM;
  return IM_ERROR_NONE;
};

im_err_t
im_presence_get_to (im_presence_t *presence, char **to)
{
  g_assert (presence);          //presence != NULL;
  *to = g_strdup (presence->to);        //this will return a pointer to the user instead of a user copy
  return IM_ERROR_NONE;
};

im_err_t
im_presence_set_to (im_presence_t *presence, char *to)
{
  g_assert (presence);
  if (presence->to)
    g_free (presence->to);
  presence->to = g_strdup (to);
  if (!presence->to)
    return IM_ERROR_NOMEM;
  return IM_ERROR_NONE;
};

im_err_t
im_presence_get_from (im_presence_t *presence, char **from)
{
  g_assert (presence);          //presence != NULL;
  *from = g_strdup (presence->from);    //this will return a pointer to the user instead of a user copy
  return IM_ERROR_NONE;
};

im_err_t
im_presence_set_from (im_presence_t *presence, char *from)
{
  g_assert (presence);
  if (presence->from)
    g_free (presence->from);
  presence->from = g_strdup (from);
  if (!presence->from)
    return IM_ERROR_NOMEM;
  return IM_ERROR_NONE;
};

im_err_t
im_presence_get_protocol (im_presence_t *presence, im_protocol_t *prtl)
{
  g_assert (presence);          //presence != NULL;
  *prtl = presence->prtl;
  return IM_ERROR_NONE;
};

im_err_t
im_presence_get_ex_info (im_presence_t *presence, char *name, char **value)
{
  g_assert (presence);          //presence != NULL;
  char *v = g_hash_table_lookup (presence->ex_info, name);

  if (!v)
    return IM_ERROR_NOSUPPORT;
  *value = g_strdup (v);
  if (!*value)
    return IM_ERROR_NULL;
  return IM_ERROR_NONE;
};

im_err_t
im_presence_set_ex_info (im_presence_t *presence, char *name, char *value)
{
  g_assert (presence);          //presence != NULL;
  g_hash_table_insert (presence->ex_info, g_strdup (name), g_strdup (value));
  return IM_ERROR_NONE;
};

im_err_t
im_presence_get_ex_name_list (im_presence_t *presence, char ***names)
{
  return _im_get_hash_keys (presence->ex_info, names);
};

