/*  libim - Library of Instant Messaging and Presence enabler
 * improxy.cpp - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 */


/**
 *@file API implememt of the Instance Message Interface - IM Proxy
 *
 *@version 0.1
 */

#include "improxy.h"
#include <glib.h>
#include <string.h>
#include "imconnect_base.h"
#include "im_xmpp_connect.h"

static im_proxy *IM_PROXY = NULL;
DBusConnection *bus_conn = NULL;

typedef struct
{
  char *server;
  char *port;
  im_uri_t *user;
  im_protocol_t prtl;
}
im_conn_cmp_t;

static gboolean
im_proxy_conn_find (gpointer key, gpointer value, gpointer user_data)
{
  im_connection *c = ((im_session_t *) value)->conn;
  im_conn_cmp_t *cmp = (im_conn_cmp_t *) user_data;

  return (c->protocol == cmp->prtl &&
          strcmp (c->server_addr, cmp->server) == 0 &&
          strcmp (c->server_port, cmp->port) == 0 &&
          im_uri_cmp (c->login_uri, cmp->user, IM_URI_FULL));
};

void
im_proxy_init (im_proxy *proxy)
{
  proxy->sid_seed = 0;
  //proxy->conn_hs = g_hash_table_new_full(
  //                          g_direct_hash, g_direct_equal, NULL, im_conn_unref);
  proxy->s_hs =
    g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL,
                           (GDestroyNotify) im_session_free);
  if (!g_thread_supported ())
    g_thread_init (NULL);
};

im_proxy *
im_proxy_get (void)
{
  if (IM_PROXY == NULL)
  {
    IM_PROXY = g_new0 (im_proxy, 1);
    g_assert (IM_PROXY);
    im_proxy_init (IM_PROXY);
  }
  return IM_PROXY;
};

im_err_t
im_proxy_free (im_proxy *proxy)
{
  g_assert (proxy);
  //g_hash_table_destroy((GHashTable*)((*proxy)->conn_hs));
  g_hash_table_destroy ((GHashTable *) (proxy->s_hs));
  g_free (proxy);
  return IM_ERROR_NONE;
};

im_connection *
im_proxy_create_conn (im_proxy *proxy,
                      im_protocol_t protocol,
                      char *ver, char *server, char *port, im_uri_t *user)
{
  g_assert (proxy);

  im_connection *c = NULL;

  switch (protocol)
  {
    case IM_PROTOCOL_XMPP:
      g_message ("xxxxx new connection: %s:%s\n", server, port);
      c = im_xmpp_connect_new (ver);
      break;
    case IM_PROTOCOL_SIMPLE:
      break;
    case IM_PROTOCOL_IMPS:
      break;
    default:
      break;
  }

  if (c)
  {
    c->server_addr = g_strdup (server);
    c->server_port = g_strdup (port);
    c->login_uri = user;
    return c;
  }
  return NULL;
};

im_connection *
im_proxy_get_conn (im_proxy *proxy,
                   im_protocol_t protocol,
                   char *ver, char *server, char *port, im_uri_t *user)
{
  im_conn_cmp_t cmp_data;

  cmp_data.port = port;
  cmp_data.server = server;
  cmp_data.user = user;
  cmp_data.prtl = protocol;
  im_session_t *s = (im_session_t *) g_hash_table_find ((GHashTable *) (proxy->
                                                                        s_hs),
                                                        im_proxy_conn_find,
                                                        &cmp_data);
  if (s)
  {
    return im_session_get_conn (s);
  };
  return NULL;
};

im_sid_t
im_proxy_generate_sid (im_proxy *proxy)
{
  return ++(proxy->sid_seed);
};

im_connection *
im_proxy_get_conn_by_sid (im_proxy *proxy, im_sid_t sid)
{
  im_session_t *s = (im_session_t *) g_hash_table_lookup ((GHashTable
                                                           *) (proxy->s_hs),
                                                          GUINT_TO_POINTER
                                                          (sid));
  if (s)
  {
    return im_session_get_conn (s);
  }
  return NULL;
};

im_session_t *
im_proxy_open_session (im_proxy *proxy, char *client_name,
                       im_protocol_t protocol, char *ver, char *server,
                       char *port, im_uri_t *user)
{
  char *p;

  for (p = port; *p != '\0'; p++)
  {
    if (!isdigit (*p))
      return NULL;              //not a valid port!
  }
  im_session_t *s = im_session_new (im_proxy_generate_sid (proxy), client_name);

  if (!s)
    return NULL;
  s->conn = im_proxy_get_conn (proxy, protocol, ver, server, port, user);
  if (s->conn)
  {
    im_conn_ref (s->conn);
  }
  else
  {
    s->conn = im_proxy_create_conn (proxy, protocol, ver, server, port, user);
  }
  if (s->conn)
  {
    g_hash_table_insert ((GHashTable *) (proxy->s_hs),
                         GUINT_TO_POINTER (s->sid), s);
    return s;
  }
  im_session_free (s);
  return NULL;
};

bool
im_proxy_close_session (im_proxy *proxy, im_sid_t sid)
{
  return g_hash_table_remove ((GHashTable *) (proxy->s_hs),
                              GUINT_TO_POINTER (sid));
};

im_session_t *
im_proxy_get_session (im_proxy *proxy, im_sid_t sid)
{
  return (im_session_t *) g_hash_table_lookup ((GHashTable *) (proxy->s_hs),
                                               GUINT_TO_POINTER (sid));
};

im_trans_mgr_t *
im_proxy_get_trains_mgr (im_proxy *proxy, im_sid_t sid)
{
  im_connection *conn = im_proxy_get_conn_by_sid (proxy, sid);

  if (conn)
  {
    return im_conn_get_trans_mgr (conn);
  }
  return NULL;
};

