/*  libim - Library of Instant Messaging and Presence enabler
 * improxy.h - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 */


/**
 *@file API definition of the Instance Message Interface - IM Proxy
 *
 *@version 0.1
 */

#ifndef __LIPS_IM_PROXY_H
#define __LIPS_IM_PROXY_H

#include "imtype.h"
#include "imconnect.h"
#include "imsession.h"
#include "im_dbus.h"
#include "dbus-if.h"
#include <glib.h>

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */

  extern DBusConnection *bus_conn;

  ///IM Client structure
  /**
   *IM proxy mange the session and connections by a hash table.
   */
  typedef struct
  {
    GHashTable *s_hs;           ///< Hash table to record sessions.
    im_sid_t sid_seed;          ///< Session id seed for generation of session id.
  }
  im_proxy;

  im_proxy *im_proxy_get (void);

  im_err_t im_proxy_free (im_proxy *proxy);

  ///Create a connection
  /**
   *Create an im_connection according to the protocol & server address.
   *
   *\param proxy pointer to a proxy structure which owns the connection.
   *\param protocol specify the protocol.
   *\param ver, protocol version.
   *\param server, server url string.
   *\param port, server port.
   *
   *\return reture a new im_connection structure
   *               or NULL if any error
   */
  im_connection *im_proxy_create_conn (im_proxy *proxy,
                                       im_protocol_t protocol,
                                       char *ver,
                                       char *server,
                                       char *port, im_uri_t *user);

  im_connection *im_proxy_get_conn (im_proxy *proxy,
                                    im_protocol_t protocol,
                                    char *ver,
                                    char *server, char *port, im_uri_t *user);

  im_sid_t im_proxy_generate_sid (im_proxy *proxy);

  im_connection *im_proxy_get_conn_by_sid (im_proxy *proxy, im_sid_t sid);

  im_session_t *im_proxy_open_session (im_proxy *proxy, char *client_name,
                                       im_protocol_t protocol, char *ver,
                                       char *server, char *port,
                                       im_uri_t *user);

  bool im_proxy_close_session (im_proxy *proxy, im_sid_t sid);

  im_session_t *im_proxy_get_session (im_proxy *proxy, im_sid_t sid);

  im_trans_mgr_t *im_proxy_get_trains_mgr (im_proxy *proxy, im_sid_t sid);

  /* 
     im_connection*im_proxy_find_conn(
     char*server,
     char*port,
     im_uri_t*user,
     char*passwd);
   */

#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif /* __LIPS_IM_PROXY_H*/

