/*  libim - Library of Instant Messaging and Presence enabler
 * improxy_dbus.cpp - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 */


//
// C++ Implementation: improxy_dbus
//
// Description:
//
//
// Author: ZHAO Liangjing <zhaolj@ch-zlj>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "improxy_dbus.h"
#include "im_proxy.h"
#include "imtype.h"
#include "improxy.h"
#include "impresence.h"
#include "imevent.h"
#include "imtrans.h"
#include "imsession.h"
#include "imconnect.h"
#include "imcontact.h"

#include <glib.h>

void
open_session (DBusMessage *message)
{
  DBusMessage *reply;
  DBusError error;
  im_sid_t sid = 0;

  reply = dbus_message_new_method_return (message);

  char *v_name = NULL;
  im_protocol_t prtl = IM_PROTOCOL_INVALID;
  char *v_server = NULL;
  char *v_port = NULL;
  char *v_ver = NULL;
  char *v_uri = NULL;
  im_err_t rt = IM_ERROR_NONE;

  dbus_error_init (&error);

  dbus_message_get_args (message, &error,
                         DBUS_TYPE_STRING, &v_name,
                         DBUS_TYPE_INT32, &prtl,
                         DBUS_TYPE_STRING, &v_server,
                         DBUS_TYPE_STRING, &v_port,
                         DBUS_TYPE_STRING, &v_ver,
                         DBUS_TYPE_STRING, &v_uri, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("error: %s\n", error.message);
    rt = IM_ERROR_DBUS;
    dbus_error_free (&error);
  }
  else
  {
    dbus_error_free (&error);

    im_uri_t *uri = NULL;

    im_uri_new (v_uri, prtl, &uri);
    im_session_t *s = im_proxy_open_session (im_proxy_get (), v_name,
                                             prtl, v_ver, v_server, v_port,
                                             uri);
    if (s)
    {
      g_warning ("open session sid %d: %s %s:%s %s\n", s->sid,
                 v_name, v_server, v_port, v_uri);
      sid = s->sid;
    }
    else
    {
      g_warning ("open session: %s %s:%s %s failed\n", v_name, v_server, v_port,
                 v_uri);
      rt = IM_ERROR_GENERAL;
    }
  }
  dbus_message_append_args (reply,
                            DBUS_TYPE_UINT32, &sid,
                            DBUS_TYPE_INT32, &rt, DBUS_TYPE_INVALID);

  dbus_connection_send (bus_conn, reply, NULL);
  dbus_message_unref (reply);
};

void
close_session (DBusMessage *message)
{
  DBusMessage *reply = dbus_message_new_method_return (message);
  DBusError error;
  im_sid_t sid = 0;
  im_err_t im_err = IM_ERROR_NONE;

  dbus_error_init (&error);

  dbus_message_get_args (message, &error,
                         DBUS_TYPE_UINT32, &sid, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("error: %s\n", error.message);
  }
  dbus_error_free (&error);

  im_err = im_proxy_close_session (im_proxy_get (), sid) ?
    IM_ERROR_NONE : IM_ERROR_WRONG_SID;

  if (im_err == IM_ERROR_NONE)
  {
    g_message ("closed session %d !\n", sid);
  }
  else
  {
    g_message ("closed session %d failed!\n", sid);
  }

  dbus_message_append_args (reply, DBUS_TYPE_INT32, &im_err, DBUS_TYPE_INVALID);


  dbus_connection_send (bus_conn, reply, NULL);
  dbus_message_unref (reply);
  g_message ("im_err = %d\n", im_err);

};

void
quit_session (DBusMessage *message)
{
  DBusMessage *reply = dbus_message_new_method_return (message);

  dbus_connection_send (bus_conn, reply, NULL);
  dbus_message_unref (reply);
  g_message ("im_proxy stopped!\n");
  g_main_loop_quit (mainloop);
};

void
im_proxy_evt_reg (DBusMessage *message)
{
  DBusMessage *reply;
  DBusError error;

  reply = dbus_message_new_method_return (message);

  im_sid_t sid = 0;
  im_evt_type_t type = IM_EVT_INVALID;
  evt_err_t result = EVT_ERROR_NONE;

  dbus_error_init (&error);

  dbus_message_get_args (message, &error,
                         DBUS_TYPE_UINT32, &sid,
                         DBUS_TYPE_INT32, &type, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("error: %s\n", error.message);
  }

  dbus_error_free (&error);

  g_warning ("sid %d: register event type %d\n", sid, type);
  im_session_t *s = im_proxy_get_session (im_proxy_get (), sid);

  if (s)
  {
    result = im_session_reg_evt (s, type);
  }
  else
  {
    result = EVT_ERROR_WRONG_SID;
  }

  dbus_message_append_args (reply, DBUS_TYPE_INT32, &result, DBUS_TYPE_INVALID);

  dbus_connection_send (bus_conn, reply, NULL);
  dbus_message_unref (reply);
};

void
im_proxy_evt_unreg (DBusMessage *message)
{
  DBusMessage *reply;
  DBusError error;

  reply = dbus_message_new_method_return (message);

  im_sid_t sid = 0;
  im_evt_type_t type = IM_EVT_INVALID;
  evt_err_t result = EVT_ERROR_NONE;

  dbus_error_init (&error);

  dbus_message_get_args (message, &error,
                         DBUS_TYPE_UINT32, &sid,
                         DBUS_TYPE_INT32, &type, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("error: %s\n", error.message);
  }

  dbus_error_free (&error);

  g_warning ("sid %d: unregister event type %d\n", sid, type);
  im_session_t *s = im_proxy_get_session (im_proxy_get (), sid);

  if (s)
  {
    result = im_session_unreg_evt (s, type);
  }
  else
  {
    result = EVT_ERROR_WRONG_SID;
  }

  dbus_message_append_args (reply, DBUS_TYPE_INT32, &result, DBUS_TYPE_INVALID);

  dbus_connection_send (bus_conn, reply, NULL);
  dbus_message_unref (reply);
};

void
im_proxy_req_login (DBusMessage *message)
{
  DBusMessage *reply;
  DBusError error;
  im_err_t err = IM_ERROR_NONE;

  reply = dbus_message_new_method_return (message);

  im_sid_t sid = 0;
  char *passwd = NULL;

  dbus_error_init (&error);

  dbus_message_get_args (message, &error,
                         DBUS_TYPE_UINT32, &sid,
                         DBUS_TYPE_STRING, &passwd, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("error: %s\n", error.message);
  }

  dbus_error_free (&error);

  im_rid_t rid = 0;

  im_session_t *s = im_proxy_get_session (im_proxy_get (), sid);

  if (s)
  {
    err = im_conn_login (im_session_get_conn (s), sid, passwd, &rid);
    if (err == IM_ERROR_NONE)
    {
      g_message ("sid %d: login with password:%s\n", sid, passwd);
    }
    else
    {
      g_message ("sid %d: login with password:%s failed\n", sid, passwd);
    }
  }
  else
  {
    g_message ("sid %d: login req with wrong sid\n", sid);
    err = IM_ERROR_WRONG_SID;
  }
  dbus_message_append_args (reply,
                            DBUS_TYPE_UINT32, &rid,
                            DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);

  dbus_connection_send (bus_conn, reply, NULL);
  dbus_message_unref (reply);
};

void
im_proxy_req_logout (DBusMessage *message)
{
  DBusMessage *reply;
  DBusError error;
  im_err_t err = IM_ERROR_NONE;

  reply = dbus_message_new_method_return (message);

  im_sid_t sid = 0;

  dbus_error_init (&error);

  dbus_message_get_args (message, &error,
                         DBUS_TYPE_UINT32, &sid, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("error: %s\n", error.message);
  }

  dbus_error_free (&error);

  im_rid_t rid = 0;

  im_session_t *s = im_proxy_get_session (im_proxy_get (), sid);

  if (s)
  {
    err = im_conn_logout (im_session_get_conn (s), sid, &rid);
    if (err == IM_ERROR_NONE)
    {
      g_message ("sid %d: logout\n", sid);
    }
    else
    {
      g_message ("sid %d: logout failed\n", sid);
    }
  }
  else
  {
    g_message ("sid %d: logout with wrong sid\n", sid);
    err = IM_ERROR_WRONG_SID;
  }

  dbus_message_append_args (reply,
                            DBUS_TYPE_UINT32, &rid,
                            DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);

  dbus_connection_send (bus_conn, reply, NULL);
  dbus_message_unref (reply);
};

void
im_proxy_req_presence (DBusMessage *message)
{
  DBusMessage *reply;
  DBusError error;
  im_err_t err = IM_ERROR_NONE;

  reply = dbus_message_new_method_return (message);

  im_sid_t sid = 0;
  char *desc = NULL;
  im_presence_state_t status;
  char *uri = NULL;

  dbus_error_init (&error);

  dbus_message_get_args (message, &error,
                         DBUS_TYPE_UINT32, &sid,
                         DBUS_TYPE_STRING, &uri,
                         DBUS_TYPE_UINT32, &status,
                         DBUS_TYPE_STRING, &desc, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("error: %s\n", error.message);
  }

  dbus_error_free (&error);

  im_rid_t rid = 0;

  im_session_t *s = im_proxy_get_session (im_proxy_get (), sid);

  if (s)
  {
    err =
      im_conn_change_presence (im_session_get_conn (s), sid, uri, status, desc,
                               &rid);
    if (err == IM_ERROR_NONE)
    {
      g_message ("sid %d: change presence to %d:%s\n", sid, status, desc);
    }
    else
    {
      g_message ("sid %d: change presence to %d:%s failed\n", sid, status,
                 desc);
    }
  }
  else
  {
    err = IM_ERROR_WRONG_SID;
    g_message ("sid %d: change presence with wrong sid\n", sid);
  }
  dbus_message_append_args (reply,
                            DBUS_TYPE_UINT32, &rid,
                            DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);

  dbus_connection_send (bus_conn, reply, NULL);
  dbus_message_unref (reply);
};

void
im_proxy_req_presence_probe (DBusMessage *message)
{
  DBusMessage *reply;
  DBusError error;
  im_err_t err = IM_ERROR_NONE;

  reply = dbus_message_new_method_return (message);

  im_sid_t sid = 0;
  char *uri = NULL;

  dbus_error_init (&error);

  dbus_message_get_args (message, &error,
                         DBUS_TYPE_UINT32, &sid,
                         DBUS_TYPE_STRING, &uri, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("error: %s\n", error.message);
  }

  dbus_error_free (&error);

  im_rid_t rid = 0;

  im_session_t *s = im_proxy_get_session (im_proxy_get (), sid);

  if (s)
  {
    err = im_conn_presence_probe (im_session_get_conn (s), sid, uri, &rid);
    //err = im_conn_change_presence(im_session_get_conn(s), sid, uri, status, desc, &rid);
    if (err == IM_ERROR_NONE)
    {
      g_message ("sid %d: presence probe to %s\n", sid, uri);
    }
    else
    {
      g_message ("sid %d: presence probe to %s failed\n", sid, uri);
    }
  }
  else
  {
    err = IM_ERROR_WRONG_SID;
    g_message ("sid %d: change presence with wrong sid\n", sid);
  }
  dbus_message_append_args (reply,
                            DBUS_TYPE_UINT32, &rid,
                            DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);

  dbus_connection_send (bus_conn, reply, NULL);
  dbus_message_unref (reply);
};

void
im_proxy_req_msg (DBusMessage *message)
{
  DBusMessage *reply;
  DBusError error;
  im_err_t err = IM_ERROR_NONE;
  im_sid_t sid = 0;
  char *to = NULL;
  char *msg = NULL;
  char *encode = NULL;
  dbus_uint32_t body_len = 0;
  im_msg_type_t mtype;
  im_msg_ctype_t mctype;
  im_rid_t rid = 0;

  reply = dbus_message_new_method_return (message);

  dbus_error_init (&error);

  dbus_message_get_args (message, &error,
                         DBUS_TYPE_UINT32, &sid,
                         DBUS_TYPE_STRING, &to,
                         DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, &msg, &body_len,
                         DBUS_TYPE_STRING, &encode,
                         DBUS_TYPE_INT32, &mtype,
                         DBUS_TYPE_INT32, &mctype, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("error: %s\n", error.message);
    err = IM_ERROR_DBUS;
  }
  else
  {
    im_session_t *s = im_proxy_get_session (im_proxy_get (), sid);

    if (s)
    {
      err =
        im_conn_send_msg (im_session_get_conn (s), sid, to, msg, body_len,
                          encode, mtype, mctype, &rid);
      if (err == IM_ERROR_NONE)
      {
        g_message ("sid %d: send msg to %s:%s\n", sid, to, msg);
      }
      else
      {
        g_message ("sid %d: send msg to %s:%s failed\n", sid, to, msg);
      }
    }
    else
    {
      err = IM_ERROR_WRONG_SID;
      g_message ("sid %d: send msg with wrong\n", sid);
    }
  }

  dbus_error_free (&error);

  dbus_message_append_args (reply,
                            DBUS_TYPE_UINT32, &rid,
                            DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);
  dbus_connection_send (bus_conn, reply, NULL);
  dbus_message_unref (reply);
};

void
im_proxy_req_subscribe (DBusMessage *message)
{
  DBusMessage *reply;
  DBusError error;
  im_err_t err = IM_ERROR_NONE;

  reply = dbus_message_new_method_return (message);

  im_sid_t sid = 0;
  char *uri = NULL;
  char *msg = NULL;
  char *nick = NULL;
  char *group = NULL;
  bool subscribe = FALSE;

  dbus_error_init (&error);

  dbus_message_get_args (message, &error,
                         DBUS_TYPE_UINT32, &sid,
                         DBUS_TYPE_STRING, &uri,
                         DBUS_TYPE_BOOLEAN, &subscribe,
                         DBUS_TYPE_STRING, &msg,
                         DBUS_TYPE_STRING, &nick,
                         DBUS_TYPE_STRING, &group, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("error: %s\n", error.message);
  }

  dbus_error_free (&error);

  im_rid_t rid = 0;

  im_session_t *s = im_proxy_get_session (im_proxy_get (), sid);

  if (s)
  {
    err =
      im_conn_subscribe (im_session_get_conn (s), sid, uri, msg, subscribe,
                         nick, group, &rid);
    if (err == IM_ERROR_NONE)
    {
      g_message ("sid %d: send subscribe msg to %s:%s\n", sid, uri, msg);
    }
    else
    {
      g_message ("sid %d: send subscribe msg to %s:%s failed\n", sid, uri, msg);
    }
  }
  else
  {
    err = IM_ERROR_WRONG_SID;
    g_message ("sid %d: send msg with wrong\n", sid);
  }
  dbus_message_append_args (reply,
                            DBUS_TYPE_UINT32, &rid,
                            DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);

  dbus_connection_send (bus_conn, reply, NULL);
  dbus_message_unref (reply);
};

void
im_proxy_req_accept (DBusMessage *message)
{
  DBusMessage *reply;
  DBusError error;
  im_err_t err = IM_ERROR_NONE;

  reply = dbus_message_new_method_return (message);

  im_sid_t sid = 0;
  char *to = NULL;
  char *msg = NULL;
  bool accept = FALSE;

  dbus_error_init (&error);

  dbus_message_get_args (message, &error,
                         DBUS_TYPE_UINT32, &sid,
                         DBUS_TYPE_STRING, &to,
                         DBUS_TYPE_BOOLEAN, &accept,
                         DBUS_TYPE_STRING, &msg, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("error: %s\n", error.message);
  }

  dbus_error_free (&error);

  im_rid_t rid = 0;

  im_session_t *s = im_proxy_get_session (im_proxy_get (), sid);

  if (s)
  {
    err = im_conn_accept (im_session_get_conn (s), sid, to, msg, accept, &rid);
    if (err == IM_ERROR_NONE)
    {
      g_message ("sid %d: send acceptance msg to %s:%s\n", sid, to, msg);
    }
    else
    {
      g_message ("sid %d: send acceptance msg to %s:%s failed\n", sid, to, msg);
    }
  }
  else
  {
    err = IM_ERROR_WRONG_SID;
    g_message ("sid %d: send msg with wrong\n", sid);
  }
  dbus_message_append_args (reply,
                            DBUS_TYPE_UINT32, &rid,
                            DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);

  dbus_connection_send (bus_conn, reply, NULL);
  dbus_message_unref (reply);
};

void
im_proxy_req_roster (DBusMessage *message)
{
  DBusMessage *reply;
  DBusError error;
  im_err_t err = IM_ERROR_NONE;

  reply = dbus_message_new_method_return (message);

  im_sid_t sid = 0;

  dbus_error_init (&error);

  dbus_message_get_args (message, &error,
                         DBUS_TYPE_UINT32, &sid, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("error: %s\n", error.message);
  }

  dbus_error_free (&error);

  im_rid_t rid = 0;

  im_session_t *s = im_proxy_get_session (im_proxy_get (), sid);

  if (s)
  {
    err = im_conn_get_rosters (im_session_get_conn (s), sid, &rid);
    if (err == IM_ERROR_NONE)
    {
      g_message ("sid %d: request rosters\n", sid);
    }
    else
    {
      g_message ("sid %d: request rosters failed\n", sid);
    }
  }
  else
  {
    err = IM_ERROR_WRONG_SID;
    g_message ("sid %d: send msg with wrong\n", sid);
  }
  dbus_message_append_args (reply,
                            DBUS_TYPE_UINT32, &rid,
                            DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);

  dbus_connection_send (bus_conn, reply, NULL);
  dbus_message_unref (reply);
};

void
im_proxy_req_buddy_op (DBusMessage *message)
{
  DBusMessage *reply;
  DBusError error;
  im_err_t err = IM_ERROR_NONE;

  reply = dbus_message_new_method_return (message);

  im_sid_t sid = 0;
  char *contactor = NULL;
  char *nickname = NULL;
  char *group = NULL;
  im_buddy_operation_t op_type;

  dbus_error_init (&error);

  dbus_message_get_args (message, &error,
                         DBUS_TYPE_UINT32, &sid,
                         DBUS_TYPE_STRING, &contactor,
                         DBUS_TYPE_STRING, &nickname,
                         DBUS_TYPE_STRING, &group,
                         DBUS_TYPE_INT32, &op_type, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("error: %s\n", error.message);
  }

  dbus_error_free (&error);

  im_rid_t rid = 0;

  im_session_t *s = im_proxy_get_session (im_proxy_get (), sid);

  if (s)
  {
    err =
      im_conn_buddy_op (im_session_get_conn (s), sid, contactor, nickname,
                        group, op_type, &rid);
    if (err == IM_ERROR_NONE)
    {
      g_message ("sid %d: request rosters\n", sid);
    }
    else
    {
      g_message ("sid %d: request rosters failed\n", sid);
    }
  }
  else
  {
    err = IM_ERROR_WRONG_SID;
    g_message ("sid %d: send msg with wrong\n", sid);
  }
  dbus_message_append_args (reply,
                            DBUS_TYPE_UINT32, &rid,
                            DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);

  dbus_connection_send (bus_conn, reply, NULL);
  dbus_message_unref (reply);
};

void
im_proxy_req_synch (DBusMessage *message)
{
  DBusMessage *reply;
  DBusError error;
  im_err_t err = IM_ERROR_NONE;

  reply = dbus_message_new_method_return (message);

  im_sid_t sid = 0;

  dbus_error_init (&error);

  dbus_message_get_args (message, &error,
                         DBUS_TYPE_UINT32, &sid, DBUS_TYPE_INVALID);

  if (dbus_error_is_set (&error))
  {
    g_warning ("error: %s\n", error.message);
  }

  dbus_error_free (&error);

  im_rid_t rid = 0;

  im_session_t *s = im_proxy_get_session (im_proxy_get (), sid);

  if (s)
  {
    err = im_conn_buddies_synch (im_session_get_conn (s), sid, &rid);
    if (err == IM_ERROR_NONE)
    {
      g_message ("sid %d: request rosters\n", sid);
    }
    else
    {
      g_message ("sid %d: request rosters failed\n", sid);
    }
  }
  else
  {
    err = IM_ERROR_WRONG_SID;
    g_message ("sid %d: send msg with wrong\n", sid);
  }
  dbus_message_append_args (reply,
                            DBUS_TYPE_UINT32, &rid,
                            DBUS_TYPE_INT32, &err, DBUS_TYPE_INVALID);

  dbus_connection_send (bus_conn, reply, NULL);
  dbus_message_unref (reply);
};

