/*  libim - Library of Instant Messaging and Presence enabler
 * improxy_loop.c - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 */


/**
 *@file API implememt of the Instance Message Interface - IM Proxy
 *
 *@version 0.1
 */

#include "improxy_loop.h"
#include "imsession.h"

G_LOCK_DEFINE_STATIC (im_source);

static GSource *im_source = NULL;

static GAsyncQueue *qTrans = NULL;

static GThread *mainThread = NULL;

static gboolean
im_source_prepare (GSource *source, gint *timeout_)
{
  *timeout_ = -1;
  return FALSE;
};

static gboolean
im_source_check (GSource *source)
{
  return g_async_queue_length (qTrans) > 0;
};

static gboolean
im_source_dispatch (GSource *source, GSourceFunc callback, gpointer user_data)
{
  im_trans_t *t = NULL;

  while ((t = (im_trans_t *) g_async_queue_try_pop (qTrans)) != NULL)
  {
    g_message ("dispatch event %d\n", t->type);
    im_session_send_evt (t);
    im_trans_free (t);
    if (callback)
      callback (user_data);
  }
  return TRUE;
};

static void
im_source_finalize (GSource *source)
{
  g_async_queue_unref (qTrans);
};

static GSourceFuncs im_source_funcs = {
  im_source_prepare,
  im_source_check,
  im_source_dispatch,
  im_source_finalize,
  0,
  0
};

GSource *
im_recv_src (void)
{
  if (!im_source)
  {
    G_LOCK (im_source);
    if (!im_source)
    {
      qTrans = g_async_queue_new ();
      im_source = g_source_new (&im_source_funcs, sizeof (GSource));
      mainThread = g_thread_self ();
    }
    G_UNLOCK (im_source);
  }
  return im_source;
};

bool
im_proxy_trans_notify (im_trans_t *t)
{
  if (mainThread != g_thread_self ())
  {
    g_message ("push some thing in to q...\n");
    g_async_queue_push (qTrans, (gpointer) t);
    g_main_context_wakeup (NULL);
  }
  else
  {
    g_message ("call sendevt directly...\n");
    im_session_send_evt (t);
    im_trans_free (t);
  }
  return TRUE;
};

