/*  libim - Library of Instant Messaging and Presence enabler
 * imsession.c - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 */


/**
 *@file API implememt of the Instance Message Interface - IM Session
 *
 *@version 0.1
 */

#include "imsession.h"
#include "imtype.h"
#include <glib.h>
#include "imtrans.h"
#include "imconnect.h"
#include "imcontact.h"
#include "im_dbus.h"
#include "improxy.h"
#include <event.h>

im_list *bad_sessions = NULL;

//static GList*close_session_list = NULL;

im_session_t *
im_session_new (im_sid_t sid, char *client_name)
{
  im_session_t *s = g_new0 (im_session_t, 1);

  s->client_name = g_strdup (client_name);
  s->sid = sid;
  s->evt_type_list = NULL;
  return s;
};

void
im_session_free (im_session_t *session)
{
  g_assert (session);
  g_free (session->client_name);
  if (session->conn)
    im_conn_unref (session->conn);
  if (session->evt_type_list)
  {
    g_list_free ((GList *) session->evt_type_list);
  }
  g_free (session);
};


im_sid_t
im_session_get_sid (im_session_t *session)
{
  return session->sid;
};

char *
im_session_get_client_name (im_session_t *session)
{
  return g_strdup (session->client_name);
};

im_connection *
im_session_get_conn (im_session_t *session)
{
  return session->conn;
};

im_err_t
im_session_reg_evt (im_session_t *session, im_evt_type_t type)
{
  if (g_list_find ((GList *) session->evt_type_list, GINT_TO_POINTER (type)))
    return EVT_ERROR_DUPLICATED_REG;
  session->evt_type_list =
    (im_list *) g_list_append ((GList *) session->evt_type_list,
                               GINT_TO_POINTER (type));
  return IM_ERROR_NONE;
};

im_err_t
im_session_unreg_evt (im_session_t *session, im_evt_type_t type)
{
  session->evt_type_list =
    (im_list *) g_list_remove ((GList *) session->evt_type_list,
                               GINT_TO_POINTER (type));
  return IM_ERROR_NONE;
};

bool
im_session_is_registered (im_session_t *session, im_evt_type_t type)
{
  return (g_list_find ((GList *) session->evt_type_list, GINT_TO_POINTER (type))
          != NULL);
};

bool
im_session_build_roster_dmsg (im_sid_t sid, im_trans_t *t,
                              DBusMessageIter *pIter)
{
  im_roster_t *roster = (im_roster_t *) (t->out);
  char *snickname = NULL;
  unsigned int length = 0;
  im_roster_item_t *ri = NULL;

  if (im_roster_get_length (roster, &length) != IM_ERROR_NONE)
    return false;
  unsigned int i = 0;

  for (; i < length; i++)
  {
    if (im_roster_get_item (roster, i, &ri) != IM_ERROR_NONE)
      return false;
    snickname = (ri->nickname) ? (ri->nickname) : "";
    g_print ("+++++++++contactor=%s\n", ri->contactor);
    g_print ("+++++++++nick=%s\n", snickname);
    g_print ("+++++++++subs_state=%s\n", im_subs_state_2_str (ri->subs_state));
    g_print ("+++++++++group=%s\n", ri->group);
    if (!dbus_message_iter_append_basic
        (pIter, DBUS_TYPE_STRING, &(ri->contactor))
        || !dbus_message_iter_append_basic (pIter, DBUS_TYPE_STRING, &snickname)
        || !dbus_message_iter_append_basic (pIter, DBUS_TYPE_INT32,
                                            &(ri->subs_state))
        || !dbus_message_iter_append_basic (pIter, DBUS_TYPE_STRING,
                                            &(ri->group)))
    {
      g_warning ("sid=%d evt_type=%d no enough memory for dmsg\n", sid,
                 t->type);
      return false;
    }
  }
  return true;
};

bool
im_session_build_presence_dmsg (im_sid_t sid, im_trans_t *t,
                                DBusMessageIter *pIter)
{
  im_presence_param_t *p = (im_presence_param_t *) (t->out);

  if (!p)
    p = (im_presence_param_t *) (t->in);        //IM_REQ_PRESENCE event.
  const char *sdesc = (p->desc) ? (p->desc) : "";
  bool rt = true;

  g_print ("+++++++++uri=%s\n", p->uri);
  g_print ("+++++++++pstate=%s\n", im_presence_2_str (p->pstate));
  g_print ("+++++++++desc=%s\n", sdesc);
  g_print ("+++++++++protocol=%d\n", p->prtl);
  if (!dbus_message_iter_append_basic (pIter, DBUS_TYPE_STRING, &(p->uri)) ||
      !dbus_message_iter_append_basic (pIter, DBUS_TYPE_INT32, &(p->pstate)) ||
      !dbus_message_iter_append_basic (pIter, DBUS_TYPE_STRING, &sdesc) ||
      !dbus_message_iter_append_basic (pIter, DBUS_TYPE_INT32, &(p->prtl)))
  {
    g_warning ("sid=%d evt_type=%d no enough memory for dmsg\n", sid, t->type);
    rt = false;
  }

  return rt;
};

bool
im_session_build_rmsg_dmsg (im_sid_t sid, im_trans_t *t,
                            DBusMessageIter *pIter)
{
  im_msg_param_t *msg = (im_msg_param_t *) t->out;
  char *blank = "";
  DBusMessageIter array;
  bool rt = true;
  char buf[2];

  switch (msg->msg_ctype)
  {
    case IM_MSG_CTYPE_TEXT:
      buf[0] = DBUS_TYPE_BYTE;
      buf[1] = '\0';
      g_print ("+++++++++uri=%s\n", (msg->uri) ? (msg->uri) : blank);
      g_print ("+++++++++msg=%s\n", (char *) ((msg->msg) ? (msg->msg) : blank));
      g_print ("+++++++++encode=%s\n", (msg->encode) ? (msg->encode) : blank);
      if (!dbus_message_iter_append_basic
          (pIter, DBUS_TYPE_STRING, (msg->uri) ? &(msg->uri) : &blank)
          || !dbus_message_iter_open_container (pIter, DBUS_TYPE_ARRAY, buf,
                                                &array)
          || !dbus_message_iter_append_fixed_array (&array, DBUS_TYPE_BYTE,
                                                    &(msg->msg), msg->msg_size)
          || !dbus_message_iter_close_container (pIter, &array)
          || !dbus_message_iter_append_basic (pIter, DBUS_TYPE_STRING,
                                              (msg->encode) ? &(msg->
                                                                encode) :
                                              &blank)
          || !dbus_message_iter_append_basic (pIter, DBUS_TYPE_UINT32,
                                              &(msg->msg_id))
          || !dbus_message_iter_append_basic (pIter, DBUS_TYPE_UINT32,
                                              &(msg->msg_time))
          || !dbus_message_iter_append_basic (pIter, DBUS_TYPE_INT32,
                                              &(msg->msg_type))
          || !dbus_message_iter_append_basic (pIter, DBUS_TYPE_INT32,
                                              &(msg->msg_ctype))
          || !dbus_message_iter_append_basic (pIter, DBUS_TYPE_INT32,
                                              &(msg->msg_state)))
      {
        g_warning ("sid=%d evt_type=%d no enough memory for dmsg\n", sid,
                   t->type);
        rt = false;
      }
      break;
    default:
      rt = false;
      break;
  }

  return rt;
};

bool
im_session_build_subscribe_dmsg (im_sid_t sid, im_trans_t *t,
                                 DBusMessageIter *pIter)
{
  im_subs_msg_t *msg = (im_subs_msg_t *) t->out;
  const char *smsg = (msg->msg) ? (msg->msg) : "";

  g_print ("+++++++++contactor=%s\n", msg->contactor);
  g_print ("+++++++++msg=%s\n", smsg);
  if (!dbus_message_iter_append_basic
      (pIter, DBUS_TYPE_STRING, &(msg->contactor))
      || !dbus_message_iter_append_basic (pIter, DBUS_TYPE_STRING, &smsg)
      || !dbus_message_iter_append_basic (pIter, DBUS_TYPE_BOOLEAN,
                                          &(msg->subscribe)))
  {
    g_warning ("sid=%d evt_type=%d no enough memory for dmsg\n", sid, t->type);
    return false;
  }
  return true;
};

bool
im_session_build_accept_dmsg (im_sid_t sid, im_trans_t *t,
                              DBusMessageIter *pIter)
{
  im_accept_msg_t *msg = (im_accept_msg_t *) t->out;
  const char *smsg = (msg->msg) ? (msg->msg) : "";

  g_print ("+++++++++contactor=%s\n", msg->contactor);
  g_print ("+++++++++msg=%s\n", smsg);
  if (!dbus_message_iter_append_basic
      (pIter, DBUS_TYPE_STRING, &(msg->contactor))
      || !dbus_message_iter_append_basic (pIter, DBUS_TYPE_STRING, &smsg)
      || !dbus_message_iter_append_basic (pIter, DBUS_TYPE_BOOLEAN,
                                          &(msg->accept)))
  {
    g_warning ("sid=%d evt_type=%d no enough memory for dmsg\n", sid, t->type);
    return false;
  }
  return true;
};

bool
im_session_build_err_dmsg (im_sid_t sid, im_trans_t *t,
                           DBusMessageIter *pIter)
{
  im_network_error_t *msg = (im_network_error_t *) t->out;
  const char *smsg = (msg->msg) ? (msg->msg) : "";

  g_print ("+++++++++msg=%s\n", smsg);
  if (!dbus_message_iter_append_basic (pIter, DBUS_TYPE_INT32, &(msg->err_type))
      || !dbus_message_iter_append_basic (pIter, DBUS_TYPE_STRING, &smsg))
  {
    g_warning ("sid=%d evt_type=%d no enough memory for dmsg\n", sid, t->type);
    return false;
  }
  return true;
};

bool
im_session_build_buddy_op_dmsg (im_sid_t sid, im_trans_t *t,
                                DBusMessageIter *pIter)
{
  im_buddy_op_t *op = (im_buddy_op_t *) t->out;
  im_roster_item_t *buddy = NULL;
  im_buddy_operation_t op_type;

  if (im_buddy_op_get_buddy (op, &buddy) != IM_ERROR_NONE)
    return false;
  if (im_buddy_op_get_operation (op, &op_type) != IM_ERROR_NONE)
    return false;
  char *blank = "";
  char *uri = buddy->contactor ? buddy->contactor : blank;
  char *nick = buddy->nickname ? buddy->nickname : blank;
  char *group = buddy->group ? buddy->group : blank;

  bool rt = true;

  g_print ("+++++++++uri=%s\n", uri);
  g_print ("+++++++++nick=%s\n", nick);
  g_print ("+++++++++group=%s\n", group);
  if (!dbus_message_iter_append_basic (pIter, DBUS_TYPE_STRING, &uri) ||
      !dbus_message_iter_append_basic (pIter, DBUS_TYPE_STRING, &nick) ||
      !dbus_message_iter_append_basic (pIter, DBUS_TYPE_INT32,
                                       &buddy->subs_state)
      || !dbus_message_iter_append_basic (pIter, DBUS_TYPE_STRING, &group)
      || !dbus_message_iter_append_basic (pIter, DBUS_TYPE_INT32, &op_type))
  {
    g_warning ("sid=%d evt_type=%d no enough memory for dmsg\n", sid, t->type);
    rt = false;
  }

  return rt;
};

static im_err_t
im_session_reply_req_fun (im_trans_t *t)
{
  g_assert (t->session);
  im_sid_t sid = im_session_get_sid (t->session);
  req_state_t state = im_trans_req_state (t);
  char *cname = im_session_get_client_name (t->session);

  g_print ("cname = %s\n", cname);

  bool success = FALSE;
  DBusMessageIter iter;
  DBusMessage *message = dbus_message_new_method_call (cname,
                                                       IM_CLIENT_OBJECT,
                                                       IM_CLIENT_INTERFACE,
                                                       IM_CLIENT_MSG_EVT_IN);

  g_print ("+++++++sid=%d evt_type=%s\n", sid, im_evt_type_2_str (t->type));
  g_print ("+++++++rid=%d err_code=%d\n", t->rid, t->err_code);
  dbus_message_iter_init_append (message, &iter);
  /// append evt_type, sid, rid, err_code, state & others
  if (!dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &sid) ||
      !dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT32, &(t->type)) ||
      !dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &(t->rid)) ||
      !dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT32, &(t->err_code))
      || !dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT32, &state))
  {
    g_warning ("sid=%d evt_type=%d no enough memory for dmsg\n", sid, t->type);
    dbus_message_unref (message);
    g_free (cname);
    return IM_ERROR_NOMEM;
  }

  if ((t->type == IM_REQ_PRESENCE || t->type == IM_REQ_PRESENCE_PROBE) &&
      !im_session_build_presence_dmsg (sid, t, &iter))
  {
    dbus_message_unref (message);
    g_free (cname);
    return IM_ERROR_NOMEM;
  }

  success = dbus_connection_send (bus_conn, message, NULL);
  dbus_message_unref (message);
  g_free (cname);
  ///if error, return an IM_ERROR_NOMEM error.
  if (!success)
  {
    return IM_ERROR_NOMEM;
  }
  return IM_ERROR_NONE;
};

static void
im_session_send_evt_fun (im_sid_t sid, im_session_t *s, im_trans_t *t)
{
  if ((t->session != s) && (t->conn == s->conn) &&      /// not the same session but same connection
      im_session_is_registered (s, t->type))    /// evt_type is registered
  {
    char *cname = im_session_get_client_name (s);

    g_print ("cname = %s\n", cname);

    bool success = FALSE;
    DBusMessageIter iter;
    DBusMessage *message = dbus_message_new_method_call (cname,
                                                         IM_CLIENT_OBJECT,
                                                         IM_CLIENT_INTERFACE,
                                                         IM_CLIENT_MSG_EVT_IN);

    dbus_message_iter_init_append (message, &iter);
    /// append evt_type & sid
    g_print ("+++++++sid=%d evt_type=%s\n", sid, im_evt_type_2_str (t->type));
    if (!dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &sid) ||
        !dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT32, &(t->type)))
    {
      g_warning ("sid=%d evt_type=%d no enough memory for dmsg\n", sid,
                 t->type);
      dbus_message_unref (message);
      g_free (cname);
      return;
    }

    switch (t->type)
    {
      case IM_REQ_LOGIN:
      case IM_REQ_LOGOUT:
      case IM_REQ_SUBSCRIBE:
      case IM_REQ_ACCEPTANCE:
      case IM_REQ_MESSAGE:
      case IM_REQ_ROSTER:
      case IM_REQ_PRESENCE:
      case IM_REQ_BUDDY:
      case IM_REQ_SYNCH:
        dbus_message_unref (message);
        g_free (cname);
        return;
        break;
      case IM_EVT_ROSTER:
        im_session_build_roster_dmsg (sid, t, &iter);
        break;
      case IM_EVT_PRESENCE:
        im_session_build_presence_dmsg (sid, t, &iter);
        break;
      case IM_EVT_MESSAGE:
        im_session_build_rmsg_dmsg (sid, t, &iter);
        break;
      case IM_EVT_SUBSCRIBE:
        im_session_build_subscribe_dmsg (sid, t, &iter);
        break;
      case IM_EVT_ACCEPTANCE:
        im_session_build_accept_dmsg (sid, t, &iter);
        break;
      case IM_EVT_ERROR:
        im_session_build_err_dmsg (sid, t, &iter);
        break;
      case IM_EVT_BUDDY:
        im_session_build_buddy_op_dmsg (sid, t, &iter);
        break;
      case IM_EVT_LOGIN:
      case IM_EVT_LOGOUT:
        break;
      default:
        break;
    }

    success = dbus_connection_send (bus_conn, message, NULL);
    dbus_connection_flush (bus_conn);
    dbus_message_unref (message);
    g_free (cname);
    ///if error, the session should be closed!
    //if (!success)
    //{
    //g_warning ("Failed to send message: %s\n", error.message);
    //dbus_error_free(&error);
    //* ((GList**)close_session_list) = g_list_append(*((GList**)close_session_list), sid);
    //return;
    //}

    //dbus_error_free(&error);
    //dbus_message_unref (reply);

    //g_free(cname);
  }
};

/* 
static void closed_abnormal_session(im_sid_t sid, gpointer data)
{
  g_warning("close no reaction session %d...\n", sid);
  im_proxy_close_session(im_proxy_get(), sid);
}*/

void
im_session_send_evt (im_trans_t *t)
{
  bool broadcast = false;

  if (t->rid > 0)
  {                             ///is an asynchronous event
    im_session_reply_req_fun (t);
    /* if( == IM_ERROR_DBUS)
       {
       /// DBUS error, should remove the bad session.
       im_proxy_close_session(im_proxy_get(), im_session_get_sid(t->session));
       }
     */
    /// if type is one of the IM_REQ_LOGIN, IM_REQ_LOGOUT or IM_REQ_PRESENCE
    /// and the status is IM_TRANS_OK (be completed),
    /// the event need to be broadcast.
    switch (t->type)
    {
      case IM_REQ_LOGIN:
        if (t->status == IM_TRANS_OK)
        {
          t->type = IM_EVT_LOGIN;
          broadcast = true;
          t->conn = im_session_get_conn (t->session);
        }
        break;
      case IM_REQ_LOGOUT:
        if (t->status == IM_TRANS_OK)
        {
          t->type = IM_EVT_LOGOUT;
          broadcast = true;
          t->conn = im_session_get_conn (t->session);
        }
        break;
      case IM_REQ_PRESENCE:
        if (t->status == IM_TRANS_OK)
        {
          t->type = IM_EVT_PRESENCE;
          broadcast = true;
          t->conn = im_session_get_conn (t->session);
        }
        break;
      default:
        break;
    };
  }
  if (t->rid == IM_TRANS_RID_EVT || broadcast)
  {                             /// is an unsolicited event or asynchronous event needed to be broadcasted.
    //close_session_list = NULL;
    g_hash_table_foreach ((GHashTable *) im_proxy_get ()->s_hs,
                          (GHFunc) im_session_send_evt_fun, t);
    //g_list_foreach(close_session_list, (GFunc*)closed_abnormal_session, NULL);
    //g_list_free(close_session_list);
  }
};

