/*  libim - Library of Instant Messaging and Presence enabler
 * imtrans.h - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 */


/**
 *@file API definition of the Instance Message Interface - IM Transaction
 *
 *@version 0.1
 */

#ifndef __LIPS_IM_TRANS_H
#define __LIPS_IM_TRANS_H

#include <stdint.h>
#include "imtype.h"
#include "imsession.h"
#include "impresence.h"
#include "immsg.h"

#ifdef __cplusplus
extern "C"
{
#endif                          /* __cplusplus */

  //typedef uint16_t im_trans_status_t;

  typedef enum
  {
    IM_TRANS_INIT,
    IM_TRANS_PENDING,
    IM_TRANS_CANCEL,
    IM_TRANS_OK,
    IM_TRANS_FAIL,
    IM_TRANS_TIMEOUT,
    IM_TRANS_NOSUPPORT
  } im_trans_status_t;

  typedef uint32_t im_trans_type_t;

  typedef char *im_tid_t;

  //typedef struct _im_trans_t im_trans_t;

  struct _im_trans_t
  {
    im_session_t *session;
    const im_connection *conn;
    im_tid_t tid;
    im_rid_t rid;
    im_trans_status_t status;
    im_trans_type_t type;
    im_err_code_t err_code;
    void *in;
    void *out;
    void (*cb) (im_trans_t *t);        //callback method when transaction is finished.
  };

  typedef struct
  {
    char *uri;
    im_presence_state_t pstate;
    im_protocol_t prtl;
    char *desc;
  }
  im_presence_param_t;

  void im_presence_param_free (im_presence_param_t *p);

  //   typedef struct {
  //     char *uri;
  //     void *msg;
  //     size_t msg_size;
  //     im_msg_id_t msg_id;
  //     time_t msg_time;
  //     char *encode;
  //     im_msg_type_t msg_type;
  //     im_msg_ctype_t msg_ctype;
  //     im_msg_state_t msg_state;
  //   } _im_msg_param_t;

  void im_msg_param_free (im_msg_param_t *msg);

  im_trans_t *im_trans_new (im_session_t *session, im_rid_t rid,
                            im_trans_type_t type, void *in_data);

  im_trans_t *im_trans_new_with_sid (im_sid_t sid, im_rid_t rid,
                                     im_trans_type_t type, void *in_data);

  void im_trans_free (im_trans_t *trans);

  im_err_t im_trans_cancel (im_trans_t *trans);

  req_state_t im_trans_req_state (im_trans_t *trans);

  /**
   * use to mange all the transactions related to the same connection
   */
  //typedef struct _im_trans_mgr_t im_trans_mgr_t;

  im_trans_mgr_t *im_trans_mgr_new ();

  void im_trans_mgr_free (im_trans_mgr_t *t_mgr);

  im_trans_t *im_trans_mgr_add (im_trans_mgr_t *t_mgr, im_session_t *session,
                                im_trans_type_t type, void *in_data);

  im_trans_t *im_trans_mgr_add_with_sid (im_trans_mgr_t *t_mgr, im_sid_t sid,
                                         im_trans_type_t type, void *in_data);

  im_trans_t *im_trans_mgr_get_by_tid (im_trans_mgr_t *t_mgr, im_tid_t tid);

  im_trans_t *im_trans_mgr_get_by_rid (im_trans_mgr_t *t_mgr, im_rid_t rid);

  im_trans_t *im_trans_mgr_remove_by_tid (im_trans_mgr_t *t_mgr, im_tid_t tid);

  im_trans_t *im_trans_mgr_remove_by_rid (im_trans_mgr_t *t_mgr, im_rid_t rid);

  bool im_trans_mgr_has_tid (im_trans_mgr_t *t_mgr, im_tid_t tid);

  bool im_trans_mgr_has_rid (im_trans_mgr_t *t_mgr, im_rid_t rid);

  uint32_t im_trans_mgr_len (im_trans_mgr_t *t_mgr);

  im_trans_t *im_trans_mgr_push (im_trans_mgr_t *t_mgr, im_trans_t *t);

  im_trans_t *im_trans_mgr_try_pop (im_trans_mgr_t *t_mgr);

  bool im_trans_equalto (const im_trans_t *me, const im_trans_t *other);

#ifdef __cplusplus
}
#endif                          /* __cplusplus */

#endif /* __LIPS_IM_TRANS_H_*/

