/*  libim - Library of Instant Messaging and Presence enabler
 * imuri.c - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 * This software and associated documentation files (the "Software") 
 * are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 * All Rights Reserved. 
 *
 * A copyright license is hereby granted for redistribution and use of 
 * the Software in source and binary forms, with or without modification, 
 * provided that the following conditions are met: 
 * - Redistributions of source code must retain the above copyright notice, 
 * this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 * notice, this copyright license and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution. 
 * - Neither the name of LiPS  nor the names of its Members may be used 
 * to endorse or promote products derived from the Software without 
 * specific prior written permission. 
 *
 * A patent license for any Necessary Claims owned by Members of LiPS Forum 
 * to make, have made, use, import, offer to sell, lease and sell or otherwise 
 * distribute any implementation compliant with the any specification adopted 
 * by the LiPS Forumcan be obtained from the respective Members on reasonable 
 * and non-discriminatory terms and conditions and under reciprocity, as 
 * regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 * "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 * AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 * ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 */


/**
 *@file API implememt of the Instance Message Interface - IM URI
 *
 *@version 0.1
 */

#include "imuri_base.h"
#include "im_xmpp_uri.h"
#include "imtype_imp.h"
#include <string.h>
#include <glib.h>

im_err_t
im_uri_new (char *uri_str, im_protocol_t prtl, im_uri_t **uri)
{
  switch (prtl)
  {
    case IM_PROTOCOL_XMPP:
      return im_uri_new_xmpp (uri_str, uri);
      break;
    case IM_PROTOCOL_IMPS:
      break;
    case IM_PROTOCOL_SIMPLE:
      break;
    default:
      break;
  }
  return IM_ERROR_NOSUPPORT;
};

im_err_t
im_uri_init_base (im_uri_t **uri)
{
  *uri = (im_uri_t *) g_new0 (im_uri_t, 1);
  if (!*uri)
    return IM_ERROR_NULL;
  (*uri)->protocol = IM_PROTOCOL_XMPP;
  (*uri)->ext = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
  return ((*uri)->ext) ? IM_ERROR_NONE : IM_ERROR_NULL;
};

im_err_t
im_uri_free_base (im_uri_t *uri)
{
  g_assert (uri);
  if (uri->ext)
    g_hash_table_destroy (uri->ext);
  if (uri->domain)
    g_free (uri->domain);
  if (uri->resource)
    g_free (uri->resource);
  if (uri->uid)
    g_free (uri->uid);
  g_free (uri);
  return IM_ERROR_NONE;
};

bool
im_uri_cmp_base (im_uri_t *first, im_uri_t *second, im_uri_part_t part)
{
  if (first->protocol != second->protocol)
    return FALSE;
  switch (part)
  {
    case IM_URI_SCHEME:
      return TRUE;
      break;
    case IM_URI_USER:
      return strcmp (first->uid, second->uid) == 0;
      break;
    case IM_URI_DOMAIN:
      return strcmp (first->domain, second->domain) == 0;
      break;
    case IM_URI_RESOURCE:
      return strcmp (first->resource, second->resource) == 0;
      break;
    case IM_URI_PARTIAL:
      return (strcmp (first->uid, second->uid) == 0) &&
        (strcmp (first->domain, second->domain) == 0);
      break;
    case IM_URI_FULL:
      return (strcmp (first->uid, second->uid) == 0) &&
        (strcmp (first->resource, second->resource) == 0) &&
        (strcmp (first->domain, second->domain) == 0);
      break;
    default:
      break;
  };
  return FALSE;
};

im_err_t
im_uri_free (im_uri_t *uri)
{
  return uri->free (uri);
};

bool
im_uri_cmp (im_uri_t *first, im_uri_t *second, im_uri_part_t part)
{
  return first->cmp (first, second, part);
};

im_err_t
im_uri_get_ext (im_uri_t *uri, char *name, char **value)
{
  g_assert (uri);
  *value = g_strdup (g_hash_table_lookup (uri->ext, name));
  if (!*value)
    return IM_ERROR_NOSUPPORT;
  return IM_ERROR_NONE;
};

im_err_t
im_uri_set_ext (im_uri_t *uri, char *name, char *value)
{
  g_assert (uri);
  g_hash_table_insert (uri->ext, g_strdup (name), g_strdup (value));
  return IM_ERROR_NONE;
};

im_err_t
im_uri_get_ext_names (im_uri_t *uri, char ***names)
{
  return _im_get_hash_keys (uri->ext, names);
};

im_err_t
im_uri_get_full_ext (im_uri_t *uri, char **str)
{
  g_assert (uri);
  return uri->get_full_ext (uri, str);
};

im_err_t
im_uri_get_uri (im_uri_t *uri, char **std_str)
{
  g_assert (uri);
  return uri->get_uri (uri, std_str);
};

im_err_t
im_uri_get_uri_full (im_uri_t *uri, char **full_str)
{
  g_assert (uri);
  return uri->get_uri_full (uri, full_str);
};

im_err_t
im_uri_get_protocol (im_uri_t *uri, im_protocol_t *prtl)
{
  g_assert (uri);
  *prtl = uri->protocol;
  return IM_ERROR_NONE;
};

im_err_t
im_uri_get_user (im_uri_t *uri, char **uid)
{
  g_assert (uri);
  *uid = g_strdup (uri->uid);
  return (*uid) ? IM_ERROR_NONE : IM_ERROR_NULL;
};

im_err_t
im_uri_set_user (im_uri_t *uri, char *uid)
{
  g_assert (uri);
  uri->uid = g_strdup (uid);
  return (uri->uid) ? IM_ERROR_NONE : IM_ERROR_NULL;
};

im_err_t
im_uri_get_domain (im_uri_t *uri, char **domain)
{
  g_assert (uri);
  *domain = g_strdup (uri->domain);
  return (*domain) ? IM_ERROR_NONE : IM_ERROR_NULL;
};

im_err_t
im_uri_set_domain (im_uri_t *uri, char *domain)
{
  g_assert (uri);
  uri->domain = g_strdup (domain);
  return (uri->domain) ? IM_ERROR_NONE : IM_ERROR_NULL;
};

im_err_t
im_uri_get_res (im_uri_t *uri, char **res)
{
  g_assert (uri);
  *res = g_strdup (uri->resource);
  return (*res) ? IM_ERROR_NONE : IM_ERROR_NULL;
};

im_err_t
im_uri_set_res (im_uri_t *uri, char *res)
{
  g_assert (uri);
  uri->resource = g_strdup (res);
  return (uri->resource) ? IM_ERROR_NONE : IM_ERROR_NULL;
};

