/*  libim - Library of Instant Messaging and Presence enabler
 * im_log.c - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 * This software and associated documentation files (the "Software") 
 * are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 * All Rights Reserved. 
 *
 * A copyright license is hereby granted for redistribution and use of 
 * the Software in source and binary forms, with or without modification, 
 * provided that the following conditions are met: 
 * - Redistributions of source code must retain the above copyright notice, 
 * this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 * notice, this copyright license and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution. 
 * - Neither the name of LiPS  nor the names of its Members may be used 
 * to endorse or promote products derived from the Software without 
 * specific prior written permission. 
 *
 * A patent license for any Necessary Claims owned by Members of LiPS Forum 
 * to make, have made, use, import, offer to sell, lease and sell or otherwise 
 * distribute any implementation compliant with the any specification adopted 
 * by the LiPS Forumcan be obtained from the respective Members on reasonable 
 * and non-discriminatory terms and conditions and under reciprocity, as 
 * regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 * "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 * AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 * ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 */


/***************************************************************************
 *           dbus_log.c
 *
 * Mon May 29 14:41:46 2006
 * Copyright  2006  ZHAO Liangjing
 * liangjing.zhao@francetelecom.com
 ****************************************************************************/

#include "log/im_log.h"
#include <string.h>
#include <time.h>

static GIOChannel *log_channel = NULL;

GLogFunc pre_log_func = NULL;

void
open_log_file (gchar *filename)
{
  GError *error = NULL;

  log_channel = g_io_channel_new_file (filename, "a+", &error);
  if (error != NULL)
  {
    g_error ("Error in open log file: %s\n", error->message);
    log_channel = NULL;
    return;
  }

  gsize w_size = 0;
  GIOStatus status = G_IO_STATUS_NORMAL;
  gchar *log = "\n\n";
  gsize len = strlen (log);
  gchar *p_msg = log;

  while (len > 0)
  {
    status = g_io_channel_write_chars (log_channel,
                                       p_msg, len, &w_size, &error);

    if (status != G_IO_STATUS_NORMAL)
    {
      g_print ("Error while writing to file: %s\n", error->message);
      break;
    }

    len -= w_size;
    p_msg += w_size;
  };
  g_io_channel_flush (log_channel, &error);

  pre_log_func = g_log_set_default_handler (log_msg, NULL);
};

void
close_log_file ()
{
  GError *error = NULL;

  g_log_set_default_handler (pre_log_func, NULL);
  if (g_io_channel_shutdown (log_channel, TRUE, &error) != G_IO_STATUS_NORMAL)
  {
    g_error ("Error in close log file: %s\n", error->message);
  }
};

char *
str_time (time_t *t)
{
  static char strtime[STR_TIME_BUF_LENTH];

  strftime (strtime, STR_TIME_BUF_LENTH, "%a %Y %b %d %H:%M:%S", localtime (t));
  return strtime;
};

void
log_msg (const gchar *log_domain,
         GLogLevelFlags log_level, const gchar *message, gpointer user_data)
{
  gchar *p_msg = NULL;
  gsize len = 0;
  gsize w_size = 0;
  GError *error = NULL;
  GIOStatus status = G_IO_STATUS_NORMAL;
  time_t t = time (NULL);

  gchar *log = g_strdup_printf ("%s: %s level%d: %s",
                                log_domain, str_time (&t), log_level, message);

  len = strlen (log);
  p_msg = log;

  while (len > 0)
  {
    status = g_io_channel_write_chars (log_channel,
                                       p_msg, len, &w_size, &error);

    if (status != G_IO_STATUS_NORMAL)
    {
      g_print ("Error while writing to file: %s\n", error->message);
      break;
    }

    len -= w_size;
    p_msg += w_size;
  };

  g_io_channel_flush (log_channel, &error);
  g_free (log);
};

