/*  libim - Library of Instant Messaging and Presence enabler
 * im_xmpp_connect.cpp - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 */


/**
 *@file API implememt of the Instance Message Interface - IM XMPP CONNECT
 *
 *@version 0.1
 */

#include "imconnect_base.h"
#include "im_xmpp_connect.h"
#include <glib.h>
#include <iksemel.h>
//#include <string.h>
#include "im_xmpp_uri.h"
#include <string>
#include <string.h>
#include <gloox/jid.h>
#include "imtrans.h"
#include "improxy_loop.h"
#include "impresence.h"
#include "immsg.h"

im_err_t
im_xmpp_connect_init (im_connection *conn, char *ver)
{
  im_err_t rt = im_conn_init_base (conn, ver);

  if (rt != IM_ERROR_NONE)
    return rt;
  conn->free = im_xmpp_connect_free;
  conn->login = im_xmpp_connect_login;
  conn->login_cancel = im_xmpp_connect_login_cancel;
  conn->logout = im_xmpp_connect_logout;
  conn->on_login = im_xmpp_connect_on_login;
  conn->on_logout = im_xmpp_connect_on_logout;
  conn->on_presence = im_xmpp_connect_on_presence;
  conn->on_message = im_xmpp_connect_on_message;
  conn->on_subscribe = im_xmpp_connect_on_subscribe;
  conn->on_accept = im_xmpp_connect_on_accept;
  conn->on_error = im_xmpp_connect_on_error;
  conn->on_roster = im_xmpp_connect_on_roster;
  conn->on_buddy_op = im_xmpp_connect_on_buddy_op;
  return IM_ERROR_NONE;
};

im_connection *
im_xmpp_connect_new (char *ver)
{
  if (strcmp (ver, "1.0") != 0)
    return NULL;
  im_connection *conn = g_new0 (im_connection, 1);

  im_xmpp_connect_init (conn, ver);
  return conn;
};

im_err_t
im_xmpp_connect_free (im_connection *conn)
{
  return im_conn_free_base (conn);
};

im_err_t
im_xmpp_connect_login (im_connection *conn, char *passwd)
{
  int port = std::atoi (conn->server_port);
  char *jid_str = NULL;

  im_uri_get_uri_full (conn->login_uri, &jid_str);
  JID jid (jid_str);
  XmppConn *conn_data = new XmppConn (conn, jid, passwd,
                                      conn->server_addr, port);

  conn->data = conn_data;
  g_free (jid_str);
  return (conn_data->start_loop (conn)) ? IM_ERROR_NONE : IM_ERROR_NETWORK;
};

im_err_t
im_xmpp_connect_login_cancel (im_connection *conn)
{
  return IM_ERROR_NONE;
};

im_err_t
im_xmpp_connect_logout (im_connection *conn)
{
  XmppConn *x = reinterpret_cast < XmppConn *>(conn->data);

  x->quit_loop ();
  return IM_ERROR_NONE;
};

im_err_t
im_xmpp_connect_on_login (im_connection *conn, im_err_code_t err_code)
{
  return im_conn_on_login (conn, err_code);
};

im_err_t
im_xmpp_connect_on_logout (im_connection *conn)
{
  return im_conn_on_logout (conn);
};

im_err_t
im_xmpp_connect_on_presence (const im_connection *conn, const char *uri,
                             const im_presence_state_t pstate, const char *desc)
{
  return im_conn_on_presence (conn, uri, pstate, IM_PROTOCOL_XMPP, desc);
};

im_err_t
im_xmpp_connect_on_message (const im_connection *conn, im_msg_param_t *msg)
{
  return im_conn_on_message (conn, msg);
};

im_err_t
im_xmpp_connect_on_subscribe (const im_connection *conn, const char *from,
                              const char *msg, const bool subscribe)
{
  return im_conn_on_subscribe (conn, from, msg, subscribe);
};

im_err_t
im_xmpp_connect_on_accept (const im_connection *conn, const char *from,
                           const char *msg, const bool accept)
{
  return im_conn_on_accept (conn, from, msg, accept);
};

im_err_t
im_xmpp_connect_on_error (im_connection *conn, const im_err_code_t err_code,
                          const char *msg)
{
  return im_conn_on_error (conn, err_code, msg);
};

im_err_t
im_xmpp_connect_on_roster (const im_connection *conn)
{
  return im_conn_on_roster (conn);
};

im_err_t
im_xmpp_connect_on_buddy_op (const im_connection *conn, im_buddy_op_t *op)
{
  return im_conn_on_buddy_op (conn, op);
};

gpointer
XmppConn::loop (gpointer data)
{
  im_connection *conn = reinterpret_cast < im_connection *>(data);
  XmppConn *x = reinterpret_cast < XmppConn *>(conn->data);
  im_trans_t *t = NULL;

  while (!(x->m_quit))
  {
    ConnectionError err = x->m_Client->recv (1);

    if (err == ConnNoError)
    {
      t = im_trans_mgr_try_pop (conn->tmgr);
    }

    if (t)
    {
      switch (t->type)
      {
        case IM_REQ_LOGIN:
          t->status = IM_TRANS_PENDING;
          break;
        case IM_REQ_LOGOUT:
          x->quit_loop ();
          t->status = IM_TRANS_PENDING;
          break;
        case IM_REQ_PRESENCE:
          x->changePresence (((im_presence_param_t *) t->in)->uri,
                             ((im_presence_param_t *) t->in)->pstate,
                             ((im_presence_param_t *) t->in)->desc);
          g_free (((im_presence_param_t *) t->in)->uri);
          im_uri_get_uri_full (x->m_conn->login_uri,
                               &((im_presence_param_t *) t->in)->uri);
          t->status = IM_TRANS_OK;
          break;
        case IM_REQ_MESSAGE:
          x->sendMsg (((im_msg_param_t *) t->in));
          t->status = IM_TRANS_OK;
          break;
        case IM_REQ_SUBSCRIBE:
          x->subscribe (((im_subs_msg_t *) t->in)->contactor,
                        ((im_subs_msg_t *) t->in)->msg,
                        ((im_subs_msg_t *) t->in)->subscribe,
                        ((im_subs_msg_t *) t->in)->nick,
                        ((im_subs_msg_t *) t->in)->group);
          t->status = IM_TRANS_OK;
          break;
        case IM_REQ_ACCEPTANCE:
          x->authorize (((im_accept_msg_t *) t->in)->contactor,
                        ((im_accept_msg_t *) t->in)->msg,
                        ((im_accept_msg_t *) t->in)->accept);
          t->status = IM_TRANS_OK;
          break;
        case IM_REQ_ROSTER:
          x->get_rosters ();
          t->status = IM_TRANS_OK;
          break;
        case IM_REQ_BUDDY:
          x->buddy_op ((im_buddy_op_t *) t->in);
          t->status = IM_TRANS_OK;
          break;
        case IM_REQ_SYNCH:
          x->synchronize ();
          t->status = IM_TRANS_OK;
          break;
        case IM_REQ_PRESENCE_PROBE:
          {
            im_presence_param_t *p = g_new0 (im_presence_param_t, 1);

            p->prtl = IM_PROTOCOL_XMPP;
            if (p)
            {
              im_err_t err =
                x->probePresence ((char *) t->in, &p->pstate, &p->desc);
              p->uri = (char *) t->in;
              t->in = NULL;
              if (err == IM_ERROR_NONE)
              {
                t->status = IM_TRANS_OK;
              }
              else
              {
                t->status = IM_TRANS_FAIL;
                t->err_code = IM_ERR_CODE_BAD_URI;
              }
              t->out = p;
            }
            else
            {
              g_free (t->in);
              t->in = NULL;
              t->status = IM_TRANS_FAIL;
              t->err_code = IM_ERR_CODE_MEM;
            }
          }
          break;
        default:
          t->status = IM_TRANS_NOSUPPORT;
          break;
      };
      switch (t->status)
      {
        case IM_TRANS_PENDING:
          im_trans_mgr_push (conn->pendings, t);
          break;
        case IM_TRANS_NOSUPPORT:
          t->err_code = IM_ERR_CODE_NOSUPPORT;
        case IM_TRANS_CANCEL:
        case IM_TRANS_OK:
        case IM_TRANS_FAIL:
        case IM_TRANS_TIMEOUT:
          im_proxy_trans_notify (t);
          break;
        default:
          im_trans_free (t);
          break;
      }
      t = NULL;
    }
  }
  g_message ("quit thread.\n");
  return NULL;
}

bool
XmppConn::start_loop (im_connection *conn)
{
  GError *error = NULL;

  if (m_Client->connect (false))
  {
    m_quit = false;
    m_thread = g_thread_create (XmppConn::loop, conn, false, &error);
    if (error)
    {
      g_error ("%s\n", error->message);
      return false;
    }
    return true;
  }
  else
  {
    return false;
  }

}

void
XmppConn::onConnect ()
{
  // connection established, auth done (see API docs for exceptions)
  g_message ("xmpp login ...\n");
  im_xmpp_connect_on_login (m_conn, IM_ERR_CODE_NONE);
}

void
XmppConn::onDisconnect (ConnectionError e)
{
  m_quit = true;
  g_message ("xmpp thread stopped !\n");
  switch (e)
  {
    case ConnNoError:
      //no such case!
      break;
    case ConnStreamError:
      im_xmpp_connect_on_error (m_conn, IM_ERR_CODE_TRANSACTION,
                                "XML Stream Error");
      break;
    case ConnStreamClosed:
      im_xmpp_connect_on_error (m_conn, IM_ERR_CODE_SERVER,
                                "XML Stream Closed");
      break;
    case ConnIoError:
      im_xmpp_connect_on_error (m_conn, IM_ERR_CODE_NETWORK, "IO Error");
      break;
    case ConnOutOfMemory:
      im_xmpp_connect_on_error (m_conn, IM_ERR_CODE_MEM, "Out of Memory");
      break;
    case ConnNoSupportedAuth:
      im_xmpp_connect_on_login (m_conn, IM_ERR_CODE_NOSUPPORT);
      break;
    case ConnTlsFailed:
      im_xmpp_connect_on_login (m_conn, IM_ERR_CODE_NETWORK);
      break;
    case ConnAuthenticationFailed:
      im_xmpp_connect_on_login (m_conn, IM_ERR_CODE_PASSWORD);
      break;
    case ConnUserDisconnected:
      im_rec_mutex_lock (m_conn->mutex);
      if (!m_timeouted && m_timeout_id != 0)
      {
        g_source_remove (m_timeout_id);
        m_timeout_id = 0;
      }
      if (!m_logouted)
      {
        m_logouted = TRUE;
        im_xmpp_connect_on_logout (m_conn);
        g_print ("server acknowledged disconnection!\n");
      }
      im_rec_mutex_unlock (m_conn->mutex);
      break;
    case ConnNotConnected:
      break;
    default:
      break;
  }
};

bool
XmppConn::onTLSConnect (const CertInfo & info)
{
  g_message ("on TLS...\n");
  return true;
}

void
XmppConn::handleMessage (Stanza *stanza)
{
  im_msg_param_t *msg = g_new0 (im_msg_param_t, 1);

  if (!msg)
  {
    g_error ("%s %s:%d no memory\n", __FILE__, __func__, __LINE__);
    return;
  }
  switch (stanza->subtype ())
  {
    case StanzaMessageChat:
      msg->msg_type = IM_MSG_TYPE_CHAT;
      break;
    case StanzaMessageError:
      msg->msg_type = IM_MSG_TYPE_ERROR;
      break;
    case StanzaMessageGroupchat:
      msg->msg_type = IM_MSG_TYPE_GROUPCHAT;
      break;
    case StanzaMessageNormal:
      msg->msg_type = IM_MSG_TYPE_NORMAL;
      break;
    case StanzaMessageHeadline:
      msg->msg_type = IM_MSG_TYPE_HEADLINE;
      break;
    default:
      im_msg_param_free (msg);
      return;
      break;
  }
  const char *to = g_strdup (stanza->to ().full ().c_str ());

  msg->uri = g_strdup (stanza->from ().full ().c_str ());
  msg->msg = g_strdup (stanza->body ().c_str ());
  msg->msg = (msg->msg) ? (msg->msg) : (void *) "";
  msg->encode = g_strdup (stanza->xmlLang ().c_str ());
  msg->msg_size = strlen ((char *) msg->msg) + 1;
  msg->msg_id = 0;
  msg->msg_time = time (NULL);
  msg->msg_ctype = IM_MSG_CTYPE_TEXT;
  msg->msg_state = IM_MSG_STATE_RECV;
  g_print ("recv msg: to:%s from:%s\nbody:%s\nencode:%s\n", to ? to : "(null)",
           msg->uri ? msg->uri : "(null)", (char *) msg->msg,
           msg->encode ? msg->encode : "(null)");
  im_xmpp_connect_on_message (m_conn, msg);
  //TODO: release to. from & msg
};

im_presence_state_t
XmppConn::gloox_status_2_im_presence (Presence p)
{
  switch (p)
  {
    case PresenceUnknown:
      return IM_PRESENCE_CUSTOM;
      break;
    case PresenceAvailable:
    case PresenceChat:
      return IM_PRESENCE_AVAILABLE;
      break;
    case PresenceAway:
      return IM_PRESENCE_AWAY;
      break;
    case PresenceDnd:
      return IM_PRESENCE_DND;
      break;
    case PresenceXa:
      return IM_PRESENCE_FARAWAY;
      break;
    case PresenceUnavailable:
      return IM_PRESENCE_OFFLINE;
      break;
    default:
      return IM_PRESENCE_CUSTOM;
      break;
  }
}

void
XmppConn::handlePresence (Stanza *stanza)
{
  const char *str_from = stanza->from ().full ().c_str ();
  im_presence_state_t pstate = gloox_status_2_im_presence (stanza->show ());
  const char *desc = stanza->status ().c_str ();

  g_print ("from: %s presence: %d desc: %s\n", str_from, pstate, desc);
  im_xmpp_connect_on_presence (m_conn, str_from, pstate, desc);
  //TODO: release desc?
}

void
XmppConn::handleSubscription (Stanza *stanza)
{
  //g_print("++++++++++\n");
};

void
XmppConn::handleLog (LogLevel level, LogArea area, const std::string & message)
{
  g_print ("log: level: %d, area: %d, %s\n", level, area, message.c_str ());
}

XmppConn::XmppConn (im_connection *conn, const JID & jid,
                    const std::string & password, const std::string & server,
                    int port)
{
  m_conn = conn;
  m_timeout_id = 0;
  m_timeouted = FALSE;
  m_logouted = FALSE;
  m_Client = new Client (jid, password, port);
  m_Client->setServer (server);
  m_Client->setXmlLang ("utf-8");
  m_Client->registerConnectionListener (this);
  m_Client->registerPresenceHandler (this);
  m_Client->registerMessageHandler (this);
  m_Client->registerSubscriptionHandler (this);
  m_Client->rosterManager ()->registerRosterListener (this, false);     //synchronous.
  m_Client->setAutoPresence (true);
  m_Client->setInitialPriority (4);
  m_Client->logInstance ().registerLogHandler (LogLevelDebug, LogAreaAll, this);        //register log
  m_quit = false;
}

XmppConn::~XmppConn ()
{
  delete m_Client;
}

gboolean
XmppConn::force_quit (XmppConn *xc)
{
  im_rec_mutex_lock (xc->m_conn->mutex);
  g_print ("force disconnected!\n");
  if (xc->m_Client->state () != StateDisconnected)
  {
    xc->m_timeouted = TRUE;
    xc->onDisconnect (ConnUserDisconnected);
  }
  im_rec_mutex_unlock (xc->m_conn->mutex);
  return FALSE;
}

void
XmppConn::quit_loop ()
{
  changePresence ("", IM_PRESENCE_OFFLINE, "");
  m_Client->disconnect ();
  m_timeout_id = g_timeout_add (2000, (GSourceFunc) (force_quit), this);
  //g_thread_join(m_thread);
}

void
XmppConn::changePresence (const char *uri, const im_presence_state_t pstate,
                          const char *desc)
{
  Presence p = PresenceAvailable;

  switch (pstate)
  {
    case IM_PRESENCE_AVAILABLE:
      break;
    case IM_PRESENCE_AWAY:
    case IM_PRESENCE_CUSTOM:
      p = PresenceAway;
      break;
    case IM_PRESENCE_FARAWAY:
      p = PresenceXa;
      break;
    case IM_PRESENCE_DND:
    case IM_PRESENCE_INVISIBLE:
      p = PresenceDnd;
      break;
    case IM_PRESENCE_OFFLINE:
      p = PresenceUnavailable;
      break;
    default:
      break;
  }
  const JID jid (uri);
  Stanza *s = Stanza::createPresenceStanza (jid, desc, p);

  m_Client->send (s);
};

im_err_t
XmppConn::probePresence (const char *uri, im_presence_state_t *pstate,
                         char **pDesc)
{
  *pstate = IM_PRESENCE_OFFLINE;
  *pDesc = NULL;
  JID jid (uri);
  RosterItem *item = m_Client->rosterManager ()->getRosterItem (jid);

  if (!item)
    return IM_ERROR_PARAM;
  if (item->online ())
  {
    if (jid.resource ().empty ())
    {
      //      int priority = 0;
      //      Presence p = PresenceUnknown;
      RosterItem::ResourceMap::const_iterator it = item->resources ().begin ();
      RosterItem::ResourceMap::const_iterator tmp_it = it++;
      //      priority = it->second->priority();
      //      p = it->second->status();
      //      *pDesc = g_strdup(it->second->message().c_str());
      while (it != item->resources ().end ())
      {
        if (tmp_it->second->priority () < it->second->priority ())
          tmp_it = it++;
      }
      *pstate = gloox_status_2_im_presence (tmp_it->second->status ());
      *pDesc = g_strdup (tmp_it->second->message ().c_str ());
    }
    else
    {
      RosterItem::ResourceMap::const_iterator it =
        item->resources ().find (jid.resource ());
      if (it == item->resources ().end ())
      {
        g_print ("in %s:%d %s could not find jid %s\n", __FILE__, __LINE__,
                 __func__, uri);
        return IM_ERROR_PARAM;
      }
      else
      {
        *pstate = gloox_status_2_im_presence (it->second->status ());
        *pDesc = g_strdup (it->second->message ().c_str ());
      }
    }
  }
  return IM_ERROR_NONE;
};

void
XmppConn::sendMsg (const im_msg_param_t *msg)
{
  StanzaSubType sstype = StanzaSubUndefined;

  switch (msg->msg_type)
  {
    case IM_MSG_TYPE_NORMAL:
      sstype = StanzaMessageNormal;
      break;
    case IM_MSG_TYPE_CHAT:
      sstype = StanzaMessageChat;
      break;
    case IM_MSG_TYPE_GROUPCHAT:
      sstype = StanzaMessageGroupchat;
      break;
    default:
      break;
  }
  if (sstype != StanzaSubUndefined && msg->msg_ctype == IM_MSG_CTYPE_TEXT)
  {
    char *text = (char *) msg->msg;
    Stanza *s =
      Stanza::createMessageStanza (JID (msg->uri), text, sstype, "", "",
                                   msg->encode);
    m_Client->send (s);
  }
};

void
XmppConn::subscribe (const char *to, const char *msg, const bool subscribe,
                     const char *nick, const char *group)
{
  if (subscribe)
  {
    StringList groupList;

    groupList.push_front (std::string (group));
    m_Client->rosterManager ()->subscribe (to, nick, groupList, msg);
  }
  else
  {
    m_Client->rosterManager ()->unsubscribe (to, msg, false);   // don't remove the contactor from roster.
  }
};

void
XmppConn::authorize (const char *to, const char *msg, const bool accept)
{
  StanzaSubType subtype;

  if (accept)
  {
    subtype = StanzaS10nSubscribed;
  }
  else
  {
    subtype = StanzaS10nUnsubscribed;
  }
  Stanza *s = Stanza::createSubscriptionStanza (JID (to), msg, subtype);

  m_Client->send (s);
};

void
XmppConn::get_rosters ()
{
  m_Client->rosterManager ()->fill ();
};

void
XmppConn::synchronize ()
{
  m_Client->rosterManager ()->synchronize ();
};

void
XmppConn::buddy_op (im_buddy_op_t *op)
{
  im_buddy_operation_t op_type;
  im_roster_item_t *buddy = NULL;

  im_buddy_op_get_buddy (op, &buddy);
  im_buddy_op_get_operation (op, &op_type);
  switch (op_type)
  {
    case IM_BUDDY_ADD:
      {
        StringList groupList;

        if (::strcmp (buddy->group, ""))
        {
          groupList.push_front (std::string (buddy->group));
        }
        else
        {
          groupList.push_front (IM_BUDDY_DEFAULT_GROUP);
        }
        m_Client->rosterManager ()->add (buddy->contactor, buddy->nickname,
                                         groupList);
      }
      break;
    case IM_BUDDY_REMOVE:
      m_Client->rosterManager ()->unsubscribe (buddy->contactor, "", true);
      break;
    case IM_BUDDY_UPDATE:
      {
        RosterItem *item =
          m_Client->rosterManager ()->getRosterItem (JID (buddy->contactor));
        if (item)
        {
          item->setName (buddy->nickname);
          StringList groupList;

          if (::strcmp (buddy->group, ""))
          {
            groupList.push_front (std::string (buddy->group));
          }
          else
          {
            groupList.push_front (IM_BUDDY_DEFAULT_GROUP);
          }
          item->setGroups (groupList);
        }
      }
      break;
    default:
      break;
  }
};

im_roster_item_t *
gloox_roster_2_im_buddy (RosterItem *item)
{
  im_roster_item_t *buddy = (im_roster_item_t *) g_new0 (im_roster_item_t, 1);

  if (!buddy)
    return NULL;
  buddy->contactor = g_strdup (item->jid ().c_str ());
  buddy->nickname = g_strdup (item->name ().c_str ());
  switch (item->subscription ())
  {
    case RosterItem::S10N_NONE:
    case RosterItem::S10N_NONE_OUT:
    case RosterItem::S10N_NONE_IN:
    case RosterItem::S10N_NONE_OUT_IN:
      buddy->subs_state = IM_SUB_NONE;
      break;
    case RosterItem::S10N_TO:
    case RosterItem::S10N_TO_IN:
      buddy->subs_state = IM_SUB_TO;
      break;
    case RosterItem::S10N_FROM:
    case RosterItem::S10N_FROM_OUT:
      buddy->subs_state = IM_SUB_FROM;
      break;
    case RosterItem::S10N_BOTH:
      buddy->subs_state = IM_SUB_BOTH;
      break;
    default:
      buddy->subs_state = IM_SUB_INVALID;
      break;
  }
  StringList groups = item->groups ();

  if (!groups.empty ())
  {
    buddy->group = g_strdup (groups.front ().c_str ());
  }
  else
  {
    buddy->group = g_strdup (IM_BUDDY_DEFAULT_GROUP);
    //    g_warning("group of %s will be displayed as '%s'\n", buddy->contactor, IM_BUDDY_DEFAULT_GROUP);
  }
  return buddy;
}

bool
XmppConn::on_buddy_op (const std::string & jid, im_buddy_operation_t op_type)
{
  RosterItem *item = m_Client->rosterManager ()->getRosterItem (jid);
  im_roster_item_t *buddy = NULL;

  if (item)
  {
    buddy = gloox_roster_2_im_buddy (item);
  }
  else
  {
    buddy = (im_roster_item_t *) g_new0 (im_roster_item_t, 1);
    if (!buddy)
      return false;
    buddy->contactor = g_strdup (jid.c_str ());
  }
  if (!buddy)
    return false;
  im_buddy_op_t *op = NULL;

  im_buddy_op_new (buddy, op_type, &op);
  if (op)
  {
    return im_xmpp_connect_on_buddy_op (m_conn, op) == IM_ERROR_NONE;
  };
  return false;
};

void
XmppConn::itemAdded (const std::string & jid)
{
  g_print ("---add a roster: %s\n", jid.c_str ());
  this->on_buddy_op (jid, IM_BUDDY_ADD);
};

void
XmppConn::itemSubscribed (const std::string & jid)
{
  g_print ("---Subscribed to: %s\n", jid.c_str ());
  im_xmpp_connect_on_accept (m_conn, jid.c_str (), NULL, TRUE);
};

void
XmppConn::itemRemoved (const std::string & jid)
{
  g_print ("---Remove a roster: %s\n", jid.c_str ());
  this->on_buddy_op (jid, IM_BUDDY_REMOVE);
};

void
XmppConn::itemUpdated (const std::string & jid)
{
  g_print ("---Roster update: %s\n", jid.c_str ());
  this->on_buddy_op (jid, IM_BUDDY_UPDATE);
};

void
XmppConn::itemUnsubscribed (const std::string & jid)
{
  g_print ("---Unsubscribed to roster: %s\n", jid.c_str ());
  im_xmpp_connect_on_accept (m_conn, jid.c_str (), NULL, FALSE);
};

void
XmppConn::roster (const Roster & roster)
{
  g_print ("---Roster synchronized\n");
  if (this->m_conn->roster)
  {
    im_roster_free (this->m_conn->roster);
  }
  if (im_roster_new (&(this->m_conn->roster)) != IM_ERROR_NONE)
    return;
  im_roster_item_t *buddy = NULL;
  const char *uri = this->m_Client->jid ().bare ().c_str ();

  for (Roster::const_iterator iter = roster.begin ();
       iter != roster.end (); ++iter)
  {
    buddy = gloox_roster_2_im_buddy (iter->second);
    if (strcmp (uri, buddy->contactor) == 0)
      continue;
    im_roster_prepend (this->m_conn->roster, buddy);
  }
  im_xmpp_connect_on_roster (this->m_conn);
};

void
XmppConn::presenceUpdated (const RosterItem & item, int status,
                           const std::string & msg)
{
  //g_print("---%s presence changed to %d:%s\n", item.jid().c_str(), status, msg.c_str());
};

void
XmppConn::itemAvailable (const RosterItem & item, const std::string & msg)
{
  //g_print("---%s is available now: %s\n", item.jid().c_str(), msg.c_str());
};

void
XmppConn::itemUnavailable (const RosterItem & item, const std::string & msg)
{
  //g_print("---%s is unavailable now: %s\n", item.jid().c_str(), msg.c_str());
};

bool
XmppConn::subscriptionRequest (const std::string & jid, const std::string & msg)
{
  g_print ("---%s sent you a subscription request: %s\n", jid.c_str (),
           msg.c_str ());
  im_xmpp_connect_on_subscribe (m_conn, jid.c_str (), msg.c_str (), TRUE);
  return false;
};

bool
XmppConn::unsubscriptionRequest (const std::string & jid,
                                 const std::string & msg)
{
  g_print ("---%s sent you a unsubscription request: %s\n", jid.c_str (),
           msg.c_str ());
  im_xmpp_connect_on_subscribe (m_conn, jid.c_str (), msg.c_str (), FALSE);
  return false;
};

void
XmppConn::nonrosterPresenceReceived (const JID & jid)
{
  g_print ("---%s presence changed\n", jid.full ().c_str ());
};

