/*  libim - Library of Instant Messaging and Presence enabler
 * im_xmpp_connect.h - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 */


/**
 *@file API definition of the Instance Message Interface - IM XMPP Connect
 *
 *@version 0.1
 */

#ifndef __LIPS_IM_XMPP_CONNECT_H
#define __LIPS_IM_XMPP_CONNECT_H

#include "imconnect.h"
#include <gloox/client.h>
#include <gloox/connectionlistener.h>
#include <gloox/presencehandler.h>
#include <gloox/gloox.h>
#include <glib.h>
#include <gloox/loghandler.h>
#include <gloox/messagehandler.h>
#include <gloox/rostermanager.h>
#include <gloox/rosterlistener.h>
#include <gloox/subscriptionhandler.h>
#include <gloox/rosteritem.h>

#ifdef __cplusplus
//extern "C" {
#endif /* __cplusplus */

im_connection *im_xmpp_connect_new (char *ver);

im_err_t im_xmpp_connect_free (im_connection *conn);

im_err_t im_xmpp_connect_login (im_connection *conn, char *passwd);

im_err_t im_xmpp_connect_login_cancel (im_connection *conn);

im_err_t im_xmpp_connect_logout (im_connection *conn);

im_err_t im_xmpp_connect_on_login (im_connection *conn,
                                   im_err_code_t err_code);

im_err_t im_xmpp_connect_on_logout (im_connection *conn);

im_err_t im_xmpp_connect_on_presence (const im_connection *conn,
                                      const char *uri,
                                      const im_presence_state_t pstate,
                                      const char *desc);

im_err_t im_xmpp_connect_on_message (const im_connection *conn,
                                     im_msg_param_t *msg);

im_err_t im_xmpp_connect_on_subscribe (const im_connection *conn,
                                       const char *from, const char *msg,
                                       const bool subscribe);

im_err_t im_xmpp_connect_on_accept (const im_connection *conn,
                                    const char *from, const char *msg,
                                    const bool accept);

im_err_t im_xmpp_connect_on_error (im_connection *conn,
                                   const im_err_code_t err_code,
                                   const char *msg);

im_err_t im_xmpp_connect_on_roster (const im_connection *conn);

im_err_t im_xmpp_connect_on_buddy_op (const im_connection *conn,
                                      im_buddy_op_t *op);

#ifdef __cplusplus
//}
#endif /* __cplusplus */

using namespace std;
using namespace gloox;

class XmppConn:public ConnectionListener, PresenceHandler,
  LogHandler, MessageHandler, RosterListener, SubscriptionHandler
{
public:
  XmppConn (im_connection *conn, const JID & jid, const std::string & password,
            const std::string & server, int port);

  static gpointer loop (gpointer data);

  bool start_loop (im_connection *conn);

  virtual void handlePresence (Stanza *stanza);

  virtual void onConnect ();

  virtual void onDisconnect (ConnectionError e);

  virtual bool onTLSConnect (const CertInfo & info);

  virtual void handleMessage (Stanza *stanza);

  virtual void handleLog (LogLevel level, LogArea area,
                          const std::string & message);

  virtual void handleSubscription (Stanza *stanza);

  Client *get_client ()
  {
    return m_Client;
  };

  void changePresence (const char *uri, const im_presence_state_t pstate,
                       const char *desc);

  im_err_t probePresence (const char *uri, im_presence_state_t *pstate,
                          char **pDesc);

  void sendMsg (const im_msg_param_t *msg);

  void subscribe (const char *to, const char *msg, const bool subscribe,
                  const char *nick, const char *group);

  void authorize (const char *to, const char *msg, const bool accept);

  void get_rosters ();

  void buddy_op (im_buddy_op_t *op);

  void synchronize ();

  void quit_loop ();

  virtual void itemAdded (const std::string & jid);
  virtual void itemSubscribed (const std::string & jid);
  virtual void itemRemoved (const std::string & jid);
  virtual void itemUpdated (const std::string & jid);
  virtual void itemUnsubscribed (const std::string & jid);
  virtual void roster (const Roster & roster);
  virtual void presenceUpdated (const RosterItem & item, int status,
                                const std::string & msg);
  virtual void itemAvailable (const RosterItem & item, const std::string & msg);
  virtual void itemUnavailable (const RosterItem & item,
                                const std::string & msg);
  virtual bool subscriptionRequest (const std::string & jid,
                                    const std::string & msg);
  virtual bool unsubscriptionRequest (const std::string & jid,
                                      const std::string & msg);
  virtual void nonrosterPresenceReceived (const JID & jid);

  ~XmppConn ();

private:
  Client *m_Client;
  GThread *m_thread;
  bool m_quit;
  im_connection *m_conn;
  unsigned int m_timeout_id;
  bool m_timeouted;
  bool m_logouted;

  static gboolean force_quit (XmppConn *xc);
  bool on_buddy_op (const std::string & jid, im_buddy_operation_t op_type);
  im_presence_state_t gloox_status_2_im_presence (Presence p);
};

#endif /* __LIPS_IM_XMPP_CONNECT_H*/

