/*  libim - Library of Instant Messaging and Presence enabler
 * im_xmpp_uri.c - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 * This software and associated documentation files (the "Software") 
 * are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 * All Rights Reserved. 
 *
 * A copyright license is hereby granted for redistribution and use of 
 * the Software in source and binary forms, with or without modification, 
 * provided that the following conditions are met: 
 * - Redistributions of source code must retain the above copyright notice, 
 * this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 * notice, this copyright license and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution. 
 * - Neither the name of LiPS  nor the names of its Members may be used 
 * to endorse or promote products derived from the Software without 
 * specific prior written permission. 
 *
 * A patent license for any Necessary Claims owned by Members of LiPS Forum 
 * to make, have made, use, import, offer to sell, lease and sell or otherwise 
 * distribute any implementation compliant with the any specification adopted 
 * by the LiPS Forumcan be obtained from the respective Members on reasonable 
 * and non-discriminatory terms and conditions and under reciprocity, as 
 * regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 * "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 * AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 * ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 */


/**
 *@file API implememt of the Instance Message Interface - IM XMPP URI
 *
 *@version 0.1
 */

#include "im_xmpp_uri.h"
#include "imuri_base.h"
#include <iksemel.h>
#include <glib.h>
#include <string.h>

im_err_t
im_uri_new_xmpp (char *uri_str, im_uri_t **uri)
{
  im_err_t rt = im_uri_init_base (uri);

  if (rt != IM_ERROR_NONE)
    return rt;

  ikstack *stack = iks_stack_new (1024, 1024);
  iksid *jid = iks_id_new (stack, uri_str);

  (*uri)->uid = (char *) g_strdup (jid->user);
  (*uri)->domain = (char *) g_strdup (jid->server);
  (*uri)->resource = (char *) g_strdup (jid->resource);
  iks_stack_delete (stack);

  (*uri)->free = im_uri_free_base;
  (*uri)->cmp = im_xmpp_uri_cmp;
  (*uri)->get_full_ext = im_xmpp_get_full_ext;
  (*uri)->get_uri = im_xmpp_get_uri;
  (*uri)->get_uri_full = im_xmpp_get_uri_full;

  return rt;
};

bool
im_xmpp_uri_cmp (im_uri_t *first, im_uri_t *second, im_uri_part_t part)
{
  return im_uri_cmp_base (first, second, part);
};

im_err_t
im_xmpp_get_full_ext (im_uri_t *uri, char **str)
{
  return im_xmpp_get_uri_full (uri, str);
};

im_err_t
im_xmpp_get_uri (im_uri_t *uri, char **std_str)
{
  *std_str = (char *) g_strconcat (uri->uid, "@", uri->domain, NULL);
  return (*std_str) ? IM_ERROR_NONE : IM_ERROR_NULL;
};

im_err_t
im_xmpp_get_uri_full (im_uri_t *uri, char **full_str)
{
  *full_str =
    (char *) g_strconcat (uri->uid, "@", uri->domain, "/", uri->resource, NULL);
  return (*full_str) ? IM_ERROR_NONE : IM_ERROR_NULL;
};

