/*  libim - Library of Instant Messaging and Presence enabler
 * im_proxy.c - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 */


/**
 *@file API implememt of the Instance Message Interface - IM Proxy
 *
 *@version 0.1
 */

#include "im_proxy.h"
#include "improxy.h"
#include "log/im_log.h"
#include "improxy_loop.h"
#include "improxy_dbus.h"

int
main (int argc, char **argv)
{
  DBusError error;
  im_proxy *proxy;

  open_log_file (IM_DBUS_LOG_FILE);

  bus_conn = dbus_init_im ();
  if (!bus_conn)
  {
    close_log_file ();
    return 1;
  }

  dbus_error_init (&error);
  g_message ("the unique name for this connection: %s\n",
             dbus_bus_get_unique_name (bus_conn));
  dbus_bus_request_name (bus_conn, IM_DBUS_SERVICE, 0, &error);
  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to acquire service name: %s\n", IM_DBUS_SERVICE);
    dbus_error_free (&error);
    close_log_file ();
    return 1;
  }

  dbus_add_msg_listener_c (IM_DBUS_INTERFACE,
                           IM_DBUS_MSG_OPEN_SESSION, open_session);

  dbus_add_msg_listener_c (IM_DBUS_INTERFACE, IM_DBUS_MSG_QUIT, quit_session);

  dbus_add_msg_listener_c (IM_DBUS_INTERFACE,
                           IM_DBUS_MSG_CLOSE_SESSION, close_session);

  dbus_add_msg_listener_c (IM_DBUS_INTERFACE,
                           IM_DBUS_MSG_REG, im_proxy_evt_reg);

  dbus_add_msg_listener_c (IM_DBUS_INTERFACE,
                           IM_DBUS_MSG_UNREG, im_proxy_evt_unreg);

  dbus_add_msg_listener_c (IM_DBUS_INTERFACE,
                           IM_DBUS_MSG_LOGIN, im_proxy_req_login);

  dbus_add_msg_listener_c (IM_DBUS_INTERFACE,
                           IM_DBUS_MSG_LOGOUT, im_proxy_req_logout);

  dbus_add_msg_listener_c (IM_DBUS_INTERFACE,
                           IM_DBUS_MSG_CHANGE_PRESENCE, im_proxy_req_presence);

  dbus_add_msg_listener_c (IM_DBUS_INTERFACE,
                           IM_DBUS_MSG_PRESENCE_PROBE,
                           im_proxy_req_presence_probe);

  dbus_add_msg_listener_c (IM_DBUS_INTERFACE,
                           IM_DBUS_MSG_SENDMSG, im_proxy_req_msg);

  dbus_add_msg_listener_c (IM_DBUS_INTERFACE,
                           IM_DBUS_MSG_SUBSCRIBE, im_proxy_req_subscribe);

  dbus_add_msg_listener_c (IM_DBUS_INTERFACE,
                           IM_DBUS_MSG_ACCEPT, im_proxy_req_accept);

  dbus_add_msg_listener_c (IM_DBUS_INTERFACE,
                           IM_DBUS_MSG_GET_ROSTER, im_proxy_req_roster);

  dbus_add_msg_listener_c (IM_DBUS_INTERFACE,
                           IM_DBUS_MSG_BUDDY, im_proxy_req_buddy_op);

  dbus_add_msg_listener_c (IM_DBUS_INTERFACE,
                           IM_DBUS_MSG_SYNCH, im_proxy_req_synch);

  proxy = im_proxy_get ();

  mainloop = g_main_loop_new (NULL, FALSE);
  g_source_attach (im_recv_src (), NULL);       //attach the returned event source.
  g_message ("ready to receive controlling commands.\n");
  g_main_loop_run (mainloop);

  im_proxy_free (proxy);
  close_log_file ();
  return 0;
}

