/*  libim - Library of Instant Messaging and Presence enabler
 * test_dbus.c - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 * This software and associated documentation files (the "Software") 
 * are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 * All Rights Reserved. 
 *
 * A copyright license is hereby granted for redistribution and use of 
 * the Software in source and binary forms, with or without modification, 
 * provided that the following conditions are met: 
 * - Redistributions of source code must retain the above copyright notice, 
 * this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 * notice, this copyright license and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution. 
 * - Neither the name of LiPS  nor the names of its Members may be used 
 * to endorse or promote products derived from the Software without 
 * specific prior written permission. 
 *
 * A patent license for any Necessary Claims owned by Members of LiPS Forum 
 * to make, have made, use, import, offer to sell, lease and sell or otherwise 
 * distribute any implementation compliant with the any specification adopted 
 * by the LiPS Forumcan be obtained from the respective Members on reasonable 
 * and non-discriminatory terms and conditions and under reciprocity, as 
 * regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 * "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 * AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 * ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 */


#include <glib.h>
#include "im_client_dbus.h"
#include "im_dbus.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void
usage ()
{
  printf ("usage: test_dbus {start|open|close|quit}\n");
  exit (0);
}

int
main (int argc, char **argv)
{
  DBusError error;
  DBusConnection *bus_conn = NULL;
  const char *msg = NULL;
  DBusMessage *message = NULL;
  DBusMessage *reply = NULL;
  int start_service = 0;
  unsigned int result;

  msg = (char *) dbus_malloc (24);

  dbus_error_init (&error);
  bus_conn = dbus_bus_get (DBUS_BUS_SESSION, &error);

  if (argc == 1)
  {
    usage ();
  }

  if (strcmp (argv[1], "open") == 0)
  {
    msg = IM_DBUS_MSG_OPEN_SESSION;
  }
  else if (strcmp (argv[1], "close") == 0)
  {
    msg = IM_DBUS_MSG_CLOSE_SESSION;
  }
  else if (strcmp (argv[1], "quit") == 0)
  {
    msg = IM_DBUS_MSG_QUIT;
  }
  else if (strcmp (argv[1], "start") == 0)
  {
    start_service = 1;
  }
  else
  {
    usage ();
  }

  if (!bus_conn)
  {
    g_warning ("Failed to connect to the D-BUS daemon: %s", error.message);
    dbus_error_free (&error);
    return 1;
  }

  if (!dbus_bus_name_has_owner (bus_conn, IM_DBUS_SERVICE, NULL))
  {
    g_printerr
      ("No IM_Proxy service is running.\nNow starting the service...\n");
    start_service = 1;
    //return 1;
  }
  //dbus_connection_setup_with_g_main (bus_conn, NULL);

  //start up the com.ftrd.player service
  if (start_service == 1)
  {
    if (dbus_bus_start_service_by_name (bus_conn,
                                        IM_DBUS_SERVICE, 0, &result,
                                        &error) == FALSE)
    {
      if (dbus_error_is_set (&error))
        g_warning ("Failed to start up service '%s': %s",
                   IM_DBUS_SERVICE, error.message);
    }
    else
    {
      if (result == DBUS_START_REPLY_SUCCESS)
        g_warning ("service '%s' successfully started up", IM_DBUS_SERVICE);
      else if (result == DBUS_START_REPLY_ALREADY_RUNNING)
        g_warning ("service '%s' already running", IM_DBUS_SERVICE);
    }
    //return 0;
  }

  if (!msg)
    return 0;                   //no more message to send.

  //perform the commands
  message = dbus_message_new_method_call (IM_DBUS_SERVICE,
                                          IM_DBUS_OBJECT,
                                          IM_DBUS_INTERFACE, msg);

  reply = dbus_connection_send_with_reply_and_block (bus_conn,
                                                     message,
                                                     IM_DBUS_CALL_DELAY,
                                                     &error);

  dbus_message_unref (message);

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to send message '%s': %s", msg, error.message);
    dbus_error_free (&error);
    return 1;
  }

  dbus_message_unref (reply);

  return 0;

}

