/*  libim - Library of Instant Messaging and Presence enabler
 * test_im.c - 
 *
 * Authors: ZHAO Liangjing <liangjing.zhao@orange-ftgroup.com> 
 * 
 * This software and associated documentation files (the "Software") 
 * are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 * All Rights Reserved. 
 *
 * A copyright license is hereby granted for redistribution and use of 
 * the Software in source and binary forms, with or without modification, 
 * provided that the following conditions are met: 
 * - Redistributions of source code must retain the above copyright notice, 
 * this copyright license and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright 
 * notice, this copyright license and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution. 
 * - Neither the name of LiPS  nor the names of its Members may be used 
 * to endorse or promote products derived from the Software without 
 * specific prior written permission. 
 *
 * A patent license for any Necessary Claims owned by Members of LiPS Forum 
 * to make, have made, use, import, offer to sell, lease and sell or otherwise 
 * distribute any implementation compliant with the any specification adopted 
 * by the LiPS Forumcan be obtained from the respective Members on reasonable 
 * and non-discriminatory terms and conditions and under reciprocity, as 
 * regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 * "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 * AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 * ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE. 
 */


/**
 *test program for Instance Message Interface APIs
 *
 */

#include "im.h"
#include <stdio.h>
#include <glib.h>
#include <string.h>
#include "immsg_imp.h"
#include "im_client_dbus.h"
#include <signal.h>

char *server = NULL;
char *port = NULL;
char *login = NULL;
char *password = NULL;
char *client_name = NULL;
char *buddy = NULL;
char *buddy_nick = NULL;
char *buddy_group = NULL;

GMainLoop *loop = NULL;

#define IM_CONFIG_FILE "im.conf"
#define IM_CONFIG_GROUP "IM_CONFIG"
#define IM_CONFIG_LOGIN "login"
#define IM_CONFIG_PASSWD "passwd"
#define IM_CONFIG_SERVER "server"
#define IM_CONFIG_PORT "port"
#define IM_CONFIG_CNAME "cname"
#define IM_CONFIG_BUDDY "buddy"
#define IM_CONFIG_BUDDY_NICK "buddy_nick"
#define IM_CONFIG_BUDDY_GROUP "buddy_group"

typedef void (*sighandler_t) (int);

static gboolean quit_loop (im_sid_t sid);
static sighandler_t dft_sigint_hdl = NULL;
static void sigint_hdl (int signum);
static im_sid_t my_sid = 0;
static void reg (sid_t sid);
static void unreg (sid_t sid);
static void req_acceptance (im_sid_t sid, char *to, char *msg, bool accept);
static gboolean run_loop (im_sid_t sid);

void
load_config ()
{
  GKeyFile *f = g_key_file_new ();
  GError *err = NULL;

  g_key_file_load_from_file (f, IM_CONFIG_FILE, G_KEY_FILE_NONE, &err);
  if (!err)
  {
    login = g_key_file_get_value (f, IM_CONFIG_GROUP, IM_CONFIG_LOGIN, &err);
    if (err)
      g_clear_error (&err);
    password =
      g_key_file_get_value (f, IM_CONFIG_GROUP, IM_CONFIG_PASSWD, &err);
    if (err)
      g_clear_error (&err);
    server = g_key_file_get_value (f, IM_CONFIG_GROUP, IM_CONFIG_SERVER, &err);
    if (err)
      g_clear_error (&err);
    port = g_key_file_get_value (f, IM_CONFIG_GROUP, IM_CONFIG_PORT, &err);
    if (err)
      g_clear_error (&err);
    client_name =
      g_key_file_get_value (f, IM_CONFIG_GROUP, IM_CONFIG_CNAME, &err);
    if (err)
      g_clear_error (&err);
    buddy = g_key_file_get_value (f, IM_CONFIG_GROUP, IM_CONFIG_BUDDY, &err);
    if (err)
      g_clear_error (&err);
    buddy_nick =
      g_key_file_get_value (f, IM_CONFIG_GROUP, IM_CONFIG_BUDDY_NICK, &err);
    if (err)
      g_clear_error (&err);
    buddy_group =
      g_key_file_get_value (f, IM_CONFIG_GROUP, IM_CONFIG_BUDDY_GROUP, &err);
    if (err)
      g_clear_error (&err);
  }
  else
  {
    g_clear_error (&err);
    g_print ("failed to load config file\n");
  }
  if (!login)
  {
    login = "jabber:tester1@ch-zlj.rd.francetelecom.fr/test";
    g_key_file_set_value (f, IM_CONFIG_GROUP, IM_CONFIG_LOGIN, login);
  }
  if (!password)
  {
    password = "123456";
    g_key_file_set_value (f, IM_CONFIG_GROUP, IM_CONFIG_PASSWD, password);
  }
  if (!server)
  {
    server = "ch-zlj.rd.francetelecom.fr";
    g_key_file_set_value (f, IM_CONFIG_GROUP, IM_CONFIG_SERVER, server);
  }
  if (!port)
  {
    port = "5222";
    g_key_file_set_value (f, IM_CONFIG_GROUP, IM_CONFIG_PORT, port);
  }
  if (!buddy)
  {
    buddy = "lzhao@ch-zlj.rd.francetelecom.fr";
    g_key_file_set_value (f, IM_CONFIG_GROUP, IM_CONFIG_BUDDY, buddy);
  }
  if (!buddy_nick)
  {
    buddy_nick = "Oops!";
    g_key_file_set_value (f, IM_CONFIG_GROUP, IM_CONFIG_BUDDY_NICK, buddy_nick);
  }
  if (!buddy_group)
  {
    buddy_group = "LiPS_TEST";
    g_key_file_set_value (f, IM_CONFIG_GROUP, IM_CONFIG_BUDDY_GROUP,
                          buddy_group);
  }
  if (!client_name)
  {
    client_name = "org.lips.enabler.imclientdemo";
    g_key_file_set_value (f, IM_CONFIG_GROUP, IM_CONFIG_CNAME, client_name);
  }
  gsize len = 0;
  gchar *data = g_key_file_to_data (f, &len, &err);

  if (data)
  {
    gsize count = 0;
    GIOChannel *io = g_io_channel_new_file (IM_CONFIG_FILE, "w+", NULL);

    g_io_channel_write_chars (io, data, len, &count, NULL);
    g_io_channel_shutdown (io, TRUE, NULL);
  }

  g_key_file_free (f);
};

static void
cb_incoming_req (sid_t sid, evt_t *evt, void *user_data)
{
  if (evt->async_data)
  {
    g_message
      ("Asynchronous event::sid=%d, rid=%d, type='%s', req_statue=%d, err_code=%d\n",
       sid, evt->async_data->rid, im_evt_type_2_str (evt->evt_type),
       evt->async_data->state, evt->async_data->err_code);
  }
}

static void
cb_incoming_logout (sid_t sid, evt_t *evt, void *user_data)
{
  g_message ("Logout event::sid=%d\n", sid);
  quit_loop (sid);
};

static void
cb_incoming_login (sid_t sid, evt_t *evt, void *user_data)
{
  gboolean islogin = FALSE;

  if (evt->async_data)
  {
    if (evt->async_data->err_code != IM_ERR_CODE_NONE)
    {
      g_message ("Login event::sid=%d rid=%d, login failed!\n", sid,
                 evt->async_data->rid);
      quit_loop (sid);
    }
    else
    {
      g_message ("Login event::sid=%d rid=%d, login succeeded!\n", sid,
                 evt->async_data->rid);
      islogin = TRUE;
    }
  }
  else
  {
    g_message ("Login event::sid=%d, login succeeded!\n", sid);
  }
  if (islogin)
  {
    run_loop (sid);
  }
};

static void
cb_incoming_presence (sid_t sid, evt_t *evt, void *user_data)
{
  im_presence_t *p = (im_presence_t *) evt->evt_data;
  char *desc, *from;
  im_presence_state_t pstate;

  im_presence_get_desc (p, &desc);
  im_presence_get_from (p, &from);
  im_presence_get_state (p, &pstate);
  if (evt->async_data)
  {
    g_message
      ("Presence event::sid=%d, rid=%d, from='%s', state='%s', desc='%s', req_statue=%d, err_code=%d\n",
       sid, evt->async_data->rid, from, im_presence_2_str (pstate), desc,
       evt->async_data->state, evt->async_data->err_code);
  }
  else
  {
    g_message ("Presence event::sid=%d, from='%s', state='%s', desc='%s'\n",
               sid, from, im_presence_2_str (pstate), desc);
  }
  g_free (desc);
  g_free (from);
}

static void
cb_incoming_msg (sid_t sid, evt_t *evt, void *user_data)
{
  im_msg_t *m = (im_msg_t *) evt->evt_data;
  char *body, *from;
  im_msg_type_t msg_type;

  im_msg_get_from (m, &from);
  im_msg_get_body (m, (void **) (&body));
  im_msg_get_type (m, &msg_type);
  g_message ("Recv msg event::sid=%d, from='%s', msg='%s', type='%s'\n", sid,
             from, body, im_msg_type_2_str (msg_type));
  g_free (from);
  g_free (body);
}

static void
cb_incoming_subscribe (sid_t sid, evt_t *evt, void *user_data)
{
  im_subs_msg_t *m = (im_subs_msg_t *) evt->evt_data;

  g_message
    ("Subscription event::sid=%d, %s would like to %s you with msg='%s'\n", sid,
     m->contactor, m->subscribe ? "subscribe" : "unsubscribe", m->msg);
  if (m->subscribe)
  {
    g_print ("Do you want to allow the subscription? [Y]: ");
    char ans = getchar ();

    if (ans == '\n' || ans == 'Y' || ans == 'y')
    {
      req_acceptance (evt->sid, m->contactor, "OK", TRUE);
    }
    else
    {
      req_acceptance (evt->sid, m->contactor, "NO", FALSE);
    }
  }
}

static void
cb_incoming_error (sid_t sid, evt_t *evt, void *user_data)
{
  im_network_error_t *e = (im_network_error_t *) evt->evt_data;

  g_message ("Network error event::sid=%d, err_type=%d, err_msg='%s'\n", sid,
             e->err_type, e->msg);
  quit_loop (sid);
}

static void
cb_incoming_accept (sid_t sid, evt_t *evt, void *user_data)
{
  im_accept_msg_t *m = (im_accept_msg_t *) evt->evt_data;

  g_message ("Acceptance event::sid=%d, %s %s your subscription: '%s'\n", sid,
             m->contactor, m->accept ? "accepted" : "refuzed", m->msg);
}

static void
cb_incoming_roster (sid_t sid, evt_t *evt, void *user_data)
{
  im_roster_t *roster = (im_roster_t *) evt->evt_data;

  g_message ("Roster synchronized!");
  unsigned int length;
  im_roster_item_t *item = NULL;
  unsigned int i = 0;

  im_roster_get_length (roster, &length);
  for (; i < length; i++)
  {
    if (im_roster_get_item (roster, i, &item) == IM_ERROR_NONE)
    {
      g_print ("\t%s<%s>: subscribe='%s', group='%s'\n", item->nickname,
               item->contactor, im_subs_state_2_str (item->subs_state),
               item->group);
    }
  }
  g_print ("\n");
}

static void
cb_imcoming_buddy_op (sid_t sid, evt_t *evt, void *user_data)
{
  im_buddy_op_t *op = (im_buddy_op_t *) evt->evt_data;
  im_roster_item_t *buddy = NULL;
  im_buddy_operation_t op_type;

  im_buddy_op_get_buddy (op, &buddy);
  im_buddy_op_get_operation (op, &op_type);
  g_message
    ("Buddy event::sid=%d, %s<%s>, group='%s', subscription='%s', operation='%s'\n",
     sid, buddy->nickname ? buddy->nickname : "(none)",
     buddy->contactor ? buddy->contactor : "(none)",
     buddy->group ? buddy->group : "(none)",
     im_subs_state_2_str (buddy->subs_state),
     im_buddy_operation_2_str (op_type));
}

static im_err_t
reg_unsolicited_evt (im_sid_t sid, cb_t cb, evt_type_t evt_type)
{
  evt_handler_t *handler = g_new0 (evt_handler_t, 1);

  if (!handler)
    return IM_ERROR_NOMEM;
  handler->cb = cb;
  handler->user_data = NULL;
  return im_evt_register (sid, evt_type, handler);
}

static bool
req_login (im_sid_t sid)
{
  bool rt = TRUE;
  evt_handler_t *handler = g_new0 (evt_handler_t, 1);

  handler->cb = cb_incoming_login;
  handler->user_data = NULL;
  im_param_login_t *login_param = g_new0 (im_param_login_t, 1);
  im_rid_t rid;

  im_uri_new (login, IM_PROTOCOL_XMPP, &(login_param->user));
  login_param->passwd = g_strdup (password);
  im_err_t err = im_login (sid, handler, &rid, login_param);

  if (err == IM_ERROR_NONE)
  {
    g_message ("req id = %d::Start login\n", rid);
  }
  else if (err == IM_ERROR_LOGINED)
  {
    g_message ("req id = %d::Start login\n", rid);
    run_loop (sid);
  }
  else
  {
    g_message ("req id = %d::Start login failed!\n", rid);
    rt = FALSE;
  }
  g_free (login_param->passwd);
  im_uri_free (login_param->user);
  g_free (login_param);
  return rt;
};

static void
req_logout (im_sid_t sid)
{
  evt_handler_t *handler = g_new0 (evt_handler_t, 1);

  handler->cb = cb_incoming_logout;
  handler->user_data = NULL;
  im_rid_t rid;

  im_logout (sid, handler, &rid);
  g_message ("req id = %d::Logout\n", rid);
};

static void
req_subscribe (im_sid_t sid, char *to, char *msg, char *group, char *nick,
               bool subscribe)
{
  evt_handler_t *handler = g_new0 (evt_handler_t, 1);

  handler->cb = cb_incoming_req;
  handler->user_data = NULL;
  im_rid_t rid;
  im_subs_msg_t *subsmsg = g_new0 (im_subs_msg_t, 1);

  subsmsg->contactor = g_strdup (to);
  subsmsg->msg = g_strdup (msg);
  subsmsg->subscribe = subscribe;
  subsmsg->group = g_strdup (group);
  subsmsg->nick = g_strdup (nick);
  im_subscribe (sid, handler, &rid, subsmsg);
  g_message ("req id = %d::%s to %s<%s> group:%s msg:'%s'\n", rid,
             subscribe ? "Subscribe" : "Unsubscribe", nick ? nick : "(null)",
             to ? to : "(null)", group ? group : "(null)",
             msg ? msg : "(null)");
  im_subs_msg_free (subsmsg);
};

static void
req_change_presence (im_sid_t sid, char *to, char *desc,
                     im_presence_state_t state)
{
  evt_handler_t *handler = g_new0 (evt_handler_t, 1);

  handler->cb = cb_incoming_presence;
  handler->user_data = NULL;
  im_presence_t *presence;
  im_rid_t rid;

  im_presence_new (IM_PROTOCOL_XMPP, to, &presence);
  im_presence_set_state (presence, state);
  im_presence_set_desc (presence, desc);
  im_change_presence (sid, handler, &rid, presence);
  g_message ("req id = %d::change presence to %s<%s>\n", rid,
             im_presence_2_str (state), desc ? desc : "(null)");
  im_presence_free (presence);
};

static void
req_sendmsg (im_sid_t sid, char *uri, char *msg)
{
  evt_handler_t *handler = g_new0 (evt_handler_t, 1);

  handler->cb = cb_incoming_req;
  handler->user_data = NULL;
  im_msg_t *smsg;
  im_rid_t rid;

  im_msg_new (sid, IM_MSG_TYPE_CHAT, IM_MSG_CTYPE_TEXT, &smsg);
  im_msg_set_id (smsg, 0);
  im_msg_set_recipient (smsg, uri);
  im_msg_set_body (smsg, msg, strlen (msg) + 1, NULL);
  im_send_message (sid, handler, &rid, smsg);
  g_message ("req id = %d::send msg to %s: '%s'\n", rid,
             uri ? uri : "(null)", msg ? msg : "(null)");
  im_msg_free (smsg);
};

static void
req_acceptance (im_sid_t sid, char *to, char *msg, bool accept)
{
  evt_handler_t *handler = g_new0 (evt_handler_t, 1);

  handler->cb = cb_incoming_req;
  handler->user_data = NULL;
  im_rid_t rid;
  im_accept_msg_t *acceptmsg = g_new0 (im_accept_msg_t, 1);

  acceptmsg->contactor = g_strdup (to);
  acceptmsg->accept = accept;
  acceptmsg->msg = g_strdup (msg);
  im_acceptance (sid, handler, &rid, acceptmsg);
  g_message ("req id = %d::%s the subscription from %s: '%s'\n", rid,
             accept ? "Accept" : "Refuse", to ? to : "(null)",
             msg ? msg : "(null)");
  im_accept_msg_free (acceptmsg);
};

static void
req_get_roster (im_sid_t sid)
{
  evt_handler_t *handler = g_new0 (evt_handler_t, 1);

  handler->cb = cb_incoming_req;
  handler->user_data = NULL;
  im_rid_t rid;

  im_get_roster (sid, handler, &rid);
  g_message ("req id = %d::Get roster\n", rid);
};

static void
req_buddy_op (im_sid_t sid, char *uri, char *nick, char *group,
              im_buddy_operation_t op_type)
{
  evt_handler_t *handler = g_new0 (evt_handler_t, 1);

  handler->cb = cb_incoming_req;
  handler->user_data = NULL;
  im_rid_t rid;
  im_roster_item_t *buddy = (im_roster_item_t *) g_new0 (im_roster_item_t, 1);

  buddy->contactor = g_strdup (uri);
  buddy->nickname = g_strdup (nick);
  buddy->group = g_strdup (group);
  im_buddy_op_t *op = NULL;

  im_buddy_op_new (buddy, op_type, &op);
  im_buddy_op (sid, handler, &rid, op);
  g_message ("req id = %d::%s buddy %s<%s> in group %s \n", rid,
             im_buddy_operation_2_str (op_type), nick ? nick : "(null)",
             uri ? uri : "(null)", group ? group : "(null)");
  im_buddy_op_free_all (op);
};

static void
req_buddies_synch (im_sid_t sid)
{
  evt_handler_t *handler = g_new0 (evt_handler_t, 1);

  handler->cb = cb_incoming_req;
  handler->user_data = NULL;
  im_rid_t rid;

  im_buddies_synch (sid, handler, &rid);
  g_message ("req id = %d::synchronize \n", rid);
};

static void
req_presence_probe (im_sid_t sid, char *uri)
{
  evt_handler_t *handler = g_new0 (evt_handler_t, 1);

  handler->cb = cb_incoming_presence;
  handler->user_data = NULL;
  im_rid_t rid;

  im_presence_probe (sid, handler, &rid, uri);
  g_message ("req id = %d::presence probe \n", rid);
};

static void
reg (sid_t sid)
{
  reg_unsolicited_evt (sid, cb_incoming_logout, IM_EVT_LOGOUT);
  reg_unsolicited_evt (sid, cb_incoming_presence, IM_EVT_PRESENCE);
  reg_unsolicited_evt (sid, cb_incoming_msg, IM_EVT_MESSAGE);
  reg_unsolicited_evt (sid, cb_incoming_subscribe, IM_EVT_SUBSCRIBE);
  reg_unsolicited_evt (sid, cb_incoming_accept, IM_EVT_ACCEPTANCE);
  reg_unsolicited_evt (sid, cb_incoming_error, IM_EVT_ERROR);
  reg_unsolicited_evt (sid, cb_incoming_roster, IM_EVT_ROSTER);
  reg_unsolicited_evt (sid, cb_imcoming_buddy_op, IM_EVT_BUDDY);
}

static void
unreg (sid_t sid)
{
  im_evt_unregister (sid, IM_EVT_MESSAGE);
  im_evt_unregister (sid, IM_EVT_PRESENCE);
  im_evt_unregister (sid, IM_EVT_LOGOUT);
  im_evt_unregister (sid, IM_EVT_SUBSCRIBE);
  im_evt_unregister (sid, IM_EVT_ACCEPTANCE);
  im_evt_unregister (sid, IM_EVT_ERROR);
  im_evt_unregister (sid, IM_EVT_ROSTER);
  im_evt_unregister (sid, IM_EVT_BUDDY);
}

gboolean
run_loop (im_sid_t sid)
{
  req_change_presence (sid, NULL, "I am available", IM_PRESENCE_AVAILABLE);
  req_buddy_op (sid, buddy, NULL, NULL, IM_BUDDY_REMOVE);
  req_buddy_op (sid, buddy, buddy_nick, buddy_group, IM_BUDDY_ADD);
  req_subscribe (sid, buddy, "Hello!", "zzz", "ttt", TRUE);
  req_buddy_op (sid, buddy, buddy_nick, buddy_group, IM_BUDDY_UPDATE);
  req_buddies_synch (sid);
  req_sendmsg (sid, buddy, "Hello, LiPS!");
  req_presence_probe (sid, buddy);
  req_change_presence (sid, NULL, "I am busy now!", IM_PRESENCE_AWAY);
  req_get_roster (sid);
  req_buddy_op (sid, "tester3@ch-zlj.rd.francetelecom.fr", buddy_nick,
                buddy_group, IM_BUDDY_ADD);
  req_buddy_op (sid, "tester3@ch-zlj.rd.francetelecom.fr", "buddy_nick",
                "buddy_group", IM_BUDDY_UPDATE);
  req_buddy_op (sid, "tester4@ch-zlj.rd.francetelecom.fr", buddy_nick,
                buddy_group, IM_BUDDY_ADD);
  req_buddy_op (sid, "tester4@ch-zlj.rd.francetelecom.fr", "buddy_nick",
                "buddy_group", IM_BUDDY_UPDATE);
  req_buddies_synch (sid);
  req_get_roster (sid);
  req_buddy_op (sid, "tester3@ch-zlj.rd.francetelecom.fr", NULL, NULL,
                IM_BUDDY_REMOVE);
  req_buddy_op (sid, "tester4@ch-zlj.rd.francetelecom.fr", NULL, NULL,
                IM_BUDDY_REMOVE);
  req_buddies_synch (sid);
  req_get_roster (sid);
  //g_print("Press any key to logout: ");
  //getchar();
  //req_subscribe(sid, "tester3@ch-zlj.rd.francetelecom.fr", "Hello!", "", "", FALSE);
  return FALSE;
}

int
main (int argc, char *args[])
{
  load_config ();
  im_sid_t sid = 0;

  if (im_init () != IM_ERROR_NONE)
    return 1;
  if (im_open_session (&sid, IM_PROTOCOL_XMPP, "1.0", server,
                       port, login) != IM_ERROR_NONE)
  {
    g_print ("failed to open session! \n");
    return -1;
  }
  my_sid = sid;
  reg (sid);
  if (!req_login (sid))
    return -1;
  //req_login(sid);

  loop = g_main_loop_new (NULL, FALSE);

  //  g_timeout_add (1000, (GSourceFunc)run_loop, sid);
  if ((dft_sigint_hdl = signal (SIGINT, sigint_hdl)) == SIG_IGN)
    signal (SIGINT, SIG_IGN);
  g_main_loop_run (loop);

  im_finalize ();
  return 0;
}

gboolean
quit_loop (im_sid_t sid)
{
  unreg (sid);
  im_close_session (sid);
  g_main_loop_quit (loop);
  return FALSE;
};

void
sigint_hdl (int signum)
{
  signal (SIGINT, dft_sigint_hdl);
  req_logout (my_sid);
};

